"""
This is a automatically generated file from Psi4's density functional metadata.
Psi4 Version: 1.4a2.dev215

File Authors: QCElemental Authors
"""


data_blob = {
    "version": "1.4a2.dev215",
    "functionals": {
        "teter93": {
            "ansatz": 0,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "zlp": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "ksdt": {
            "ansatz": 0,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hcth407p": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hcthp76": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hcthp14": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b97-gga1": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "kt2": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "th1": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "th2": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "th3": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "th4": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hcth": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hcth93": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hcth120": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hcth147": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hcth407": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "edf1": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "xlyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbe1w": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpwlyp1w": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbelyp1w": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mohlyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mohlyp2": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "th-fl": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "th-fc": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "th-fcfo": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "th-fco": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "vv10": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": True,
            "deriv": 1,
        },
        "b97-1p": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b3pw91": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b3lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b3p86": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "o3lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpw1k": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbeh": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbe0": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b97-0": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b97-1": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b97-2": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "x3lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b1wc": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b97-k": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b97-3": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpw3pw": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b1lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b1pw91": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpw1pw91": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpw1pw": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpw3lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sb98-1a": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sb98-1b": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sb98-1c": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sb98-2a": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sb98-2b": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sb98-2c": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hse03": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "hse06": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "hjs-pbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "hjs-pbesol": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "hjs-pbe-sol": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "hjs-b88": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "hjs-b97x": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "cam-b3lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "tuned-cam-b3lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "bhandh": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "bhhlyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "bhandhlyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mb3lyp-rc04": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpwlyp1m": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "revb3lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbe0-13": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b3lyps": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "wb97": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "wb97x": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "lrc-wpbeh": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "wb97xv": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": True,
            "deriv": 1,
        },
        "wb97x-v": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": True,
            "deriv": 1,
        },
        "lc-vv10": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": True,
            "deriv": 1,
        },
        "hpbeint": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "lrc-wpbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "b3lyp5": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "edf2": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "cap0": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b1b95": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b88b95": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b86b95": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pw86b95": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "bb1k": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpw1b95": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpwb1k": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "x1b95": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "xb1k": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pw6b95": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pwb6k": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "tpss0": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "tpssh": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "revtpssh": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "wb97mv": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": True,
            "deriv": 1,
        },
        "wb97m-v": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": True,
            "deriv": 1,
        },
        "tpsslyp1w": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b97m-v": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": True,
            "deriv": 1,
        },
        "b5050lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "svwn": {
            "ansatz": 0,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "blyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "bop": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b86bpbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pw86pbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "rpbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "revpbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "n12": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pw91": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpwpw": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "bp86": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "ft97": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sogga": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sogga11": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "gam": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "oppbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbeop": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "op-pbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m06l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m06-l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "revm06l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "revm06-l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m11l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m11-l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mn12l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mn12-l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mn15l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mn15-l": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga-ms0": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga_ms0": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga-ms1": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga_ms1": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga-ms2": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga_ms2": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga-mvs": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga_mvs": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "tpss": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "revtpss": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pkzb": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "vsxc": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "scan": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "revscan": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "lc-wpbe0": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "wpbe0": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "lc-wpbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "lcwpbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "wpbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": False,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "scf": {
            "ansatz": 0,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hf": {
            "ansatz": 0,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hf+d": {
            "ansatz": 0,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hf-3c": {
            "ansatz": 0,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "hf3c": {
            "ansatz": 0,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbeh-3c": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbeh3c": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sogga11x": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "sogga11-x": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mn12sx": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "mn12-sx": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "mn15": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "n12sx": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "n12-sx": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "pbe50": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "revpbe0": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpw1lyp": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mpw1pbe": {
            "ansatz": 1,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga-ms2h": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "ms2h": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga_ms2h": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga-mvsh": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mvsh": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mgga_mvsh": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "dldf": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "dldf+d": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m05": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m052x": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m05-2x": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m06": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m062x": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m06-2x": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m06hf": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m06-hf": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m08hx": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m08-hx": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m08so": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m08-so": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "m11": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": True,
            "nlc": False,
            "deriv": 1,
        },
        "scan0": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "revscan0": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "bmk": {
            "ansatz": 2,
            "c_hybrid": False,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mp2mp2": {
            "ansatz": 0,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "mp2d": {
            "ansatz": 0,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b2plyp": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "dsd-blyp": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "core-dsd-blyp": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbe02": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbe0-2": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbe0dh": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pbe0-dh": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "dsdpbep86": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "dsd-pbep86": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "dsdpbepbe": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "dsd-pbepbe": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "b2gpplyp": {
            "ansatz": 1,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "pwpb95": {
            "ansatz": 2,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "ptpss": {
            "ansatz": 2,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
        "dsd-pbeb95": {
            "ansatz": 2,
            "c_hybrid": True,
            "x_hybrid": True,
            "c_lrc": False,
            "x_lrc": False,
            "nlc": False,
            "deriv": 1,
        },
    },
    "empirical_dispersion_suffixes": [
        "-chg",
        "-d2",
        "-d3mbj",
        "+d09",
        "-d3zero",
        "+d10",
        "-atmgr",
        "-d10",
        "-das2010",
        "-d09",
        "-d3mzero",
        "-d",
        "-dmp2",
        "-d3m(bj)",
        "-d3(bj)",
        "-nl",
        "-d3bj",
        "-d3m",
        "-d3",
        "-das2009",
    ],
}
