"""
Processor data from multiple sources and vendors.

File Authors: QCElemental Authors
"""

data_rows = [
    [
        3400000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        460,
        3,
        3,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 460 @ 3.4 GHz",
    ],
    [
        3200000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        555,
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 555 @ 3.2 GHz",
    ],
    [
        3400000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        565,
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 565 @ 3.4 GHz",
    ],
    [
        4000000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        570,
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 570 @ 4.0 GHz",
    ],
    [
        2600000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        631,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 631 @ 2.6 GHz",
    ],
    [
        2700000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        638,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 638 @ 2.7 GHz",
    ],
    [
        2800000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        641,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 641 @ 2.8 GHz",
    ],
    [
        3200000000,
        3700000000.0,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        740,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 740 @ 3.2 GHz",
    ],
    [
        3400000000,
        4000000000.0,
        "AMD Athlon\u2122 Processors",
        2,
        None,
        None,
        750,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 750 @ 3.4 GHz",
    ],
    [
        3200000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        840,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 840 @ 3.2 GHz",
    ],
    [
        3300000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        850,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 850 @ 3.3 GHz",
    ],
    [
        3400000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        965,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 965 @ 3.4 GHz",
    ],
    [
        3600000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        975,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 975 @ 3.6 GHz",
    ],
    [
        3700000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        980,
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 980 @ 3.7 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        3280,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 3280 @ 2.4 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        3365,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 3365 @ 2.3 GHz",
    ],
    [
        2600000000,
        3600000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        3380,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 3380 @ 2.6 GHz",
    ],
    [
        2700000000,
        3100000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4226,
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4226 @ 2.7 GHz",
    ],
    [
        3100000000,
        3500000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4234,
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4234 @ 3.1 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4238,
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4238 @ 3.3 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4240,
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4240 @ 3.4 GHz",
    ],
    [
        2800000000,
        2200000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4280,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4280 @ 2.8 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4284,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4284 @ 3.0 GHz",
    ],
    [
        3100000000,
        3500000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4334,
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4334 @ 3.1 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4340,
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4340 @ 3.5 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4365,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4365 @ 2.0 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        4386,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4386 @ 3.1 GHz",
    ],
    [
        2600000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6140,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6140 @ 2.6 GHz",
    ],
    [
        2300000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6176,
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6176 @ 2.3 GHz",
    ],
    [
        3300000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6204,
        4,
        4,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6204 @ 3.3 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6212,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6212 @ 2.6 GHz",
    ],
    [
        3000000000,
        3600000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6220,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6220 @ 3.0 GHz",
    ],
    [
        2400000000,
        3000000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6234,
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6234 @ 2.4 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6238,
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6238 @ 2.6 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6272,
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6272 @ 2.1 GHz",
    ],
    [
        2200000000,
        3100000000.0,
        "AMD Opteron\u2122",
        1,
        2011.0,
        None,
        6274,
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6274 @ 2.2 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "AMD Opteron\u2122",
        1,
        2011.0,
        None,
        6276,
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6276 @ 2.3 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6278,
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6278 @ 2.4 GHz",
    ],
    [
        3500000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6308,
        4,
        4,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6308 @ 3.5 GHz",
    ],
    [
        2800000000,
        3300000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6320,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6320 @ 2.8 GHz",
    ],
    [
        3200000000,
        3800000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6328,
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6328 @ 3.2 GHz",
    ],
    [
        2600000000,
        2000000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6344,
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6344 @ 2.6 GHz",
    ],
    [
        2800000000,
        3400000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6348,
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6348 @ 2.8 GHz",
    ],
    [
        2300000000,
        2000000000.0,
        "AMD Opteron\u2122",
        1,
        2012.0,
        None,
        6376,
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6376 @ 2.3 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6378,
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6378 @ 2.4 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        6380,
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6380 @ 2.5 GHz",
    ],
    [
        3500000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "1075T",
        6,
        6,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 1075T @ 3.5 GHz",
    ],
    [
        3100000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "255e",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 255e @ 3.1 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "3260 HE",
        4,
        4,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 3260 HE @ 2.7 GHz",
    ],
    [
        1900000000,
        2500000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "3320 EE",
        4,
        4,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 3320 EE @ 1.9 GHz",
    ],
    [
        2800000000,
        3800000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "3350 HE",
        4,
        4,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 3350 HE @ 2.8 GHz",
    ],
    [
        2800000000,
        3600000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "4228 HE",
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4228 HE @ 2.8 GHz",
    ],
    [
        2900000000,
        3700000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "4230 HE",
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4230 HE @ 2.9 GHz",
    ],
    [
        1600000000,
        2800000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "4256 EE",
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4256 EE @ 1.6 GHz",
    ],
    [
        2700000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "425e",
        3,
        3,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 425e @ 2.7 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "4274 HE",
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4274 HE @ 2.5 GHz",
    ],
    [
        2600000000,
        3600000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "4276 HE",
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4276 HE @ 2.6 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "4310 EE",
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4310 EE @ 2.2 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "4332 HE",
        6,
        6,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4332 HE @ 3.0 GHz",
    ],
    [
        2600000000,
        3600000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "4376 HE",
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 4376 HE @ 2.6 GHz",
    ],
    [
        2200000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6132 HE",
        8,
        8,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6132 HE @ 2.2 GHz",
    ],
    [
        1800000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6166 HE",
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6166 HE @ 1.8 GHz",
    ],
    [
        1900000000,
        None,
        "AMD Opteron\u2122",
        1,
        2010.0,
        None,
        "6168",
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6168 @ 1.9 GHz",
    ],
    [
        2200000000,
        None,
        "AMD Opteron\u2122",
        1,
        2010.0,
        None,
        "6174",
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6174 @ 2.2 GHz",
    ],
    [
        2500000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6180 SE",
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6180 SE @ 2.5 GHz",
    ],
    [
        2700000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "620e",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 620e @ 2.7 GHz",
    ],
    [
        1600000000,
        2900000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6262 HE",
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6262 HE @ 1.6 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6282 SE",
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6282 SE @ 2.6 GHz",
    ],
    [
        2700000000,
        3400000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6284 SE",
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6284 SE @ 2.7 GHz",
    ],
    [
        2300000000,
        2800000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6338P",
        12,
        12,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6338P @ 2.3 GHz",
    ],
    [
        1800000000,
        3100000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6366 HE",
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6366 HE @ 1.8 GHz",
    ],
    [
        2000000000,
        2500000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6370P",
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6370P @ 2.0 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "6386 SE",
        16,
        16,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 6386 SE @ 2.8 GHz",
    ],
    [
        1800000000,
        3200000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "6th Gen A10-8700P APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 6th Gen A10-8700P APU @ 1.8 GHz",
    ],
    [
        1600000000,
        3000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "6th Gen A8-8600P APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 6th Gen A8-8600P APU @ 1.6 GHz",
    ],
    [
        1800000000,
        3200000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A10-8700B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A10-8700B APU @ 1.8 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "6th Gen AMD PRO A10-8730B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A10-8730B APU @ 2.4 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A10-8750B APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A10-8750B APU @ 3.6 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "6th Gen AMD PRO A10-8770 APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A10-8770 APU @ 3.5 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "6th Gen AMD PRO A10-8770E APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A10-8770E APU @ 2.8 GHz",
    ],
    [
        3900000000,
        4099999999.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A10-8850B APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A10-8850B APU @ 3.9 GHz",
    ],
    [
        2100000000,
        3400000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A12-8800B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A12-8800B APU @ 2.1 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "6th Gen AMD PRO A12-8830B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A12-8830B APU @ 2.5 GHz",
    ],
    [
        3700000000,
        4200000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "6th Gen AMD PRO A12-8870 APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A12-8870 APU @ 3.7 GHz",
    ],
    [
        2900000000,
        3800000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A12-8870E APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A12-8870E APU @ 2.9 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A4-8350B APU",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A4-8350B APU @ 3.5 GHz",
    ],
    [
        1600000000,
        3000000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A6-8500B APU",
        2,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A6-8500B APU @ 1.6 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "6th Gen AMD PRO A6-8530B APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A6-8530B APU @ 2.3 GHz",
    ],
    [
        3700000000,
        4000000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A6-8550B APU",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A6-8550B APU @ 3.7 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A6-8570 APU",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A6-8570 APU @ 3.5 GHz",
    ],
    [
        3000000000,
        3400000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A6-8570E APU",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A6-8570E APU @ 3.0 GHz",
    ],
    [
        1600000000,
        3000000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A8-8600B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A8-8600B APU @ 1.6 GHz",
    ],
    [
        3200000000,
        3900000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "6th Gen AMD PRO A8-8650B APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 6th Gen AMD PRO A8-8650B APU @ 3.2 GHz",
    ],
    [
        2100000000,
        3400000000.0,
        "AMD FX-Series Processors",
        2,
        None,
        None,
        "6th Gen FX-8800P APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD FX-Series Processors 6th Gen FX-8800P APU @ 2.1 GHz",
    ],
    [
        3400000000,
        4000000000.0,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "750K",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 750K @ 3.4 GHz",
    ],
    [
        3800000000,
        4099999999.0,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "760K",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 760K @ 3.8 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "AMD A-Series Processors",
        0,
        16.0,
        None,
        "7th Gen A10-9600P APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A10-9600P APU @ 2.4 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "AMD A-Series Processors",
        0,
        16.0,
        None,
        "7th Gen A10-9630P APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A10-9630P APU @ 2.6 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD A-Series Processors",
        0,
        2016.0,
        None,
        "7th Gen A10-9700 APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A10-9700 APU @ 3.5 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "AMD A-Series Processors",
        0,
        2016.0,
        None,
        "7th Gen A10-9700E APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A10-9700E APU @ 3.0 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "AMD A-Series Processors",
        0,
        16.0,
        None,
        "7th Gen A12-9700P APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A12-9700P APU @ 2.5 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "AMD A-Series Processors",
        0,
        16.0,
        None,
        "7th Gen A12-9730P APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A12-9730P APU @ 2.8 GHz",
    ],
    [
        3800000000,
        4200000000.0,
        "AMD A-Series Processors",
        0,
        2016.0,
        None,
        "7th Gen A12-9800 APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A12-9800 APU @ 3.8 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "AMD A-Series Processors",
        0,
        2016.0,
        None,
        "7th Gen A12-9800E APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A12-9800E APU @ 3.1 GHz",
    ],
    [
        2200000000,
        2500000000.0,
        "AMD A-Series Processors",
        0,
        17.0,
        None,
        "7th Gen A4-9120 APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A4-9120 APU @ 2.2 GHz",
    ],
    [
        1600000000,
        2400000000.0,
        "AMD A-Series Processors",
        0,
        None,
        None,
        "7th Gen A4-9120C APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A4-9120C APU @ 1.6 GHz",
    ],
    [
        2300000000,
        2600000000.0,
        "AMD A-Series Processors",
        0,
        18.0,
        None,
        "7th Gen A4-9125 APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A4-9125 APU @ 2.3 GHz",
    ],
    [
        2400000000,
        2800000000.0,
        "AMD A-Series Processors",
        0,
        16.0,
        None,
        "7th Gen A6-9210 APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A6-9210 APU @ 2.4 GHz",
    ],
    [
        2500000000,
        2900000000.0,
        "AMD A-Series Processors",
        0,
        17.0,
        None,
        "7th Gen A6-9220 APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A6-9220 APU @ 2.5 GHz",
    ],
    [
        1800000000,
        2700000000.0,
        "AMD A-Series Processors",
        0,
        None,
        None,
        "7th Gen A6-9220C APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A6-9220C APU @ 1.8 GHz",
    ],
    [
        2600000000,
        3100000000.0,
        "AMD A-Series Processors",
        0,
        18.0,
        None,
        "7th Gen A6-9225 APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A6-9225 APU @ 2.6 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD A-Series Processors",
        0,
        2016.0,
        None,
        "7th Gen A6-9500 APU",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A6-9500 APU @ 3.5 GHz",
    ],
    [
        3000000000,
        3400000000.0,
        "AMD A-Series Processors",
        0,
        2016.0,
        None,
        "7th Gen A6-9500E APU",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A6-9500E APU @ 3.0 GHz",
    ],
    [
        3800000000,
        4000000000.0,
        "AMD A-Series Processors",
        0,
        2017.0,
        None,
        "7th Gen A6-9550 APU",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A6-9550 APU @ 3.8 GHz",
    ],
    [
        3100000000,
        3400000000.0,
        "AMD A-Series Processors",
        0,
        2016.0,
        None,
        "7th Gen A8-9600 APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A8-9600 APU @ 3.1 GHz",
    ],
    [
        2900000000,
        3500000000.0,
        "AMD A-Series Processors",
        0,
        16.0,
        None,
        "7th Gen A9-9410 APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A9-9410 APU @ 2.9 GHz",
    ],
    [
        3000000000,
        3600000000.0,
        "AMD A-Series Processors",
        0,
        17.0,
        None,
        "7th Gen A9-9420 APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A9-9420 APU @ 3.0 GHz",
    ],
    [
        3100000000,
        3700000000.0,
        "AMD A-Series Processors",
        0,
        18.0,
        None,
        "7th Gen A9-9425 APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors 7th Gen A9-9425 APU @ 3.1 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "AMD Athlon\u2122 Processors",
        1,
        2017.0,
        None,
        "7th Gen AMD Athlon\u2122 X4 940",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  7th Gen AMD Athlon\u2122 X4 940 @ 3.2 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD Athlon\u2122 Processors",
        1,
        2017.0,
        None,
        "7th Gen AMD Athlon\u2122 X4 950",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  7th Gen AMD Athlon\u2122 X4 950 @ 3.5 GHz",
    ],
    [
        3800000000,
        4000000000.0,
        "AMD Athlon\u2122 Processors",
        1,
        2017.0,
        None,
        "7th Gen AMD Athlon\u2122 X4 970",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  7th Gen AMD Athlon\u2122 X4 970 @ 3.8 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A10-9700 APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A10-9700 APU @ 3.5 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A10-9700B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A10-9700B APU @ 2.5 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A10-9700E APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A10-9700E APU @ 3.0 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A10-9730B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A10-9730B APU @ 2.8 GHz",
    ],
    [
        3800000000,
        4200000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A12-9800 APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A12-9800 APU @ 3.8 GHz",
    ],
    [
        2700000000,
        3600000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A12-9800B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A12-9800B APU @ 2.7 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A12-9800E APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A12-9800E APU @ 3.1 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A12-9830B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A12-9830B APU @ 3.0 GHz",
    ],
    [
        2500000000,
        2900000000.0,
        "AMD PRO A-Series Processors",
        0,
        2018.0,
        None,
        "7th Gen AMD PRO A4-4350B APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A4-4350B APU @ 2.5 GHz",
    ],
    [
        3000000000,
        3600000000.0,
        "AMD PRO A-Series Processors",
        0,
        2018.0,
        None,
        "7th Gen AMD PRO A6-7350B APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A6-7350B APU @ 3.0 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A6-9500 APU",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A6-9500 APU @ 3.5 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A6-9500B APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A6-9500B APU @ 2.3 GHz",
    ],
    [
        3000000000,
        3400000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A6-9500E APU",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A6-9500E APU @ 3.0 GHz",
    ],
    [
        3100000000,
        3400000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A8-9600 APU",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A8-9600 APU @ 3.1 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A8-9600B APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A8-9600B APU @ 2.4 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "7th Gen AMD PRO A8-9630B",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors 7th Gen AMD PRO A8-9630B @ 2.6 GHz",
    ],
    [
        200000000,
        2200000000.0,
        "AMD E-Series Processors",
        0,
        17.0,
        None,
        "7th Gen E2-9010 APU",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD E-Series Processors 7th Gen E2-9010 APU @ 0.2 GHz",
    ],
    [
        2700000000,
        3600000000.0,
        "AMD FX-Series Processors",
        1,
        16.0,
        None,
        "7th Gen FX\u2122 9800P APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD FX-Series Processors 7th Gen FX\u2122 9800P APU @ 2.7 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "AMD FX-Series Processors",
        1,
        16.0,
        None,
        "7th Gen FX\u2122 9830P APU",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD FX-Series Processors 7th Gen FX\u2122 9830P APU @ 3.0 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD Athlon\u2122 Processors",
        2,
        None,
        None,
        "845 with Near Silent Thermal Solution",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 845 with Near Silent Thermal Solution @ 3.5 GHz",
    ],
    [
        3700000000,
        4000000000.0,
        "AMD Athlon\u2122 Processors",
        2,
        None,
        None,
        "860K",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 860K @ 3.7 GHz",
    ],
    [
        3700000000,
        4000000000.0,
        "AMD Athlon\u2122 Processors",
        2,
        None,
        None,
        "860K with Near Silent Thermal Solution",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 860K with Near Silent Thermal Solution @ 3.7 GHz",
    ],
    [
        3900000000,
        4099999999.0,
        "AMD Athlon\u2122 Processors",
        2,
        None,
        None,
        "870K with Near Silent Thermal Solution",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 870K with Near Silent Thermal Solution @ 3.9 GHz",
    ],
    [
        4000000000,
        4200000000.0,
        "AMD Athlon\u2122 Processors",
        2,
        None,
        None,
        "880K with Near Silent Thermal Solution",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors 880K with Near Silent Thermal Solution @ 4.0 GHz",
    ],
    [
        3000000000,
        3400000000.0,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "960T",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 960T @ 3.0 GHz",
    ],
    [
        2200000000,
        None,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10 Micro-6700T with Radeon\u2122 R6 Graphics",
        4,
        4,
        "Laptop, Tablet",
        "cpu",
        "amd",
        " AMD A-Series Processors A10 Micro-6700T with Radeon\u2122 R6 Graphics @ 2.2 GHz",
    ],
    [
        2100000000,
        3300000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A10 PRO-7350B with Radeon\u2122 R6 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A10 PRO-7350B with Radeon\u2122 R6 Graphics @ 2.1 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A10 PRO-7800B with Radeon\u2122 R7 Graphics",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A10 PRO-7800B with Radeon\u2122 R7 Graphics @ 3.5 GHz",
    ],
    [
        3700000000,
        4000000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A10 PRO-7850B with Radeon\u2122 R7 Graphics",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A10 PRO-7850B with Radeon\u2122 R7 Graphics @ 3.7 GHz",
    ],
    [
        3700000000,
        4300000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-6700 with Radeon\u2122 HD 8670D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-6700 with Radeon\u2122 HD 8670D @ 3.7 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-6700T with Radeon\u2122 HD 8650D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-6700T with Radeon\u2122 HD 8650D @ 2.5 GHz",
    ],
    [
        4000000000,
        4300000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-6790B with Radeon\u2122 HD 8670D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-6790B with Radeon\u2122 HD 8670D @ 4.0 GHz",
    ],
    [
        4000000000,
        4300000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-6790K with Radeon\u2122 HD 8670D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-6790K with Radeon\u2122 HD 8670D @ 4.0 GHz",
    ],
    [
        4099999999,
        4400000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-6800B with Radeon\u2122 HD 8670D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-6800B with Radeon\u2122 HD 8670D @ 4.1 GHz",
    ],
    [
        4099999999,
        4400000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-6800K with Radeon\u2122 HD 8670D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-6800K with Radeon\u2122 HD 8670D @ 4.1 GHz",
    ],
    [
        1900000000,
        3200000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7300 with Radeon\u2122 R6 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7300 with Radeon\u2122 R6 Graphics @ 1.9 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7400P with Radeon\u2122 R6 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7400P with Radeon\u2122 R6 Graphics @ 2.5 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7700K with Radeon\u2122 R7 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7700K with Radeon\u2122 R7 Series @ 3.4 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7800 with Radeon\u2122 R7 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7800 with Radeon\u2122 R7 Series @ 3.5 GHz",
    ],
    [
        3700000000,
        4000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7850K with Radeon\u2122 R7 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7850K with Radeon\u2122 R7 Series @ 3.7 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7860K with Radeon\u2122 R7 Graphics and Near Silent Thermal Solution",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7860K with Radeon\u2122 R7 Graphics and Near Silent Thermal Solution @ 3.6 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7860K with Radeon\u2122 R7 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7860K with Radeon\u2122 R7 Series @ 3.6 GHz",
    ],
    [
        3900000000,
        4099999999.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7870K with Radeon\u2122 R7 Graphics and Near Silent Thermal Solution",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7870K with Radeon\u2122 R7 Graphics and Near Silent Thermal Solution @ 3.9 GHz",
    ],
    [
        3900000000,
        4099999999.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7870K with Radeon\u2122 R7 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7870K with Radeon\u2122 R7 Series @ 3.9 GHz",
    ],
    [
        4099999999,
        4300000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-7890K with Radeon\u2122 R7 Graphics and Wraith cooler",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-7890K with Radeon\u2122 R7 Graphics and Wraith cooler @ 4.1 GHz",
    ],
    [
        1800000000,
        3200000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A10-8700P with Radeon\u2122 R6 Graphics",
        4,
        4,
        "All-in-One",
        "cpu",
        "amd",
        " AMD A-Series Processors A10-8700P with Radeon\u2122 R6 Graphics @ 1.8 GHz",
    ],
    [
        1600000000,
        None,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A4 Micro-6400T with Radeon\u2122 R3 Graphics",
        4,
        4,
        "Laptop, Tablet",
        "cpu",
        "amd",
        " AMD A-Series Processors A4 Micro-6400T with Radeon\u2122 R3 Graphics @ 1.6 GHz",
    ],
    [
        2200000000,
        None,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A4 PRO-3340B with Radeon\u2122 HD 8240 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A4 PRO-3340B with Radeon\u2122 HD 8240 Graphics @ 2.2 GHz",
    ],
    [
        2000000000,
        2400000000.0,
        "AMD PRO A-Series Processors",
        2,
        2016.0,
        None,
        "A4 PRO-3350B with Radeon\u2122 R4 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A4 PRO-3350B with Radeon\u2122 R4 Graphics @ 2.0 GHz",
    ],
    [
        3800000000,
        4000000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A4 PRO-7300B with Radeon\u2122 HD 8470D",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A4 PRO-7300B with Radeon\u2122 HD 8470D @ 3.8 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A4 PRO-7350B with Radeon\u2122 R5 Graphics",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A4 PRO-7350B with Radeon\u2122 R5 Graphics @ 3.4 GHz",
    ],
    [
        1500000000,
        None,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A4-5000 with Radeon\u2122 HD 8330",
        4,
        4,
        "Laptop, Tablet",
        "cpu",
        "amd",
        " AMD A-Series Processors A4-5000 with Radeon\u2122 HD 8330 @ 1.5 GHz",
    ],
    [
        1550000000,
        None,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A4-5100 with Radeon\u2122 HD 8330",
        4,
        4,
        "Laptop, Tablet",
        "cpu",
        "amd",
        " AMD A-Series Processors A4-5100 with Radeon\u2122 HD 8330 @ 1.6 GHz",
    ],
    [
        1800000000,
        None,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A4-6210 with Radeon\u2122 R3 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A4-6210 with Radeon\u2122 R3 Graphics @ 1.8 GHz",
    ],
    [
        3700000000,
        3900000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A4-6300 with Radeon\u2122 HD 8370D",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A4-6300 with Radeon\u2122 HD 8370D @ 3.7 GHz",
    ],
    [
        3800000000,
        4000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A4-6320 with Radeon\u2122 HD 8370D",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A4-6320 with Radeon\u2122 HD 8370D @ 3.8 GHz",
    ],
    [
        1800000000,
        2200000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A4-7210 with Radeon\u2122 R3 Graphics",
        4,
        4,
        "All-in-One",
        "cpu",
        "amd",
        " AMD A-Series Processors A4-7210 with Radeon\u2122 R3 Graphics @ 1.8 GHz",
    ],
    [
        3800000000,
        4000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A4-7300 with Radeon\u2122 HD 8470D",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A4-7300 with Radeon\u2122 HD 8470D @ 3.8 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A6 PRO-7050B with Radeon\u2122 R4 Graphics",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A6 PRO-7050B with Radeon\u2122 R4 Graphics @ 2.2 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A6 PRO-7400B with Radeon\u2122 R5 Graphics",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A6 PRO-7400B with Radeon\u2122 R5 Graphics @ 3.5 GHz",
    ],
    [
        2000000000,
        None,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-5200 with Radeon\u2122 HD 8400",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-5200 with Radeon\u2122 HD 8400 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-5200M with Radeon\u2122 HD 8400",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-5200M with Radeon\u2122 HD 8400 @ 2.0 GHz",
    ],
    [
        2900000000,
        3500000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-5350M with Radeon\u2122 HD 8450G",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-5350M with Radeon\u2122 HD 8450G @ 2.9 GHz",
    ],
    [
        2400000000,
        None,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-6310 with Radeon\u2122 R4 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-6310 with Radeon\u2122 R4 Graphics @ 2.4 GHz",
    ],
    [
        3900000000,
        4099999999.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-6400B with Radeon\u2122 HD 8470D",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-6400B with Radeon\u2122 HD 8470D @ 3.9 GHz",
    ],
    [
        3900000000,
        4099999999.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-6400K with Radeon\u2122 HD 8470D",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-6400K with Radeon\u2122 HD 8470D @ 3.9 GHz",
    ],
    [
        4000000000,
        4200000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-6420B with Radeon\u2122 HD 8470D",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-6420B with Radeon\u2122 HD 8470D @ 4.0 GHz",
    ],
    [
        4000000000,
        4200000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-6420K with Radeon\u2122 HD 8470D",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-6420K with Radeon\u2122 HD 8470D @ 4.0 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-7000 with Radeon\u2122 R4 Graphics",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-7000 with Radeon\u2122 R4 Graphics @ 2.2 GHz",
    ],
    [
        2000000000,
        2400000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-7310 with Radeon\u2122 R4 Graphics",
        4,
        4,
        "All-in-One",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-7310 with Radeon\u2122 R4 Graphics @ 2.0 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-7400K with Radeon\u2122 R5 Series",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-7400K with Radeon\u2122 R5 Series @ 3.5 GHz",
    ],
    [
        3700000000,
        4000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-7470K with Radeon\u2122 R5 Series",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-7470K with Radeon\u2122 R5 Series @ 3.7 GHz",
    ],
    [
        1600000000,
        3000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A6-8500P with Radeon\u2122 R5 Graphics",
        2,
        2,
        "All-in-One",
        "cpu",
        "amd",
        " AMD A-Series Processors A6-8500P with Radeon\u2122 R5 Graphics @ 1.6 GHz",
    ],
    [
        1900000000,
        3200000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A8 PRO-7150B with Radeon\u2122 R5 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A8 PRO-7150B with Radeon\u2122 R5 Graphics @ 1.9 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "AMD PRO A-Series Processors",
        2,
        None,
        None,
        "A8 PRO-7600B with Radeon\u2122 R7 Graphics",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD PRO A-Series Processors A8 PRO-7600B with Radeon\u2122 R7 Graphics @ 3.1 GHz",
    ],
    [
        2200000000,
        2500000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-6410 with Radeon\u2122 R5 Graphics",
        4,
        4,
        "All-in-One",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-6410 with Radeon\u2122 R5 Graphics @ 2.2 GHz",
    ],
    [
        3500000000,
        4099999999.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-6500 with Radeon\u2122 HD 8570D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-6500 with Radeon\u2122 HD 8570D @ 3.5 GHz",
    ],
    [
        3500000000,
        4099999999.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-6500B with Radeon\u2122 HD 8570D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-6500B with Radeon\u2122 HD 8570D @ 3.5 GHz",
    ],
    [
        2100000000,
        3100000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-6500T with Radeon\u2122 HD 8550D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-6500T with Radeon\u2122 HD 8550D @ 2.1 GHz",
    ],
    [
        3900000000,
        4200000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-6600K with Radeon\u2122 HD 8570D",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-6600K with Radeon\u2122 HD 8570D @ 3.9 GHz",
    ],
    [
        1800000000,
        3000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-7100 with Radeon\u2122 R5 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-7100 with Radeon\u2122 R5 Graphics @ 1.8 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-7200P with Radeon\u2122 R5 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-7200P with Radeon\u2122 R5 Graphics @ 2.4 GHz",
    ],
    [
        2200000000,
        2500000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-7410 with Radeon\u2122 R5 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-7410 with Radeon\u2122 R5 Graphics @ 2.2 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-7600 with Radeon\u2122 R7 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-7600 with Radeon\u2122 R7 Series @ 3.1 GHz",
    ],
    [
        3300000000,
        3800000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-7650K with Radeon\u2122 R7 Graphics and Near Silent Thermal Solution",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-7650K with Radeon\u2122 R7 Graphics and Near Silent Thermal Solution @ 3.3 GHz",
    ],
    [
        3300000000,
        3800000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-7650K with Radeon\u2122 R7 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-7650K with Radeon\u2122 R7 Series @ 3.3 GHz",
    ],
    [
        3600000000,
        3900000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-7670K with Radeon\u2122 R7 Graphics and Near Silent Thermal Solution",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-7670K with Radeon\u2122 R7 Graphics and Near Silent Thermal Solution @ 3.6 GHz",
    ],
    [
        1600000000,
        3000000000.0,
        "AMD A-Series Processors",
        2,
        None,
        None,
        "A8-8600P with Radeon\u2122 R6 Graphics",
        4,
        4,
        "All-in-One",
        "cpu",
        "amd",
        " AMD A-Series Processors A8-8600P with Radeon\u2122 R6 Graphics @ 1.6 GHz",
    ],
    [
        3200000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "AMD Athlon\u2122 200GE",
        2,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 200GE @ 3.2 GHz",
    ],
    [
        3400000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "AMD Athlon\u2122 220GE",
        2,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 220GE @ 3.4 GHz",
    ],
    [
        3500000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "AMD Athlon\u2122 240GE",
        2,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 240GE @ 3.5 GHz",
    ],
    [
        3500000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "AMD Athlon\u2122 3000G",
        2,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 3000G @ 3.5 GHz",
    ],
    [
        3400000000,
        None,
        "AMD Athlon\u2122 Processors",
        1,
        19.0,
        None,
        "AMD Athlon\u2122 300GE",
        2,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 300GE @ 3.4 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "AMD Athlon\u2122 Processors",
        1,
        None,
        None,
        "AMD Athlon\u2122 300U",
        2,
        4,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 300U @ 2.4 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "AMD Athlon\u2122 Processors",
        1,
        2020.0,
        None,
        "AMD Athlon\u2122 Gold 3150U",
        2,
        4,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 Gold 3150U @ 2.6 GHz",
    ],
    [
        3200000000,
        None,
        "AMD Athlon\u2122 PRO Processors",
        1,
        None,
        None,
        "AMD Athlon\u2122 PRO 200GE",
        2,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 PRO 200GE @ 3.2 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "AMD Athlon\u2122 PRO Processors",
        1,
        None,
        None,
        "AMD Athlon\u2122 PRO 200U Mobile Processor with Radeon\u2122 Vega 3 Graphics",
        2,
        4,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 PRO 200U Mobile Processor with Radeon\u2122 Vega 3 Graphics @ 2.3 GHz",
    ],
    [
        3400000000,
        None,
        "AMD Athlon\u2122 PRO Processors",
        1,
        19.0,
        None,
        "AMD Athlon\u2122 PRO 300GE",
        2,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 PRO 300GE @ 3.4 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "AMD Athlon\u2122 PRO Processors",
        1,
        None,
        None,
        "AMD Athlon\u2122 PRO 300U Mobile Processor with Radeon\u2122 Vega 3 Graphics",
        2,
        4,
        "",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 PRO 300U Mobile Processor with Radeon\u2122 Vega 3 Graphics @ 2.4 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "AMD Athlon\u2122 Processors",
        1,
        2020.0,
        None,
        "AMD Athlon\u2122 Silver 3050U",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Athlon\u2122 Silver 3050U @ 2.3 GHz",
    ],
    [
        3100000000,
        3200000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7232P",
        8,
        16,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7232P @ 3.1 GHz",
    ],
    [
        2100000000,
        2900000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7251",
        8,
        16,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7251 @ 2.1 GHz",
    ],
    [
        3100000000,
        3200000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7252",
        8,
        16,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7252 @ 3.1 GHz",
    ],
    [
        2500000000,
        2900000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7261",
        8,
        16,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7261 @ 2.5 GHz",
    ],
    [
        3200000000,
        3400000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7262",
        8,
        16,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7262 @ 3.2 GHz",
    ],
    [
        2900000000,
        3200000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7272",
        12,
        24,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7272 @ 2.9 GHz",
    ],
    [
        2100000000,
        2700000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7281",
        16,
        32,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7281 @ 2.1 GHz",
    ],
    [
        2800000000,
        3200000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7282",
        16,
        32,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7282 @ 2.8 GHz",
    ],
    [
        2200000000,
        2700000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7301",
        16,
        32,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7301 @ 2.2 GHz",
    ],
    [
        3000000000,
        3300000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7302",
        16,
        32,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7302 @ 3.0 GHz",
    ],
    [
        3000000000,
        3300000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7302P",
        16,
        32,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7302P @ 3.0 GHz",
    ],
    [
        2400000000,
        2900000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7351",
        16,
        32,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7351 @ 2.4 GHz",
    ],
    [
        2400000000,
        2900000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7351P",
        16,
        32,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7351P @ 2.4 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7352",
        24,
        48,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7352 @ 2.3 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7371",
        16,
        32,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7371 @ 3.1 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7401",
        24,
        48,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7401 @ 2.0 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "AMD EPYC\u2122",
        3,
        2017.0,
        None,
        "AMD EPYC\u2122 7401P",
        24,
        48,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7401P @ 2.0 GHz",
    ],
    [
        2800000000,
        3350000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7402",
        24,
        48,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7402 @ 2.8 GHz",
    ],
    [
        2800000000,
        3350000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7402P",
        24,
        48,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7402P @ 2.8 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7451",
        24,
        48,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7451 @ 2.3 GHz",
    ],
    [
        2350000000,
        3350000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7452",
        32,
        64,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7452 @ 2.4 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7501",
        32,
        64,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7501 @ 2.0 GHz",
    ],
    [
        2500000000,
        3350000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7502",
        32,
        64,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7502 @ 2.5 GHz",
    ],
    [
        2500000000,
        3350000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7502P",
        32,
        64,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7502P @ 2.5 GHz",
    ],
    [
        2900000000,
        3400000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7542",
        32,
        64,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7542 @ 2.9 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7551",
        32,
        64,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7551 @ 2.0 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "AMD EPYC\u2122",
        3,
        2017.0,
        None,
        "AMD EPYC\u2122 7551P",
        32,
        64,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7551P @ 2.0 GHz",
    ],
    [
        2200000000,
        3300000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7552",
        48,
        96,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7552 @ 2.2 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "AMD EPYC\u2122",
        3,
        2017.0,
        None,
        "AMD EPYC\u2122 7601",
        32,
        64,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7601 @ 2.2 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7642",
        48,
        96,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7642 @ 2.3 GHz",
    ],
    [
        2000000000,
        3350000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7702",
        64,
        128,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7702 @ 2.0 GHz",
    ],
    [
        2000000000,
        3350000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7702P",
        64,
        128,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7702P @ 2.0 GHz",
    ],
    [
        2250000000,
        3400000000.0,
        "AMD EPYC\u2122",
        3,
        None,
        None,
        "AMD EPYC\u2122 7742",
        64,
        128,
        "Server",
        "cpu",
        "amd",
        "  AMD EPYC\u2122 7742 @ 2.2 GHz",
    ],
    [
        1900000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "AMD Opteron\u2122 X2150 APU",
        4,
        4,
        "Server",
        "cpu",
        "amd",
        "  AMD Opteron\u2122 X2150 APU @ 1.9 GHz",
    ],
    [
        2400000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "AMD Opteron\u2122 X2170",
        4,
        4,
        "Server",
        "cpu",
        "amd",
        "  AMD Opteron\u2122 X2170 @ 2.4 GHz",
    ],
    [
        3100000000,
        3400000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 3 1200",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 1200 @ 3.1 GHz",
    ],
    [
        3500000000,
        3700000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 3 1300X",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 1300X @ 3.5 GHz",
    ],
    [
        3500000000,
        3700000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 3 2200G with Radeon\u2122 Vega 8 Graphics",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 2200G with Radeon\u2122 Vega 8 Graphics @ 3.5 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 3 2200GE with Radeon\u2122 Vega 8 Graphics",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 2200GE with Radeon\u2122 Vega 8 Graphics @ 3.2 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 3 2200U",
        2,
        4,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 2200U @ 2.5 GHz",
    ],
    [
        2000000000,
        3400000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 3 2300U",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 2300U @ 2.0 GHz",
    ],
    [
        3500000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 3 2300X",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 2300X @ 3.5 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 3 3200G with Radeon\u2122 Vega 8 Graphics",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 3200G with Radeon\u2122 Vega 8 Graphics @ 3.6 GHz",
    ],
    [
        3300000000,
        3800000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        19.0,
        None,
        "AMD Ryzen\u2122 3 3200GE",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 3200GE @ 3.3 GHz",
    ],
    [
        2600000000,
        3500000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 3 3200U",
        2,
        4,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 3200U @ 2.6 GHz",
    ],
    [
        2600000000,
        3500000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2020.0,
        None,
        "AMD Ryzen\u2122 3 3250U",
        2,
        4,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 3250U @ 2.6 GHz",
    ],
    [
        2100000000,
        3500000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 3 3300U",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 3300U @ 2.1 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2020.0,
        None,
        "AMD Ryzen\u2122 3 4300U",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 3 4300U @ 2.7 GHz",
    ],
    [
        3100000000,
        3400000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2017.0,
        None,
        "AMD Ryzen\u2122 3 PRO 1200",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 3 PRO 1200 @ 3.1 GHz",
    ],
    [
        3500000000,
        3700000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2017.0,
        None,
        "AMD Ryzen\u2122 3 PRO 1300",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 3 PRO 1300 @ 3.5 GHz",
    ],
    [
        3500000000,
        3700000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 3 PRO 2200G with Radeon\u2122 Vega 8 Graphics",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 3 PRO 2200G with Radeon\u2122 Vega 8 Graphics @ 3.5 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 3 PRO 2200GE with Radeon\u2122 Vega 8 Graphics",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 3 PRO 2200GE with Radeon\u2122 Vega 8 Graphics @ 3.2 GHz",
    ],
    [
        2000000000,
        3400000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 3 PRO 2300U",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 3 PRO 2300U @ 2.0 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        19.0,
        None,
        "AMD Ryzen\u2122 3 PRO 3200G",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 3 PRO 3200G @ 3.6 GHz",
    ],
    [
        3300000000,
        3800000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        19.0,
        None,
        "AMD Ryzen\u2122 3 PRO 3200GE",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 3 PRO 3200GE @ 3.3 GHz",
    ],
    [
        2100000000,
        3500000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 3 PRO 3300U",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 3 PRO 3300U @ 2.1 GHz",
    ],
    [
        3200000000,
        3400000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 5 1400",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 1400 @ 3.2 GHz",
    ],
    [
        3500000000,
        3700000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 5 1500X",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 1500X @ 3.5 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 5 1600",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 1600 @ 3.2 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 5 1600X",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 1600X @ 3.6 GHz",
    ],
    [
        3600000000,
        3900000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 5 2400G with Radeon\u2122 RX Vega 11 Graphics",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 2400G with Radeon\u2122 RX Vega 11 Graphics @ 3.6 GHz",
    ],
    [
        3200000000,
        3800000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 5 2400GE with Radeon\u2122 RX Vega 11 Graphics",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 2400GE with Radeon\u2122 RX Vega 11 Graphics @ 3.2 GHz",
    ],
    [
        2000000000,
        3600000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 5 2500U",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 2500U @ 2.0 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 5 2500X",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 2500X @ 3.6 GHz",
    ],
    [
        3400000000,
        3900000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 5 2600",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 2600 @ 3.4 GHz",
    ],
    [
        3100000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 5 2600E",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 2600E @ 3.1 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 5 2600H",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 2600H @ 3.2 GHz",
    ],
    [
        3600000000,
        4200000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 5 2600X",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 2600X @ 3.6 GHz",
    ],
    [
        3700000000,
        4200000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 5 3400G with Radeon\u2122 RX Vega 11 Graphics",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 3400G with Radeon\u2122 RX Vega 11 Graphics @ 3.7 GHz",
    ],
    [
        3300000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        19.0,
        None,
        "AMD Ryzen\u2122 5 3400GE",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 3400GE @ 3.3 GHz",
    ],
    [
        3600000000,
        4099999999.0,
        "AMD Ryzen\u2122 Processors",
        3,
        None,
        None,
        "AMD Ryzen\u2122 5 3500 Processor (OEM Only)",
        6,
        6,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 3500 Processor (OEM Only) @ 3.6 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 5 3500U",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 3500U @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 5 3550H",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 3550H @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        None,
        "Zen+",
        "AMD Ryzen\u2122 5 3580U Microsoft Surface\u00ae Edition",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 3580U Microsoft Surface\u00ae Edition @ 2.1 GHz",
    ],
    [
        3600000000,
        4200000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 5 3600",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 3600 @ 3.6 GHz",
    ],
    [
        3800000000,
        4400000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 5 3600X",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 3600X @ 3.8 GHz",
    ],
    [
        2300000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2020.0,
        None,
        "AMD Ryzen\u2122 5 4500U",
        6,
        6,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 4500U @ 2.3 GHz",
    ],
    [
        3000000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2020.0,
        None,
        "AMD Ryzen\u2122 5 4600H",
        6,
        12,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 4600H @ 3.0 GHz",
    ],
    [
        2100000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2020.0,
        None,
        "AMD Ryzen\u2122 5 4600U",
        6,
        12,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 5 4600U @ 2.1 GHz",
    ],
    [
        3500000000,
        3700000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2017.0,
        None,
        "AMD Ryzen\u2122 5 PRO 1500",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 1500 @ 3.5 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2017.0,
        None,
        "AMD Ryzen\u2122 5 PRO 1600",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 1600 @ 3.2 GHz",
    ],
    [
        3600000000,
        3900000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 5 PRO 2400G with Radeon\u2122 Vega 11 Graphics",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 2400G with Radeon\u2122 Vega 11 Graphics @ 3.6 GHz",
    ],
    [
        3200000000,
        3800000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 5 PRO 2400GE with Radeon\u2122 Vega 11 Graphics",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 2400GE with Radeon\u2122 Vega 11 Graphics @ 3.2 GHz",
    ],
    [
        2000000000,
        3600000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 5 PRO 2500U",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 2500U @ 2.0 GHz",
    ],
    [
        3400000000,
        3900000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        None,
        None,
        "AMD Ryzen\u2122 5 PRO 2600",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 2600 @ 3.4 GHz",
    ],
    [
        3700000000,
        4200000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        19.0,
        None,
        "AMD Ryzen\u2122 5 PRO 3400G",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 3400G @ 3.7 GHz",
    ],
    [
        3300000000,
        4000000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        19.0,
        None,
        "AMD Ryzen\u2122 5 PRO 3400GE",
        4,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 3400GE @ 3.3 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 5 PRO 3500U",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 3500U @ 2.1 GHz",
    ],
    [
        3600000000,
        4200000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        19.0,
        None,
        "AMD Ryzen\u2122 5 PRO 3600",
        6,
        12,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 5 PRO 3600 @ 3.6 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 7 1700 Processor",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 1700 Processor @ 3.0 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 7 1700X",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 1700X @ 3.4 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 7 1800X",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 1800X @ 3.6 GHz",
    ],
    [
        3200000000,
        4099999999.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 7 2700",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 2700 @ 3.2 GHz",
    ],
    [
        2800000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 7 2700E Processor",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 2700E Processor @ 2.8 GHz",
    ],
    [
        2200000000,
        3800000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 7 2700U",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 2700U @ 2.2 GHz",
    ],
    [
        3700000000,
        4300000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        None,
        "AMD Ryzen\u2122 7 2700X",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 2700X @ 3.7 GHz",
    ],
    [
        3300000000,
        3800000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 7 2800H",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 2800H @ 3.3 GHz",
    ],
    [
        2300000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 7 3700U",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 3700U @ 2.3 GHz",
    ],
    [
        3600000000,
        4400000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 7 3700X",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 3700X @ 3.6 GHz",
    ],
    [
        2300000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 7 3750H",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 3750H @ 2.3 GHz",
    ],
    [
        2300000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        None,
        "Zen+",
        "AMD Ryzen\u2122 7 3780U Microsoft Surface\u00ae Edition",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 3780U Microsoft Surface\u00ae Edition @ 2.3 GHz",
    ],
    [
        3900000000,
        4500000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 7 3800X",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 3800X @ 3.9 GHz",
    ],
    [
        2000000000,
        4099999999.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2020.0,
        None,
        "AMD Ryzen\u2122 7 4700U",
        8,
        8,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 4700U @ 2.0 GHz",
    ],
    [
        2900000000,
        4200000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2020.0,
        None,
        "AMD Ryzen\u2122 7 4800H",
        8,
        16,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 4800H @ 2.9 GHz",
    ],
    [
        1800000000,
        4200000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2020.0,
        None,
        "AMD Ryzen\u2122 7 4800U",
        8,
        16,
        "Laptop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 7 4800U @ 1.8 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2017.0,
        None,
        "AMD Ryzen\u2122 7 PRO 1700",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 7 PRO 1700 @ 3.0 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2017.0,
        None,
        "AMD Ryzen\u2122 7 PRO 1700X Processor",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 7 PRO 1700X Processor @ 3.4 GHz",
    ],
    [
        3200000000,
        4099999999.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        None,
        None,
        "AMD Ryzen\u2122 7 PRO 2700",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 7 PRO 2700 @ 3.2 GHz",
    ],
    [
        2200000000,
        3800000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 7 PRO 2700U",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 7 PRO 2700U @ 2.2 GHz",
    ],
    [
        3600000000,
        4099999999.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        None,
        None,
        "AMD Ryzen\u2122 7 PRO 2700X",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 7 PRO 2700X @ 3.6 GHz",
    ],
    [
        3600000000,
        4400000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        19.0,
        None,
        "AMD Ryzen\u2122 7 PRO 3700",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 7 PRO 3700 @ 3.6 GHz",
    ],
    [
        2300000000,
        4000000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 7 PRO 3700U",
        4,
        8,
        "Laptop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 7 PRO 3700U @ 2.3 GHz",
    ],
    [
        3100000000,
        4300000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 9 3900 Processor (OEM Only)",
        12,
        24,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 9 3900 Processor (OEM Only) @ 3.1 GHz",
    ],
    [
        3800000000,
        4600000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        None,
        "AMD Ryzen\u2122 9 3900X",
        12,
        24,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 9 3900X @ 3.8 GHz",
    ],
    [
        3500000000,
        4700000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        None,
        None,
        "AMD Ryzen\u2122 9 3950X",
        16,
        32,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 9 3950X @ 3.5 GHz",
    ],
    [
        3100000000,
        4300000000.0,
        "AMD Ryzen\u2122 PRO Processors",
        3,
        19.0,
        None,
        "AMD Ryzen\u2122 9 PRO 3900",
        12,
        24,
        "Desktop",
        "cpu",
        "amd",
        " AMD Ryzen\u2122 PRO Processors AMD Ryzen\u2122 9 PRO 3900 @ 3.1 GHz",
    ],
    [
        3800000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        None,
        "AMD Ryzen\u2122 Threadripper\u2122 1900X",
        8,
        16,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 1900X @ 3.8 GHz",
    ],
    [
        3500000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 Threadripper\u2122 1920X",
        12,
        24,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 1920X @ 3.5 GHz",
    ],
    [
        3400000000,
        4000000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2017.0,
        "Zen",
        "AMD Ryzen\u2122 Threadripper\u2122 1950X",
        16,
        32,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 1950X @ 3.4 GHz",
    ],
    [
        3500000000,
        4300000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        18.0,
        "Zen",
        "AMD Ryzen\u2122 Threadripper\u2122 2920X",
        12,
        24,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 2920X @ 3.5 GHz",
    ],
    [
        3500000000,
        4400000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 Threadripper\u2122 2950X",
        16,
        32,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 2950X @ 3.5 GHz",
    ],
    [
        3000000000,
        4200000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        18.0,
        "Zen",
        "AMD Ryzen\u2122 Threadripper\u2122 2970WX",
        24,
        48,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 2970WX @ 3.0 GHz",
    ],
    [
        3000000000,
        4200000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2018.0,
        "Zen",
        "AMD Ryzen\u2122 Threadripper\u2122 2990WX",
        32,
        64,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 2990WX @ 3.0 GHz",
    ],
    [
        3800000000,
        4500000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 Threadripper\u2122 3960X",
        24,
        48,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 3960X @ 3.8 GHz",
    ],
    [
        3700000000,
        4500000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2019.0,
        "Zen",
        "AMD Ryzen\u2122 Threadripper\u2122 3970X",
        32,
        64,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 3970X @ 3.7 GHz",
    ],
    [
        2900000000,
        4300000000.0,
        "AMD Ryzen\u2122 Processors",
        3,
        2020.0,
        "Zen",
        "AMD Ryzen\u2122 Threadripper\u2122 3990X",
        64,
        128,
        "Desktop",
        "cpu",
        "amd",
        "  AMD Ryzen\u2122 Threadripper\u2122 3990X @ 2.9 GHz",
    ],
    [
        1600000000,
        None,
        "AMD Athlon\u2122 Processors",
        2,
        None,
        None,
        "Athlon\u2122 5150 APU with Radeon\u2122 R3 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors Athlon\u2122 5150 APU with Radeon\u2122 R3 Series @ 1.6 GHz",
    ],
    [
        2049999999,
        None,
        "AMD Athlon\u2122 Processors",
        2,
        None,
        None,
        "Athlon\u2122 5350 APU with Radeon\u2122 R3 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors Athlon\u2122 5350 APU with Radeon\u2122 R3 Series @ 2.0 GHz",
    ],
    [
        2200000000,
        None,
        "AMD Athlon\u2122 Processors",
        2,
        None,
        None,
        "Athlon\u2122 5370 APU with Radeon\u2122 R3 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Athlon\u2122 Processors Athlon\u2122 5370 APU with Radeon\u2122 R3 Series @ 2.2 GHz",
    ],
    [
        3200000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "B57",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 B57 @ 3.2 GHz",
    ],
    [
        3400000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "B59",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 B59 @ 3.4 GHz",
    ],
    [
        3500000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "B60",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 B60 @ 3.5 GHz",
    ],
    [
        3000000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "B75",
        3,
        3,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 B75 @ 3.0 GHz",
    ],
    [
        3200000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "B77",
        3,
        3,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 B77 @ 3.2 GHz",
    ],
    [
        3200000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "B97",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 B97 @ 3.2 GHz",
    ],
    [
        3300000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "B99",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 B99 @ 3.3 GHz",
    ],
    [
        1400000000,
        None,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E1 Micro-6200T with Radeon\u2122 R2 Graphics",
        2,
        2,
        "Laptop, Tablet",
        "cpu",
        "amd",
        " AMD E-Series Processors E1 Micro-6200T with Radeon\u2122 R2 Graphics @ 1.4 GHz",
    ],
    [
        1000000000,
        None,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E1-2100 with Radeon\u2122 HD 8210",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD E-Series Processors E1-2100 with Radeon\u2122 HD 8210 @ 1.0 GHz",
    ],
    [
        1000000000,
        None,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E1-2200 with Radeon\u2122 HD 8210",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD E-Series Processors E1-2200 with Radeon\u2122 HD 8210 @ 1.0 GHz",
    ],
    [
        1400000000,
        None,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E1-2500 with Radeon\u2122 HD 8240",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD E-Series Processors E1-2500 with Radeon\u2122 HD 8240 @ 1.4 GHz",
    ],
    [
        1350000000,
        None,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E1-6010 with Radeon\u2122 R2 Graphics",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD E-Series Processors E1-6010 with Radeon\u2122 R2 Graphics @ 1.4 GHz",
    ],
    [
        1500000000,
        1500000000.0,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E1-7010 with Radeon\u2122 R2 Graphics",
        2,
        2,
        "All-in-One",
        "cpu",
        "amd",
        " AMD E-Series Processors E1-7010 with Radeon\u2122 R2 Graphics @ 1.5 GHz",
    ],
    [
        1650000000,
        None,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E2-3000 with Radeon\u2122 HD 8280",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD E-Series Processors E2-3000 with Radeon\u2122 HD 8280 @ 1.6 GHz",
    ],
    [
        1300000000,
        None,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E2-3800 with Radeon\u2122 HD 8280",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD E-Series Processors E2-3800 with Radeon\u2122 HD 8280 @ 1.3 GHz",
    ],
    [
        1500000000,
        None,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E2-6110 with Radeon\u2122 R2 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD E-Series Processors E2-6110 with Radeon\u2122 R2 Graphics @ 1.5 GHz",
    ],
    [
        1800000000,
        1800000000.0,
        "AMD E-Series Processors",
        2,
        None,
        None,
        "E2-7110 with Radeon\u2122 R2 Graphics",
        4,
        4,
        "All-in-One",
        "cpu",
        "amd",
        " AMD E-Series Processors E2-7110 with Radeon\u2122 R2 Graphics @ 1.8 GHz",
    ],
    [
        3300000000,
        3900000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX 6100",
        6,
        6,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX 6100 @ 3.3 GHz",
    ],
    [
        3600000000,
        3800000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-4100",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-4100 @ 3.6 GHz",
    ],
    [
        3800000000,
        3900000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-4130",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-4130 @ 3.8 GHz",
    ],
    [
        4200000000,
        4300000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-4170",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-4170 @ 4.2 GHz",
    ],
    [
        3800000000,
        4000000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-4300",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-4300 @ 3.8 GHz",
    ],
    [
        4000000000,
        4099999999.0,
        "AMD FX-Series Processors",
        2,
        None,
        None,
        "FX-4320",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-4320 @ 4.0 GHz",
    ],
    [
        4200000000,
        4300000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-4350",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-4350 @ 4.2 GHz",
    ],
    [
        3800000000,
        4099999999.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-6200",
        6,
        6,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-6200 @ 3.8 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "AMD FX-Series Processors",
        2,
        None,
        None,
        "FX-6300",
        6,
        6,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-6300 @ 3.5 GHz",
    ],
    [
        3900000000,
        4200000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-6350",
        6,
        6,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-6350 @ 3.9 GHz",
    ],
    [
        3900000000,
        4200000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-6350 with Wraith cooler",
        6,
        6,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-6350 with Wraith cooler @ 3.9 GHz",
    ],
    [
        2100000000,
        3300000000.0,
        "AMD FX-Series Processors",
        2,
        None,
        None,
        "FX-7500 with Radeon\u2122 R7 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-7500 with Radeon\u2122 R7 Graphics @ 2.1 GHz",
    ],
    [
        2700000000,
        3600000000.0,
        "AMD FX-Series Processors",
        2,
        None,
        None,
        "FX-7600P with Radeon\u2122 R7 Graphics",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-7600P with Radeon\u2122 R7 Graphics @ 2.7 GHz",
    ],
    [
        3100000000,
        4000000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8120",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8120 @ 3.1 GHz",
    ],
    [
        3600000000,
        4200000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8150",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8150 @ 3.6 GHz",
    ],
    [
        3300000000,
        4200000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8300",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8300 @ 3.3 GHz",
    ],
    [
        3400000000,
        4300000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8310",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8310 @ 3.4 GHz",
    ],
    [
        3500000000,
        4000000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8320",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8320 @ 3.5 GHz",
    ],
    [
        3200000000,
        4000000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8320E",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8320E @ 3.2 GHz",
    ],
    [
        4000000000,
        4200000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8350",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8350 @ 4.0 GHz",
    ],
    [
        4000000000,
        4200000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8350 with Wraith cooler",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8350 with Wraith cooler @ 4.0 GHz",
    ],
    [
        4000000000,
        4300000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8370",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8370 @ 4.0 GHz",
    ],
    [
        4000000000,
        4300000000.0,
        "AMD FX-Series Processors",
        2,
        None,
        None,
        "FX-8370 with Wraith cooler",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8370 with Wraith cooler @ 4.0 GHz",
    ],
    [
        3300000000,
        4300000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-8370E",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8370E @ 3.3 GHz",
    ],
    [
        2100000000,
        3400000000.0,
        "AMD FX-Series Processors",
        2,
        None,
        None,
        "FX-8800P with Radeon\u2122 R7 Graphics",
        4,
        4,
        "All-in-One",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-8800P with Radeon\u2122 R7 Graphics @ 2.1 GHz",
    ],
    [
        4400000000,
        4700000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-9370",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-9370 @ 4.4 GHz",
    ],
    [
        4700000000,
        5000000000.0,
        "AMD FX-Series Processors",
        1,
        None,
        None,
        "FX-9590",
        8,
        8,
        "Desktop",
        "cpu",
        "amd",
        " AMD FX-Series Processors FX-9590 @ 4.7 GHz",
    ],
    [
        2800000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "N640",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 N640 @ 2.8 GHz",
    ],
    [
        3000000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "N660",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 N660 @ 3.0 GHz",
    ],
    [
        2300000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "N870",
        3,
        3,
        "Laptop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 N870 @ 2.3 GHz",
    ],
    [
        1800000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "N960",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 N960 @ 1.8 GHz",
    ],
    [
        2200000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "N970",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 N970 @ 2.2 GHz",
    ],
    [
        2600000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "P650",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 P650 @ 2.6 GHz",
    ],
    [
        2000000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "P860",
        3,
        3,
        "Laptop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 P860 @ 2.0 GHz",
    ],
    [
        1450000000,
        None,
        "AMD Sempron\u2122",
        2,
        None,
        None,
        "Sempron\u2122 2650 APU with Radeon\u2122 R3 Series",
        2,
        2,
        "Desktop",
        "cpu",
        "amd",
        " AMD Sempron\u2122 Sempron\u2122 2650 APU with Radeon\u2122 R3 Series @ 1.4 GHz",
    ],
    [
        1300000000,
        None,
        "AMD Sempron\u2122",
        2,
        None,
        None,
        "Sempron\u2122 3850 APU with Radeon\u2122 R3 Series",
        4,
        4,
        "Desktop",
        "cpu",
        "amd",
        " AMD Sempron\u2122 Sempron\u2122 3850 APU with Radeon\u2122 R3 Series @ 1.3 GHz",
    ],
    [
        1600000000,
        None,
        "AMD Turion\u2122",
        0,
        None,
        None,
        "TL-52",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD Turion\u2122 TL-52 @ 1.6 GHz",
    ],
    [
        2000000000,
        None,
        "AMD Turion\u2122",
        0,
        None,
        None,
        "TL-60",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD Turion\u2122 TL-60 @ 2.0 GHz",
    ],
    [
        2200000000,
        None,
        "AMD Turion\u2122",
        0,
        None,
        None,
        "TL-64",
        2,
        2,
        "Laptop",
        "cpu",
        "amd",
        " AMD Turion\u2122 TL-64 @ 2.2 GHz",
    ],
    [
        2000000000,
        None,
        "AMD Opteron\u2122",
        1,
        None,
        None,
        "X1150",
        4,
        4,
        "Server",
        "cpu",
        "amd",
        " AMD Opteron\u2122 X1150 @ 2.0 GHz",
    ],
    [
        2400000000,
        None,
        "AMD Phenom\u2122",
        1,
        None,
        None,
        "X940",
        4,
        4,
        "Laptop",
        "cpu",
        "amd",
        " AMD Phenom\u2122 X940 @ 2.4 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Celeron Processor 1000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "1000M",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 1000 Series 1000M @ 1.8 GHz",
    ],
    [
        1900000000,
        None,
        "Intel Celeron Processor 1000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "1005M",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 1000 Series 1005M @ 1.9 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Celeron Processor 1000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "1007U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 1000 Series 1007U @ 1.5 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor 1000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "1017U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 1000 Series 1017U @ 1.6 GHz",
    ],
    [
        1000000000,
        None,
        "Intel Celeron Processor 1000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "1019Y",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 1000 Series 1019Y @ 1.0 GHz",
    ],
    [
        2200000000,
        None,
        "Intel Celeron Processor 1000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "1020E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor 1000 Series 1020E @ 2.2 GHz",
    ],
    [
        2100000000,
        None,
        "Intel Celeron Processor 1000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "1020M",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 1000 Series 1020M @ 2.1 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Celeron Processor 1000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "1037U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 1000 Series 1037U @ 1.8 GHz",
    ],
    [
        1400000000,
        None,
        "Intel Celeron Processor 1000 Series",
        0,
        2013.0,
        "Ivy Bridge",
        "1047UE",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor 1000 Series 1047UE @ 1.4 GHz",
    ],
    [
        2600000000,
        None,
        "Intel Pentium Processor 1000 Series",
        2,
        2012.0,
        "Sandy Bridge EN",
        "1403",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Intel Pentium Processor 1000 Series 1403 @ 2.6 GHz",
    ],
    [
        2600000000,
        None,
        "Intel Pentium Processor 1000 Series",
        2,
        2014.0,
        "Volcano Canyon",
        "1403V2",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Intel Pentium Processor 1000 Series 1403V2 @ 2.6 GHz",
    ],
    [
        1200000000,
        1800000000.0,
        "Intel Pentium Processor 1000 Series",
        2,
        2012.0,
        "Sandy Bridge EN",
        "1405",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Intel Pentium Processor 1000 Series 1405 @ 1.2 GHz",
    ],
    [
        1400000000,
        None,
        "Intel Pentium Processor 1000 Series",
        2,
        2014.0,
        "Ivy Bridge",
        "1405V2",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Pentium Processor 1000 Series 1405V2 @ 1.4 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Pentium Processor 1000 Series",
        2,
        2012.0,
        "Sandy Bridge EN",
        "1407",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Intel Pentium Processor 1000 Series 1407 @ 2.8 GHz",
    ],
    [
        2200000000,
        None,
        "Intel Celeron Processor 2000 Series",
        1,
        2014.0,
        "Haswell",
        "2000E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor 2000 Series 2000E @ 2.2 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Celeron Processor 2000 Series",
        1,
        2014.0,
        "Haswell",
        "2002E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor 2000 Series 2002E @ 1.5 GHz",
    ],
    [
        2400000000,
        None,
        "Intel Pentium Processor 2000 Series",
        1,
        2012.0,
        "Ivy Bridge",
        "2020M",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 2000 Series 2020M @ 2.4 GHz",
    ],
    [
        2500000000,
        None,
        "Intel Pentium Processor 2000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "2030M",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 2000 Series 2030M @ 2.5 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Pentium Processor 2000 Series",
        1,
        2012.0,
        "Ivy Bridge",
        "2117U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 2000 Series 2117U @ 1.8 GHz",
    ],
    [
        1900000000,
        None,
        "Intel Pentium Processor 2000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "2127U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 2000 Series 2127U @ 1.9 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Pentium Processor 2000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "2129Y",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 2000 Series 2129Y @ 1.1 GHz",
    ],
    [
        1330000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        None,
        "Yonah",
        "215",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 215 @ 1.3 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2007.0,
        "Conroe",
        "220",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 220 @ 1.2 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Celeron Processor 2000 Series",
        1,
        2013.0,
        "Haswell",
        "2950M",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 2000 Series 2950M @ 2.0 GHz",
    ],
    [
        1400000000,
        None,
        "Intel Celeron Processor 2000 Series",
        1,
        2013.0,
        "Haswell",
        "2955U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 2000 Series 2955U @ 1.4 GHz",
    ],
    [
        1400000000,
        None,
        "Intel Celeron Processor 2000 Series",
        1,
        2013.0,
        "Haswell",
        "2957U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 2000 Series 2957U @ 1.4 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Celeron Processor 2000 Series",
        1,
        2013.0,
        "Haswell",
        "2961Y",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 2000 Series 2961Y @ 1.1 GHz",
    ],
    [
        2200000000,
        None,
        "Intel Celeron Processor 2000 Series",
        1,
        2014.0,
        "Haswell",
        "2970M",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 2000 Series 2970M @ 2.2 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor 2000 Series",
        1,
        2013.0,
        "Haswell",
        "2980U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 2000 Series 2980U @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor 2000 Series",
        1,
        2013.0,
        "Haswell",
        "2981U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 2000 Series 2981U @ 1.6 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Conroe",
        "3040",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 3040 @ 1.9 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Conroe",
        "3050",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 3050 @ 2.1 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Conroe",
        "3060",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 3060 @ 2.4 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Conroe",
        "3065",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 3065 @ 2.3 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Conroe",
        "3070",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 3070 @ 2.7 GHz",
    ],
    [
        1700000000,
        None,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "3104",
        6,
        "6",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 3104 @ 1.7 GHz",
    ],
    [
        1700000000,
        None,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "3106",
        8,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 3106 @ 1.7 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Xeon Phi x100 Product Family",
        0,
        2013.0,
        "Knights Corner",
        "3120A",
        57,
        57,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x100 Product Family 3120A @ 1.1 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Xeon Phi x100 Product Family",
        0,
        2013.0,
        "Knights Corner",
        "3120P",
        57,
        57,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x100 Product Family 3120P @ 1.1 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Xeon Phi x100 Product Family",
        0,
        2013.0,
        "Knights Corner",
        "31S1P",
        57,
        57,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x100 Product Family 31S1P @ 1.1 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        None,
        "Prescott",
        "320",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 320 @ 2.4 GHz",
    ],
    [
        1900000000,
        1900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "3204",
        6,
        "6",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 3204 @ 1.9 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2015.0,
        "Broadwell",
        "3205U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3205U @ 1.5 GHz",
    ],
    [
        1700000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2015.0,
        "Broadwell",
        "3215U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3215U @ 1.7 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2006.0,
        "Cedarmill",
        "347",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 347 @ 3.1 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "350",
        2,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 350 @ 1.2 GHz",
    ],
    [
        2300000000,
        None,
        "Intel Pentium Processor 3000 Series",
        1,
        2013.0,
        "Haswell",
        "3550M",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 3000 Series 3550M @ 2.3 GHz",
    ],
    [
        1700000000,
        None,
        "Intel Pentium Processor 3000 Series",
        1,
        2013.0,
        "Haswell",
        "3556U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 3000 Series 3556U @ 1.7 GHz",
    ],
    [
        1700000000,
        None,
        "Intel Pentium Processor 3000 Series",
        1,
        2013.0,
        "Haswell",
        "3558U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 3000 Series 3558U @ 1.7 GHz",
    ],
    [
        2400000000,
        None,
        "Intel Pentium Processor 3000 Series",
        1,
        2014.0,
        "Haswell",
        "3560M",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 3000 Series 3560M @ 2.4 GHz",
    ],
    [
        1200000000,
        None,
        "Intel Pentium Processor 3000 Series",
        1,
        2013.0,
        "Haswell",
        "3560Y",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 3000 Series 3560Y @ 1.2 GHz",
    ],
    [
        1200000000,
        None,
        "Intel Pentium Processor 3000 Series",
        1,
        2013.0,
        "Haswell",
        "3561Y",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 3000 Series 3561Y @ 1.2 GHz",
    ],
    [
        3600000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2007.0,
        "Cedarmill",
        "365",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 365 @ 3.6 GHz",
    ],
    [
        1700000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2015.0,
        "Broadwell",
        "3755U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3755U @ 1.7 GHz",
    ],
    [
        1900000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2015.0,
        "Broadwell",
        "3765U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3765U @ 1.9 GHz",
    ],
    [
        1900000000,
        None,
        "Intel Pentium Processor 3000 Series",
        1,
        2015.0,
        "Broadwell",
        "3805U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 3000 Series 3805U @ 1.9 GHz",
    ],
    [
        1900000000,
        None,
        "Intel Pentium Processor 3000 Series",
        1,
        2015.0,
        "Broadwell",
        "3825U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 3000 Series 3825U @ 1.9 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2015.0,
        "Skylake",
        "3855U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3855U @ 1.6 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2017.0,
        "Kaby Lake",
        "3865U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3865U @ 1.8 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2019.0,
        "Kaby Lake R",
        "3867U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3867U @ 1.8 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2015.0,
        "Skylake",
        "3955U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3955U @ 2.0 GHz",
    ],
    [
        2200000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2017.0,
        "Kaby Lake",
        "3965U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3965U @ 2.2 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Celeron Processor 3000 Series",
        1,
        2017.0,
        "Kaby Lake",
        "3965Y",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 3000 Series 3965Y @ 1.5 GHz",
    ],
    [
        1800000000,
        3000000000.0,
        "Intel Xeon Scalable Processors",
        0,
        2018.0,
        "Skylake",
        "4106H",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4106H @ 1.8 GHz",
    ],
    [
        1800000000,
        3000000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "4108",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4108 @ 1.8 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "4109T",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4109T @ 2.0 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "4110",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4110 @ 2.1 GHz",
    ],
    [
        2600000000,
        3000000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "4112",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4112 @ 2.6 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "4114",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4114 @ 2.2 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "4114T",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4114T @ 2.2 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "4116",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4116 @ 2.1 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "4116T",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4116T @ 2.1 GHz",
    ],
    [
        3000000000,
        3200000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2018.0,
        "Skylake",
        "4123",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 4123 @ 3.0 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2007.0,
        "Conroe",
        "420",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 420 @ 1.6 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Celeron Processor 4000 Series",
        1,
        2019.0,
        "Whiskey Lake",
        "4205U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 4000 Series 4205U @ 1.8 GHz",
    ],
    [
        2100000000,
        3200000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "4208",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 4208 @ 2.1 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "4209T",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 4209T @ 2.2 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "4210",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 4210 @ 2.2 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "4214",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 4214 @ 2.2 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "4214Y",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 4214Y @ 2.2 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "4215",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 4215 @ 2.5 GHz",
    ],
    [
        2100000000,
        3200000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "4216",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 4216 @ 2.1 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2007.0,
        "Conroe",
        "430",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 430 @ 1.8 GHz",
    ],
    [
        2200000000,
        None,
        "Intel Celeron Processor 4000 Series",
        3,
        2019.0,
        "Whiskey Lake",
        "4305U",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor 4000 Series 4305U @ 2.2 GHz",
    ],
    [
        2000000000,
        2000000000.0,
        "Intel Celeron Processor 4000 Series",
        0,
        2019.0,
        "Whiskey Lake",
        "4305UE",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor 4000 Series 4305UE @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2006.0,
        "Conroe",
        "440",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 440 @ 2.0 GHz",
    ],
    [
        2100000000,
        None,
        "Intel Pentium Processor 4000 Series",
        1,
        2015.0,
        "Skylake",
        "4405U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 4000 Series 4405U @ 2.1 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Pentium Processor 4000 Series",
        1,
        2015.0,
        "Skylake",
        "4405Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 4000 Series 4405Y @ 1.5 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2017.0,
        "Kaby Lake",
        "4410Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series 4410Y @ 1.5 GHz",
    ],
    [
        2300000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2017.0,
        "Kaby Lake",
        "4415U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series 4415U @ 2.3 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2017.0,
        "Kaby Lake",
        "4415Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series 4415Y @ 1.6 GHz",
    ],
    [
        2300000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2019.0,
        "Kaby Lake R",
        "4417U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series 4417U @ 2.3 GHz",
    ],
    [
        1700000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2019.0,
        "Kaby Lake",
        "4425Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series 4425Y @ 1.7 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        None,
        "Yonah",
        "443",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 443 @ 1.2 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        None,
        "Conroe",
        "445",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 445 @ 1.9 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2008.0,
        "Conroe",
        "450",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 450 @ 2.2 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Dempsey",
        "5030",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5030 @ 2.7 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Dempsey",
        "5040",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5040 @ 2.8 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "505",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 505 @ 2.7 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Dempsey",
        "5050",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5050 @ 3.0 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "505J",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 505J @ 2.7 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "506",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 506 @ 2.7 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Dempsey",
        "5060",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5060 @ 3.2 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Dempsey",
        "5063",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5063 @ 3.2 GHz",
    ],
    [
        3460000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Dempsey",
        "5070",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5070 @ 3.5 GHz",
    ],
    [
        3730000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Dempsey",
        "5080",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5080 @ 3.7 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "511",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 511 @ 2.8 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Woodcrest",
        "5110",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5110 @ 1.6 GHz",
    ],
    [
        1050000000,
        None,
        "Intel Xeon Phi x100 Product Family",
        0,
        2012.0,
        "Knights Corner",
        "5110P",
        60,
        60,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x100 Product Family 5110P @ 1.1 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Woodcrest",
        "5113",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5113 @ 1.6 GHz",
    ],
    [
        2400000000,
        3200000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "5115",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 5115 @ 2.4 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "5117",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 5117 @ 2.0 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "5117F",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 5117F @ 2.0 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "5118",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 5118 @ 2.3 GHz",
    ],
    [
        1900000000,
        3200000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "5119T",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 5119T @ 1.9 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Woodcrest",
        "5120",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5120 @ 1.9 GHz",
    ],
    [
        1050000000,
        None,
        "Intel Xeon Phi x100 Product Family",
        0,
        2013.0,
        "Knights Corner",
        "5120D",
        60,
        60,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x100 Product Family 5120D @ 1.1 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "5120T",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 5120T @ 2.2 GHz",
    ],
    [
        3600000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "5122",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 5122 @ 3.6 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2006.0,
        "Woodcrest",
        "5128",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5128 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2006.0,
        "Woodcrest",
        "5130",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5130 @ 2.0 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Woodcrest",
        "5133",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5133 @ 2.2 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2006.0,
        "Woodcrest",
        "5138",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5138 @ 2.1 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2006.0,
        "Woodcrest",
        "5140",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5140 @ 2.3 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2006.0,
        "Woodcrest",
        "5148",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5148 @ 2.3 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "515",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 515 @ 2.9 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Woodcrest",
        "5150",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5150 @ 2.7 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "516",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 516 @ 2.9 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Woodcrest",
        "5160",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 5160 @ 3.0 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "517",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 517 @ 2.9 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "518",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 518 @ 2.8 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "519",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 519 @ 3.1 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2004.0,
        "Prescott",
        "519K",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 519K @ 3.1 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "520",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 520 @ 2.8 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "520J",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 520J @ 2.8 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "521",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 521 @ 2.8 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5215",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5215 @ 2.5 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5215L",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5215L @ 2.5 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5215M",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5215M @ 2.5 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5217",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5217 @ 3.0 GHz",
    ],
    [
        2300000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5218",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5218 @ 2.3 GHz",
    ],
    [
        2300000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5218B",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5218B @ 2.3 GHz",
    ],
    [
        2300000000,
        3700000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5218N",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5218N @ 2.3 GHz",
    ],
    [
        2100000000,
        3800000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5218T",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5218T @ 2.1 GHz",
    ],
    [
        2400000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5219Y",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5219Y @ 2.4 GHz",
    ],
    [
        2200000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5220",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5220 @ 2.2 GHz",
    ],
    [
        2700000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5220S",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5220S @ 2.7 GHz",
    ],
    [
        1900000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5220T",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5220T @ 1.9 GHz",
    ],
    [
        3800000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "5222",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 5222 @ 3.8 GHz",
    ],
    [
        930000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2007.0,
        "Merom",
        "523",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 523 @ 0.9 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Prescott",
        "524",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 524 @ 3.1 GHz",
    ],
    [
        1730000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2007.0,
        "Merom",
        "530",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 530 @ 1.7 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2004.0,
        "Prescott",
        "530J",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 530J @ 3.0 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2004.0,
        "Prescott",
        "531",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 531 @ 3.0 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "532",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 532 @ 3.1 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "538",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 538 @ 3.2 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2007.0,
        "Merom",
        "540",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 540 @ 1.9 GHz",
    ],
    [
        2300000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2019.0,
        "Whiskey Lake",
        "5405U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series 5405U @ 2.3 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "540J",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 540J @ 3.2 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2004.0,
        "Prescott",
        "541",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 541 @ 3.2 GHz",
    ],
    [
        3330000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "548",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 548 @ 3.3 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2006.0,
        "Merom",
        "550",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 550 @ 2.0 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "550J",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 550J @ 3.4 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "551",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 551 @ 3.4 GHz",
    ],
    [
        3460000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "552",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 552 @ 3.5 GHz",
    ],
    [
        3600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "560",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 560 @ 3.6 GHz",
    ],
    [
        3600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "560J",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 560J @ 3.6 GHz",
    ],
    [
        3600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "561",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 561 @ 3.6 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2008.0,
        "Merom",
        "570",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 570 @ 2.3 GHz",
    ],
    [
        3800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "571",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 571 @ 3.8 GHz",
    ],
    [
        1000000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2006.0,
        "Merom",
        "573",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 573 @ 1.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2006.0,
        "Merom",
        "575",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 575 @ 2.0 GHz",
    ],
    [
        2160000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2008.0,
        "Merom",
        "585",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 585 @ 2.2 GHz",
    ],
    [
        800000000,
        2000000000.0,
        "5th Generation Intel Core M Processors",
        3,
        2014.0,
        "Broadwell",
        "5Y10",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core M Processors 5Y10 @ 0.8 GHz",
    ],
    [
        800000000,
        2000000000.0,
        "5th Generation Intel Core M Processors",
        3,
        2014.0,
        "Broadwell",
        "5Y10a",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core M Processors 5Y10a @ 0.8 GHz",
    ],
    [
        800000000,
        2000000000.0,
        "5th Generation Intel Core M Processors",
        3,
        2014.0,
        "Broadwell",
        "5Y10c",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core M Processors 5Y10c @ 0.8 GHz",
    ],
    [
        900000000,
        2400000000.0,
        "5th Generation Intel Core M Processors",
        3,
        2014.0,
        "Broadwell",
        "5Y31",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core M Processors 5Y31 @ 0.9 GHz",
    ],
    [
        1100000000,
        2600000000.0,
        "5th Generation Intel Core M Processors",
        3,
        2014.0,
        "Broadwell",
        "5Y51",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core M Processors 5Y51 @ 1.1 GHz",
    ],
    [
        1100000000,
        2600000000.0,
        "5th Generation Intel Core M Processors",
        3,
        2014.0,
        "Broadwell",
        "5Y70",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core M Processors 5Y70 @ 1.1 GHz",
    ],
    [
        1200000000,
        2900000000.0,
        "5th Generation Intel Core M Processors",
        3,
        2014.0,
        "Broadwell",
        "5Y71",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core M Processors 5Y71 @ 1.2 GHz",
    ],
    [
        1800000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2018.0,
        "Skylake",
        "6122",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6122 @ 1.8 GHz",
    ],
    [
        2600000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6126",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6126 @ 2.6 GHz",
    ],
    [
        2600000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6126F",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6126F @ 2.6 GHz",
    ],
    [
        2600000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6126T",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6126T @ 2.6 GHz",
    ],
    [
        3400000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6128",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6128 @ 3.4 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2018.0,
        "Skylake",
        "6129",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6129 @ 2.3 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6130",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6130 @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6130F",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6130F @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        0,
        2018.0,
        "Skylake",
        "6130H",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6130H @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6130T",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6130T @ 2.1 GHz",
    ],
    [
        2600000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6132",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6132 @ 2.6 GHz",
    ],
    [
        3200000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6134",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6134 @ 3.2 GHz",
    ],
    [
        3200000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6134M",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6134M @ 3.2 GHz",
    ],
    [
        3400000000,
        4200000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6135",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6135 @ 3.4 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6136",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6136 @ 3.0 GHz",
    ],
    [
        3900000000,
        4099999999.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6137",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6137 @ 3.9 GHz",
    ],
    [
        2000000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6138",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6138 @ 2.0 GHz",
    ],
    [
        2000000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6138F",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6138F @ 2.0 GHz",
    ],
    [
        2000000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2018.0,
        "Skylake",
        "6138P",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6138P @ 2.0 GHz",
    ],
    [
        2000000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6138T",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6138T @ 2.0 GHz",
    ],
    [
        2300000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6140",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6140 @ 2.3 GHz",
    ],
    [
        2300000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6140M",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6140M @ 2.3 GHz",
    ],
    [
        2600000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6142",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6142 @ 2.6 GHz",
    ],
    [
        2600000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6142F",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6142F @ 2.6 GHz",
    ],
    [
        2600000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6142M",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6142M @ 2.6 GHz",
    ],
    [
        2800000000,
        4000000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6143",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6143 @ 2.8 GHz",
    ],
    [
        3500000000,
        4200000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6144",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6144 @ 3.5 GHz",
    ],
    [
        3200000000,
        4200000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6146",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6146 @ 3.2 GHz",
    ],
    [
        2400000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6148",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6148 @ 2.4 GHz",
    ],
    [
        2400000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6148F",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6148F @ 2.4 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6150",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6150 @ 2.7 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6152",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6152 @ 2.1 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "6154",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6154 @ 3.0 GHz",
    ],
    [
        1900000000,
        3500000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2018.0,
        "Skylake",
        "6162",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 6162 @ 1.9 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Prescott",
        "620",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 620 @ 2.8 GHz",
    ],
    [
        2100000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6209U",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6209U @ 2.1 GHz",
    ],
    [
        2500000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6210U",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6210U @ 2.5 GHz",
    ],
    [
        2400000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6212U",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6212U @ 2.4 GHz",
    ],
    [
        1800000000,
        3600000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6222",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6222 @ 1.8 GHz",
    ],
    [
        1800000000,
        3600000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6222V",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6222V @ 1.8 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6226",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6226 @ 2.7 GHz",
    ],
    [
        2100000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6230",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6230 @ 2.1 GHz",
    ],
    [
        2300000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6230N",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6230N @ 2.3 GHz",
    ],
    [
        2100000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6230T",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6230T @ 2.1 GHz",
    ],
    [
        3300000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6234",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6234 @ 3.3 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6238",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6238 @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6238L",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6238L @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6238M",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6238M @ 2.1 GHz",
    ],
    [
        1900000000,
        3700000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6238T",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6238T @ 1.9 GHz",
    ],
    [
        2600000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6240",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6240 @ 2.6 GHz",
    ],
    [
        2600000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6240L",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6240L @ 2.6 GHz",
    ],
    [
        2600000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6240M",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6240M @ 2.6 GHz",
    ],
    [
        2600000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6240Y",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6240Y @ 2.6 GHz",
    ],
    [
        2800000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6242",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6242 @ 2.8 GHz",
    ],
    [
        3600000000,
        4400000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6244",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6244 @ 3.6 GHz",
    ],
    [
        3300000000,
        4200000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6246",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6246 @ 3.3 GHz",
    ],
    [
        2500000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6248",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6248 @ 2.5 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6252",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6252 @ 2.1 GHz",
    ],
    [
        2300000000,
        3600000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6252N",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6252N @ 2.3 GHz",
    ],
    [
        3100000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6254",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6254 @ 3.1 GHz",
    ],
    [
        1900000000,
        3600000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6262",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6262 @ 1.9 GHz",
    ],
    [
        1900000000,
        3600000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6262V",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6262V @ 1.9 GHz",
    ],
    [
        3200000000,
        3700000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "6269Y",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 6269Y @ 3.2 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "630",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 630 @ 3.0 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Cedarmill",
        "631",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 631 @ 3.0 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "640",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 640 @ 3.2 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Cedarmill",
        "641",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 641 @ 3.2 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "650",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 650 @ 3.4 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Cedarmill",
        "651",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 651 @ 3.4 GHz",
    ],
    [
        3600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "660",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 660 @ 3.6 GHz",
    ],
    [
        3600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Cedarmill",
        "661",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 661 @ 3.6 GHz",
    ],
    [
        3600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "662",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 662 @ 3.6 GHz",
    ],
    [
        3800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "670",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 670 @ 3.8 GHz",
    ],
    [
        3800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Prescott",
        "672",
        1,
        1,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 672 @ 3.8 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Paxville",
        "7020",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7020 @ 2.7 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Paxville",
        "7030",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7030 @ 2.8 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Paxville",
        "7040",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7040 @ 3.0 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Paxville",
        "7041",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7041 @ 3.0 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Banias",
        "705",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 705 @ 1.5 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Tulsa",
        "7110M",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7110M @ 2.6 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Tulsa",
        "7110N",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7110N @ 2.5 GHz",
    ],
    [
        1240000000,
        1330000000.0,
        "Intel Xeon Phi x100 Product Family",
        0,
        2014.0,
        "Knights Corner",
        "7120A",
        61,
        61,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x100 Product Family 7120A @ 1.2 GHz",
    ],
    [
        1240000000,
        1330000000.0,
        "Intel Xeon Phi x100 Product Family",
        0,
        2014.0,
        "Knights Corner",
        "7120D",
        61,
        61,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x100 Product Family 7120D @ 1.2 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Tulsa",
        "7120M",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7120M @ 3.0 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Tulsa",
        "7120N",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7120N @ 3.0 GHz",
    ],
    [
        1240000000,
        1330000000.0,
        "Intel Xeon Phi x100 Product Family",
        0,
        2013.0,
        "Knights Corner",
        "7120P",
        61,
        61,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x100 Product Family 7120P @ 1.2 GHz",
    ],
    [
        1240000000,
        1330000000.0,
        "Intel Xeon Phi x100 Product Family",
        0,
        2013.0,
        "Knights Corner",
        "7120X",
        61,
        61,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x100 Product Family 7120X @ 1.2 GHz",
    ],
    [
        1100000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Banias",
        "713",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 713 @ 1.1 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Tulsa",
        "7130M",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7130M @ 3.2 GHz",
    ],
    [
        3160000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Tulsa",
        "7130N",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7130N @ 3.2 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Tulsa",
        "7140M",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7140M @ 3.4 GHz",
    ],
    [
        3330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Tulsa",
        "7140N",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7140N @ 3.3 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "715",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 715 @ 1.5 GHz",
    ],
    [
        3500000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Tulsa",
        "7150N",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors 7150N @ 3.5 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Banias",
        "718",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 718 @ 1.3 GHz",
    ],
    [
        1300000000,
        1500000000.0,
        "Intel Xeon Phi x200 Product Family",
        4,
        2016.0,
        "Knights Landing",
        "7210",
        64,
        64,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x200 Product Family 7210 @ 1.3 GHz",
    ],
    [
        1300000000,
        1500000000.0,
        "Intel Xeon Phi x200 Product Family",
        4,
        2016.0,
        "Knights Landing",
        "7210F",
        64,
        64,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x200 Product Family 7210F @ 1.3 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2008.0,
        "Penryn",
        "722",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 722 @ 1.2 GHz",
    ],
    [
        1000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "723",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 723 @ 1.0 GHz",
    ],
    [
        1300000000,
        1500000000.0,
        "Intel Xeon Phi x200 Product Family",
        4,
        2016.0,
        "Knights Landing",
        "7230",
        64,
        64,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x200 Product Family 7230 @ 1.3 GHz",
    ],
    [
        1300000000,
        1500000000.0,
        "Intel Xeon Phi x200 Product Family",
        4,
        2016.0,
        "Knights Landing",
        "7230F",
        64,
        64,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x200 Product Family 7230F @ 1.3 GHz",
    ],
    [
        1300000000,
        1400000000.0,
        "Intel Xeon Phi 72x5 Processor Family",
        4,
        2017.0,
        "Knights Mill",
        "7235",
        64,
        64,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi 72x5 Processor Family 7235 @ 1.3 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2004.0,
        "Dothan",
        "725",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 725 @ 1.6 GHz",
    ],
    [
        1400000000,
        1600000000.0,
        "Intel Xeon Phi x200 Product Family",
        4,
        2016.0,
        "Knights Landing",
        "7250",
        68,
        68,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x200 Product Family 7250 @ 1.4 GHz",
    ],
    [
        1400000000,
        1600000000.0,
        "Intel Xeon Phi x200 Product Family",
        4,
        2016.0,
        "Knights Landing",
        "7250F",
        68,
        68,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x200 Product Family 7250F @ 1.4 GHz",
    ],
    [
        1100000000,
        1200000000.0,
        "Intel Xeon Phi 72x5 Processor Family",
        4,
        2017.0,
        "Knights Mill",
        "7255",
        68,
        68,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi 72x5 Processor Family 7255 @ 1.1 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Celeron Processor",
        2,
        2012.0,
        "Gladden",
        "725C",
        1,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 725C @ 1.3 GHz",
    ],
    [
        1300000000,
        1400000000.0,
        "Intel Xeon Phi 72x5 Processor Family",
        4,
        2017.0,
        "Knights Mill",
        "7285",
        68,
        68,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi 72x5 Processor Family 7285 @ 1.3 GHz",
    ],
    [
        1500000000,
        1700000000.0,
        "Intel Xeon Phi x200 Product Family",
        4,
        2016.0,
        "Knights Landing",
        "7290",
        72,
        72,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x200 Product Family 7290 @ 1.5 GHz",
    ],
    [
        1500000000,
        1700000000.0,
        "Intel Xeon Phi x200 Product Family",
        4,
        2016.0,
        "Knights Landing",
        "7290F",
        72,
        72,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi x200 Product Family 7290F @ 1.5 GHz",
    ],
    [
        1500000000,
        1600000000.0,
        "Intel Xeon Phi 72x5 Processor Family",
        4,
        2017.0,
        "Knights Mill",
        "7295",
        72,
        72,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Phi 72x5 Processor Family 7295 @ 1.5 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "730",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 730 @ 1.6 GHz",
    ],
    [
        1100000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "733",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 733 @ 1.1 GHz",
    ],
    [
        1700000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "735",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 735 @ 1.7 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2004.0,
        "Dothan",
        "738",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 738 @ 1.4 GHz",
    ],
    [
        1730000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "740",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 740 @ 1.7 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2009.0,
        "Penryn",
        "743",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 743 @ 1.3 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2004.0,
        "Dothan",
        "745",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 745 @ 1.8 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2004.0,
        "Dothan",
        "745A",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 745A @ 1.8 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "750",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 750 @ 1.9 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "753",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 753 @ 1.2 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "755",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 755 @ 2.0 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "758",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 758 @ 1.5 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2004.0,
        "Dothan",
        "760",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 760 @ 2.0 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2011.0,
        "Penryn",
        "763",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 763 @ 1.4 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "765",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 765 @ 2.1 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "770",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 770 @ 2.1 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "773",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 773 @ 1.3 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "778",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 778 @ 1.6 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Dothan",
        "780",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 780 @ 2.3 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "787",
        1,
        "1",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 787 @ 1.3 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "797",
        1,
        "1",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 797 @ 1.4 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Smithfield",
        "805",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 805 @ 2.7 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "807",
        1,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 807 @ 1.5 GHz",
    ],
    [
        1000000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2011.0,
        "Sandy Bridge",
        "807UE",
        1,
        "1",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 807UE @ 1.0 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8153",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8153 @ 2.0 GHz",
    ],
    [
        3600000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8156",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8156 @ 3.6 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8158",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8158 @ 3.0 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8160",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8160 @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8160F",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8160F @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        0,
        2018.0,
        "Skylake",
        "8160H",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8160H @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8160M",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8160M @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8160T",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8160T @ 2.1 GHz",
    ],
    [
        2000000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8164",
        26,
        "52",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8164 @ 2.0 GHz",
    ],
    [
        2300000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8165",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8165 @ 2.3 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8168",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8168 @ 2.7 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8170",
        26,
        "52",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8170 @ 2.1 GHz",
    ],
    [
        2100000000,
        3700000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8170M",
        26,
        "52",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8170M @ 2.1 GHz",
    ],
    [
        3100000000,
        3900000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8174",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8174 @ 3.1 GHz",
    ],
    [
        2100000000,
        3800000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8176",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8176 @ 2.1 GHz",
    ],
    [
        2100000000,
        3800000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8176F",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8176F @ 2.1 GHz",
    ],
    [
        2100000000,
        3800000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8176M",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8176M @ 2.1 GHz",
    ],
    [
        2500000000,
        3800000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8180",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8180 @ 2.5 GHz",
    ],
    [
        2500000000,
        3800000000.0,
        "Intel Xeon Scalable Processors",
        4,
        2017.0,
        "Skylake",
        "8180M",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Scalable Processors 8180M @ 2.5 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Smithfield",
        "820",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 820 @ 2.8 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8253",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8253 @ 2.2 GHz",
    ],
    [
        3800000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8256",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8256 @ 3.8 GHz",
    ],
    [
        2400000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8260",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8260 @ 2.4 GHz",
    ],
    [
        2400000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8260L",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8260L @ 2.4 GHz",
    ],
    [
        2400000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8260M",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8260M @ 2.4 GHz",
    ],
    [
        2400000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8260Y",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8260Y @ 2.4 GHz",
    ],
    [
        2900000000,
        3900000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8268",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8268 @ 2.9 GHz",
    ],
    [
        2700000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8270",
        26,
        "52",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8270 @ 2.7 GHz",
    ],
    [
        3200000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8274",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8274 @ 3.2 GHz",
    ],
    [
        2200000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8276",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8276 @ 2.2 GHz",
    ],
    [
        2200000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8276L",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8276L @ 2.2 GHz",
    ],
    [
        2200000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8276M",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8276M @ 2.2 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2011.0,
        "Sandy Bridge",
        "827E",
        1,
        "1",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 827E @ 1.4 GHz",
    ],
    [
        2700000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8280",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8280 @ 2.7 GHz",
    ],
    [
        2700000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8280L",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8280L @ 2.7 GHz",
    ],
    [
        2700000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8280M",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8280M @ 2.7 GHz",
    ],
    [
        3000000000,
        4000000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "8284",
        28,
        "56",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 8284 @ 3.0 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Smithfield",
        "830",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 830 @ 3.0 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2005.0,
        "Smithfield",
        "840",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 840 @ 3.2 GHz",
    ],
    [
        1100000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "847",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 847 @ 1.1 GHz",
    ],
    [
        1100000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2011.0,
        "Sandy Bridge",
        "847E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 847E @ 1.1 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "857",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 857 @ 1.2 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "867",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 867 @ 1.3 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "877",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 877 @ 1.4 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "887",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 887 @ 1.5 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2009.0,
        "Penryn",
        "900",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 900 @ 2.2 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Itanium Processor 9000 Series",
        0,
        None,
        "Montecito",
        "9010",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9000 Series 9010 @ 1.6 GHz",
    ],
    [
        1400000000,
        None,
        "Intel Itanium Processor 9000 Series",
        0,
        2007.0,
        "Montecito",
        "9015",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9000 Series 9015 @ 1.4 GHz",
    ],
    [
        1420000000,
        None,
        "Intel Itanium Processor 9000 Series",
        0,
        None,
        "Montecito",
        "9020",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9000 Series 9020 @ 1.4 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Itanium Processor 9000 Series",
        0,
        None,
        "Montecito",
        "9030",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9000 Series 9030 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Itanium Processor 9000 Series",
        0,
        None,
        "Montecito",
        "9040",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9000 Series 9040 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Itanium Processor 9000 Series",
        0,
        None,
        "Montecito",
        "9050",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9000 Series 9050 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Itanium Processor 9100 Series",
        0,
        2007.0,
        "Montvale",
        "9110N",
        1,
        1,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9100 Series 9110N @ 1.6 GHz",
    ],
    [
        1420000000,
        None,
        "Intel Itanium Processor 9100 Series",
        0,
        2007.0,
        "Montvale",
        "9120N",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9100 Series 9120N @ 1.4 GHz",
    ],
    [
        1660000000,
        None,
        "Intel Itanium Processor 9100 Series",
        0,
        2007.0,
        "Montvale",
        "9130M",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9100 Series 9130M @ 1.7 GHz",
    ],
    [
        1660000000,
        None,
        "Intel Itanium Processor 9100 Series",
        0,
        2007.0,
        "Montvale",
        "9140M",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9100 Series 9140M @ 1.7 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Itanium Processor 9100 Series",
        0,
        2007.0,
        "Montvale",
        "9140N",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9100 Series 9140N @ 1.6 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Presler",
        "915",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 915 @ 2.8 GHz",
    ],
    [
        1660000000,
        None,
        "Intel Itanium Processor 9100 Series",
        0,
        2007.0,
        "Montvale",
        "9150M",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9100 Series 9150M @ 1.7 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Itanium Processor 9100 Series",
        0,
        2007.0,
        "Montvale",
        "9150N",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9100 Series 9150N @ 1.6 GHz",
    ],
    [
        1660000000,
        None,
        "Intel Itanium Processor 9100 Series",
        0,
        2007.0,
        "Montvale",
        "9152M",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9100 Series 9152M @ 1.7 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Presler",
        "920",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 920 @ 2.8 GHz",
    ],
    [
        2300000000,
        3800000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "9242",
        48,
        "96",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 9242 @ 2.3 GHz",
    ],
    [
        2300000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2011.0,
        "Penryn",
        "925",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 925 @ 2.3 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2013.0,
        "Ivy Bridge",
        "927UE",
        1,
        "1",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor 927UE @ 1.5 GHz",
    ],
    [
        2600000000,
        3800000000.0,
        "2nd Generation Intel Xeon Scalable Processors",
        4,
        2019.0,
        "Cascade Lake",
        "9282",
        56,
        "112",
        "Server",
        "cpu",
        "intel",
        " 2nd Generation Intel Xeon Scalable Processors 9282 @ 2.6 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Presler",
        "930",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 930 @ 3.0 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Itanium Processor 9300 Series",
        0,
        2010.0,
        "Tukwila",
        "9310",
        2,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9300 Series 9310 @ 1.6 GHz",
    ],
    [
        1330000000,
        1470000000.0,
        "Intel Itanium Processor 9300 Series",
        0,
        2010.0,
        "Tukwila",
        "9320",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9300 Series 9320 @ 1.3 GHz",
    ],
    [
        1460000000,
        1600000000.0,
        "Intel Itanium Processor 9300 Series",
        0,
        2010.0,
        "Tukwila",
        "9330",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9300 Series 9330 @ 1.5 GHz",
    ],
    [
        1600000000,
        1730000000.0,
        "Intel Itanium Processor 9300 Series",
        0,
        2010.0,
        "Tukwila",
        "9340",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9300 Series 9340 @ 1.6 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2007.0,
        "Presler",
        "935",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 935 @ 3.2 GHz",
    ],
    [
        1730000000,
        1870000000.0,
        "Intel Itanium Processor 9300 Series",
        0,
        2010.0,
        "Tukwila",
        "9350",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9300 Series 9350 @ 1.7 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Presler",
        "940",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 940 @ 3.2 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Presler",
        "945",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 945 @ 3.4 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Presler",
        "950",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 950 @ 3.4 GHz",
    ],
    [
        1730000000,
        None,
        "Intel Itanium Processor 9500 Series",
        0,
        2012.0,
        "Poulson",
        "9520",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9500 Series 9520 @ 1.7 GHz",
    ],
    [
        2130000000,
        None,
        "Intel Itanium Processor 9500 Series",
        0,
        2012.0,
        "Poulson",
        "9540",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9500 Series 9540 @ 2.1 GHz",
    ],
    [
        3460000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Presler",
        "955",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 955 @ 3.5 GHz",
    ],
    [
        2400000000,
        None,
        "Intel Itanium Processor 9500 Series",
        0,
        2012.0,
        "Poulson",
        "9550",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9500 Series 9550 @ 2.4 GHz",
    ],
    [
        2530000000,
        None,
        "Intel Itanium Processor 9500 Series",
        0,
        2012.0,
        "Poulson",
        "9560",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9500 Series 9560 @ 2.5 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "957",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 957 @ 1.2 GHz",
    ],
    [
        3600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Presler",
        "960",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 960 @ 3.6 GHz",
    ],
    [
        3730000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Presler",
        "965",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 965 @ 3.7 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "967",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 967 @ 1.3 GHz",
    ],
    [
        1730000000,
        None,
        "Intel Itanium Processor 9700 Series",
        0,
        2017.0,
        "Kittson",
        "9720",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9700 Series 9720 @ 1.7 GHz",
    ],
    [
        2130000000,
        None,
        "Intel Itanium Processor 9700 Series",
        0,
        2017.0,
        "Kittson",
        "9740",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9700 Series 9740 @ 2.1 GHz",
    ],
    [
        2530000000,
        None,
        "Intel Itanium Processor 9700 Series",
        0,
        2017.0,
        "Kittson",
        "9750",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9700 Series 9750 @ 2.5 GHz",
    ],
    [
        2660000000,
        None,
        "Intel Itanium Processor 9700 Series",
        0,
        2017.0,
        "Kittson",
        "9760",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Itanium Processor 9700 Series 9760 @ 2.7 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "977",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 977 @ 1.4 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "987",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 987 @ 1.5 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "997",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor 997 @ 1.6 GHz",
    ],
    [
        2100000000,
        None,
        "Intel Pentium Processor 1000 Series",
        1,
        2013.0,
        "Ivy Bridge",
        "A1018",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 1000 Series A1018 @ 2.1 GHz",
    ],
    [
        2410000000,
        None,
        "Intel Pentium Processor 1000 Series",
        0,
        2016.0,
        "Bay Trail",
        "A1020",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor 1000 Series A1020 @ 2.4 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "B710",
        1,
        "1",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor B710 @ 1.6 GHz",
    ],
    [
        1700000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "B720",
        1,
        "1",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor B720 @ 1.7 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "B800",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor B800 @ 1.5 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "B810",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor B810 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2011.0,
        "Sandy Bridge",
        "B810E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor B810E @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "B815",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor B815 @ 1.6 GHz",
    ],
    [
        1700000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "B820",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor B820 @ 1.7 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "B830",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor B830 @ 1.8 GHz",
    ],
    [
        1900000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2011.0,
        "Sandy Bridge",
        "B840",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor B840 @ 1.9 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Pentium Processor",
        2,
        2012.0,
        "Gladden",
        "B915C",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor B915C @ 1.5 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2013.0,
        "Gladden",
        "B925C",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor B925C @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "B940",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor B940 @ 2.0 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "B950",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor B950 @ 2.1 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "B960",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor B960 @ 2.2 GHz",
    ],
    [
        2300000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "B970",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor B970 @ 2.3 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "B980",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor B980 @ 2.4 GHz",
    ],
    [
        1600000000,
        2200000000.0,
        "Intel Xeon D Processor",
        3,
        2017.0,
        "Broadwell",
        "D-1513N",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1513N @ 1.6 GHz",
    ],
    [
        2200000000,
        2200000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1518",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1518 @ 2.2 GHz",
    ],
    [
        2200000000,
        2600000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1520",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1520 @ 2.2 GHz",
    ],
    [
        2400000000,
        2700000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1521",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1521 @ 2.4 GHz",
    ],
    [
        2000000000,
        2600000000.0,
        "Intel Xeon D Processor",
        3,
        2017.0,
        "Broadwell",
        "D-1523N",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1523N @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Xeon D Processor",
        0,
        2018.0,
        "Broadwell",
        "D-1524N",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1524N @ 2.0 GHz",
    ],
    [
        2200000000,
        2700000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1527",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1527 @ 2.2 GHz",
    ],
    [
        1900000000,
        2500000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1528",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1528 @ 1.9 GHz",
    ],
    [
        1300000000,
        1300000000.0,
        "Intel Xeon D Processor",
        3,
        2016.0,
        "Broadwell",
        "D-1529",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1529 @ 1.3 GHz",
    ],
    [
        2200000000,
        2700000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1531",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1531 @ 2.2 GHz",
    ],
    [
        2100000000,
        2700000000.0,
        "Intel Xeon D Processor",
        3,
        2017.0,
        "Broadwell",
        "D-1533N",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1533N @ 2.1 GHz",
    ],
    [
        1700000000,
        2300000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1537",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1537 @ 1.7 GHz",
    ],
    [
        1600000000,
        2200000000.0,
        "Intel Xeon D Processor",
        3,
        2016.0,
        "Broadwell",
        "D-1539",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1539 @ 1.6 GHz",
    ],
    [
        2000000000,
        2600000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1540",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1540 @ 2.0 GHz",
    ],
    [
        2100000000,
        2700000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1541",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1541 @ 2.1 GHz",
    ],
    [
        1900000000,
        2500000000.0,
        "Intel Xeon D Processor",
        3,
        2017.0,
        "Broadwell",
        "D-1543N",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1543N @ 1.9 GHz",
    ],
    [
        2000000000,
        2600000000.0,
        "Intel Xeon D Processor",
        3,
        2015.0,
        "Broadwell",
        "D-1548",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1548 @ 2.0 GHz",
    ],
    [
        2300000000,
        2700000000.0,
        "Intel Xeon D Processor",
        3,
        2017.0,
        "Broadwell",
        "D-1553N",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1553N @ 2.3 GHz",
    ],
    [
        1500000000,
        2100000000.0,
        "Intel Xeon D Processor",
        3,
        2016.0,
        "Broadwell",
        "D-1557",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1557 @ 1.5 GHz",
    ],
    [
        1500000000,
        2100000000.0,
        "Intel Xeon D Processor",
        3,
        2016.0,
        "Broadwell",
        "D-1559",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1559 @ 1.5 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Xeon D Processor",
        0,
        2018.0,
        "Broadwell",
        "D-1563N",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1563N @ 2.0 GHz",
    ],
    [
        2100000000,
        2700000000.0,
        "Intel Xeon D Processor",
        3,
        2016.0,
        "Broadwell",
        "D-1567",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1567 @ 2.1 GHz",
    ],
    [
        1300000000,
        2100000000.0,
        "Intel Xeon D Processor",
        3,
        2016.0,
        "Broadwell",
        "D-1571",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1571 @ 1.3 GHz",
    ],
    [
        2700000000,
        None,
        "Intel Xeon D Processor",
        0,
        2018.0,
        "Broadwell",
        "D-1573N",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1573N @ 2.7 GHz",
    ],
    [
        1300000000,
        2100000000.0,
        "Intel Xeon D Processor",
        3,
        2016.0,
        "Broadwell",
        "D-1577",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1577 @ 1.3 GHz",
    ],
    [
        1800000000,
        2400000000.0,
        "Intel Xeon D Processor",
        3,
        2016.0,
        "Broadwell",
        "D-1581",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1581 @ 1.8 GHz",
    ],
    [
        1700000000,
        2300000000.0,
        "Intel Xeon D Processor",
        3,
        2016.0,
        "Broadwell",
        "D-1587",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1587 @ 1.7 GHz",
    ],
    [
        2500000000,
        3200000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1602",
        2,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1602 @ 2.5 GHz",
    ],
    [
        1500000000,
        2500000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1612",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1612 @ 1.5 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1622",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1622 @ 2.6 GHz",
    ],
    [
        2400000000,
        3200000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1623N",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1623N @ 2.4 GHz",
    ],
    [
        2900000000,
        3200000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1627",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1627 @ 2.9 GHz",
    ],
    [
        1500000000,
        2500000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1632",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1632 @ 1.5 GHz",
    ],
    [
        2500000000,
        3200000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1633N",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1633N @ 2.5 GHz",
    ],
    [
        2900000000,
        3200000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1637",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1637 @ 2.9 GHz",
    ],
    [
        2300000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1649N",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1649N @ 2.3 GHz",
    ],
    [
        2800000000,
        3200000000.0,
        "Intel Xeon D Processor",
        3,
        2019.0,
        "Hewitt Lake",
        "D-1653N",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-1653N @ 2.8 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2123IT",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2123IT @ 2.2 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2141I",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2141I @ 2.2 GHz",
    ],
    [
        1900000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2142IT",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2142IT @ 1.9 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2143IT",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2143IT @ 2.2 GHz",
    ],
    [
        1900000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2145NT",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2145NT @ 1.9 GHz",
    ],
    [
        2300000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2146NT",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2146NT @ 2.3 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2161I",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2161I @ 2.2 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2163IT",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2163IT @ 2.1 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2166NT",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2166NT @ 2.0 GHz",
    ],
    [
        1700000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2173IT",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2173IT @ 1.7 GHz",
    ],
    [
        1900000000,
        3000000000.0,
        "Intel Xeon D Processor",
        4,
        2018.0,
        "Skylake",
        "D-2177NT",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2177NT @ 1.9 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2183IT",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2183IT @ 2.2 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2187NT",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2187NT @ 2.0 GHz",
    ],
    [
        1600000000,
        3000000000.0,
        "Intel Xeon D Processor",
        3,
        2018.0,
        "Skylake",
        "D-2191",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon D Processor D-2191 @ 1.6 GHz",
    ],
    [
        1200000000,
        None,
        "Intel Pentium Processor D Series",
        3,
        2015.0,
        "Broadwell",
        "D1507",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Intel Pentium Processor D Series D1507 @ 1.2 GHz",
    ],
    [
        2200000000,
        2600000000.0,
        "Intel Pentium Processor D Series",
        3,
        2015.0,
        "Broadwell",
        "D1508",
        2,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Pentium Processor D Series D1508 @ 2.2 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Pentium Processor D Series",
        3,
        2015.0,
        "Broadwell",
        "D1509",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Intel Pentium Processor D Series D1509 @ 1.5 GHz",
    ],
    [
        1600000000,
        2200000000.0,
        "Intel Pentium Processor D Series",
        3,
        2015.0,
        "Broadwell",
        "D1517",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Pentium Processor D Series D1517 @ 1.6 GHz",
    ],
    [
        1500000000,
        2100000000.0,
        "Intel Pentium Processor D Series",
        3,
        2016.0,
        "Broadwell",
        "D1519",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Pentium Processor D Series D1519 @ 1.5 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2104G",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2104G @ 3.2 GHz",
    ],
    [
        3300000000,
        4300000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2124",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2124 @ 3.3 GHz",
    ],
    [
        3400000000,
        4500000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2124G",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2124G @ 3.4 GHz",
    ],
    [
        3300000000,
        4500000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2126G",
        6,
        "6",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2126G @ 3.3 GHz",
    ],
    [
        3500000000,
        4500000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2134",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2134 @ 3.5 GHz",
    ],
    [
        3300000000,
        4500000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2136",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2136 @ 3.3 GHz",
    ],
    [
        3600000000,
        4500000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2144G",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2144G @ 3.6 GHz",
    ],
    [
        3500000000,
        4500000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2146G",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2146G @ 3.5 GHz",
    ],
    [
        3800000000,
        4700000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2174G",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2174G @ 3.8 GHz",
    ],
    [
        3700000000,
        4700000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2176G",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2176G @ 3.7 GHz",
    ],
    [
        2700000000,
        4400000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2176M",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2176M @ 2.7 GHz",
    ],
    [
        3800000000,
        4700000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2186G",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2186G @ 3.8 GHz",
    ],
    [
        2900000000,
        4800000000.0,
        "Intel Xeon E Processor",
        3,
        2018.0,
        "Coffee Lake",
        "E-2186M",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2186M @ 2.9 GHz",
    ],
    [
        3400000000,
        4600000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2224",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2224 @ 3.4 GHz",
    ],
    [
        3500000000,
        4700000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2224G",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2224G @ 3.5 GHz",
    ],
    [
        3400000000,
        4700000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2226G",
        6,
        "6",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2226G @ 3.4 GHz",
    ],
    [
        3400000000,
        4600000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2226GE",
        6,
        "6",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2226GE @ 3.4 GHz",
    ],
    [
        3600000000,
        4800000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2234",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2234 @ 3.6 GHz",
    ],
    [
        3400000000,
        4800000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2236",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2236 @ 3.4 GHz",
    ],
    [
        3800000000,
        4800000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2244G",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2244G @ 3.8 GHz",
    ],
    [
        3600000000,
        4800000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2246G",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2246G @ 3.6 GHz",
    ],
    [
        2600000000,
        3800000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2254ME",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2254ME @ 2.6 GHz",
    ],
    [
        1700000000,
        3500000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2254ML",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2254ML @ 1.7 GHz",
    ],
    [
        4000000000,
        4900000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2274G",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2274G @ 4.0 GHz",
    ],
    [
        3800000000,
        4900000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2276G",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2276G @ 3.8 GHz",
    ],
    [
        2800000000,
        4700000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2276M",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2276M @ 2.8 GHz",
    ],
    [
        2800000000,
        4500000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2276ME",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2276ME @ 2.8 GHz",
    ],
    [
        2000000000,
        4200000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2276ML",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2276ML @ 2.0 GHz",
    ],
    [
        3400000000,
        5000000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2278G",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2278G @ 3.4 GHz",
    ],
    [
        3300000000,
        4700000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2278GE",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2278GE @ 3.3 GHz",
    ],
    [
        2000000000,
        3900000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2278GEL",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2278GEL @ 2.0 GHz",
    ],
    [
        4000000000,
        4900000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2286G",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2286G @ 4.0 GHz",
    ],
    [
        2400000000,
        5000000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2286M",
        8,
        "16",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2286M @ 2.4 GHz",
    ],
    [
        3700000000,
        5000000000.0,
        "Intel Xeon E Processor",
        3,
        2019.0,
        "Coffee Lake",
        "E-2288G",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon E Processor E-2288G @ 3.7 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2008.0,
        "Conroe",
        "E1200",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor E1200 @ 1.6 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2008.0,
        "Conroe",
        "E1400",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor E1400 @ 2.0 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2006.0,
        "Conroe",
        "E1500",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor E1500 @ 2.2 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2009.0,
        "Conroe",
        "E1600",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor E1600 @ 2.4 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2007.0,
        "Conroe",
        "E2140",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E2140 @ 1.6 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2006.0,
        "Conroe",
        "E2160",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E2160 @ 1.8 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2007.0,
        "Conroe",
        "E2180",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E2180 @ 2.0 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2007.0,
        "Conroe",
        "E2200",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E2200 @ 2.2 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2009.0,
        "Wolfdale",
        "E2210",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E2210 @ 2.2 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2008.0,
        "Conroe",
        "E2220",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E2220 @ 2.4 GHz",
    ],
    [
        1000000000,
        None,
        "Intel Xeon Processor E3 Family",
        2,
        2012.0,
        "Gladden",
        "E3-1105C",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1105C @ 1.0 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2013.0,
        "Gladden",
        "E3-1105CV2",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1105CV2 @ 1.8 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Xeon Processor E3 Family",
        2,
        2012.0,
        "Gladden",
        "E3-1125C",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1125C @ 2.0 GHz",
    ],
    [
        2500000000,
        None,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2013.0,
        "Gladden",
        "E3-1125CV2",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1125CV2 @ 2.5 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2013.0,
        "Gladden",
        "E3-1135CV2",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1135CV2 @ 3.0 GHz",
    ],
    [
        3000000000,
        3000000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1205V6",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1205V6 @ 3.0 GHz",
    ],
    [
        3100000000,
        3400000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1220",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1220 @ 3.1 GHz",
    ],
    [
        3100000000,
        3500000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1220 v3",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1220 v3 @ 3.1 GHz",
    ],
    [
        2200000000,
        3400000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1220L",
        2,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1220L @ 2.2 GHz",
    ],
    [
        2300000000,
        3500000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1220LV2",
        2,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1220LV2 @ 2.3 GHz",
    ],
    [
        1100000000,
        1500000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1220LV3",
        2,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1220LV3 @ 1.1 GHz",
    ],
    [
        3100000000,
        3500000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1220V2",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1220V2 @ 3.1 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1220V5",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1220V5 @ 3.0 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1220V6",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1220V6 @ 3.0 GHz",
    ],
    [
        3100000000,
        3400000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1225",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1225 @ 3.1 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1225V2",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1225V2 @ 3.2 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1225V3",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1225V3 @ 3.2 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1225V5",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1225V5 @ 3.3 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1225V6",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1225V6 @ 3.3 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1226V3",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1226V3 @ 3.3 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1230",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1230 @ 3.2 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1230 v3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1230 v3 @ 3.3 GHz",
    ],
    [
        1800000000,
        2800000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1230Lv3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1230Lv3 @ 1.8 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1230V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1230V2 @ 3.3 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1230V5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1230V5 @ 3.4 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1230V6",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1230V6 @ 3.5 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1231V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1231V3 @ 3.4 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1235",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1235 @ 3.2 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1235LV5",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1235LV5 @ 2.0 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1240",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1240 @ 3.3 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1240 v3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1240 v3 @ 3.4 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1240LV3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1240LV3 @ 2.0 GHz",
    ],
    [
        2100000000,
        3200000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1240LV5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1240LV5 @ 2.1 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1240V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1240V2 @ 3.4 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1240V5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1240V5 @ 3.5 GHz",
    ],
    [
        3700000000,
        4099999999.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1240V6",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1240V6 @ 3.7 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1241V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1241V3 @ 3.5 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1245",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1245 @ 3.3 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1245 v3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1245 v3 @ 3.4 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1245V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1245V2 @ 3.4 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1245V5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1245V5 @ 3.5 GHz",
    ],
    [
        3700000000,
        4099999999.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1245V6",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1245V6 @ 3.7 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1246V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1246V3 @ 3.5 GHz",
    ],
    [
        1800000000,
        3200000000.0,
        "Intel Xeon Processor E3 v4 Family",
        3,
        2015.0,
        "Broadwell",
        "E3-1258LV4",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v4 Family E3-1258LV4 @ 1.8 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1260L",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1260L @ 2.4 GHz",
    ],
    [
        2900000000,
        3900000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1260LV5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1260LV5 @ 2.9 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1265LV2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1265LV2 @ 2.5 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Intel Xeon Processor E3 v4 Family",
        3,
        2015.0,
        "Broadwell",
        "E3-1265LV4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v4 Family E3-1265LV4 @ 2.3 GHz",
    ],
    [
        2500000000,
        3700000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1265Lv3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1265Lv3 @ 2.5 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1268LV3",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1268LV3 @ 2.3 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1268LV5",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1268LV5 @ 2.4 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1270",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1270 @ 3.4 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1270 v3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1270 v3 @ 3.5 GHz",
    ],
    [
        3000000000,
        3600000000.0,
        "Intel Xeon Processor E3 v4 Family",
        3,
        2015.0,
        "Broadwell",
        "E3-1270LV4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v4 Family E3-1270LV4 @ 3.0 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1270V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1270V2 @ 3.5 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1270V5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1270V5 @ 3.6 GHz",
    ],
    [
        3800000000,
        4200000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1270V6",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1270V6 @ 3.8 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1271V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1271V3 @ 3.6 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1275",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1275 @ 3.4 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1275 v3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1275 v3 @ 3.5 GHz",
    ],
    [
        2700000000,
        3900000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1275LV3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1275LV3 @ 2.7 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1275V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1275V2 @ 3.5 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1275V5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1275V5 @ 3.6 GHz",
    ],
    [
        3800000000,
        4200000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1275V6",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1275V6 @ 3.8 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1276V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1276V3 @ 3.6 GHz",
    ],
    [
        2000000000,
        3300000000.0,
        "Intel Xeon Processor E3 v4 Family",
        3,
        2015.0,
        "Broadwell",
        "E3-1278LV4",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v4 Family E3-1278LV4 @ 2.0 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1280",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1280 @ 3.5 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1280 v3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1280 v3 @ 3.6 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1280V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1280V2 @ 3.6 GHz",
    ],
    [
        3700000000,
        4000000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1280V5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1280V5 @ 3.7 GHz",
    ],
    [
        3900000000,
        4200000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1280V6",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1280V6 @ 3.9 GHz",
    ],
    [
        3700000000,
        4099999999.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1281V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1281V3 @ 3.7 GHz",
    ],
    [
        1800000000,
        3200000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Crystal Well",
        "E3-1284LV3",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1284LV3 @ 1.8 GHz",
    ],
    [
        2900000000,
        3800000000.0,
        "Intel Xeon Processor E3 v4 Family",
        3,
        2015.0,
        "Broadwell",
        "E3-1284LV4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v4 Family E3-1284LV4 @ 2.9 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1285 v3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1285 v3 @ 3.6 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Intel Xeon Processor E3 v4 Family",
        3,
        2015.0,
        "Broadwell",
        "E3-1285LV4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v4 Family E3-1285LV4 @ 3.4 GHz",
    ],
    [
        3100000000,
        3900000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2013.0,
        "Haswell",
        "E3-1285Lv3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1285Lv3 @ 3.1 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "Intel Xeon Processor E3 v4 Family",
        3,
        2015.0,
        "Broadwell",
        "E3-1285V4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v4 Family E3-1285V4 @ 3.5 GHz",
    ],
    [
        4099999999,
        4500000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1285V6",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1285V6 @ 4.1 GHz",
    ],
    [
        3200000000,
        4000000000.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1286LV3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1286LV3 @ 3.2 GHz",
    ],
    [
        3700000000,
        4099999999.0,
        "Intel Xeon Processor E3 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E3-1286V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v3 Family E3-1286V3 @ 3.7 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Xeon Processor E3 Family",
        2,
        2011.0,
        "Sandy Bridge",
        "E3-1290",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 Family E3-1290 @ 3.6 GHz",
    ],
    [
        3700000000,
        4099999999.0,
        "Intel Xeon Processor E3 v2 Family",
        2,
        2012.0,
        "Ivy Bridge",
        "E3-1290V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v2 Family E3-1290V2 @ 3.7 GHz",
    ],
    [
        2100000000,
        2900000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1501LV6",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1501LV6 @ 2.1 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1501MV6",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1501MV6 @ 2.9 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1505LV5",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1505LV5 @ 2.0 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1505LV6",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1505LV6 @ 2.2 GHz",
    ],
    [
        2800000000,
        3700000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1505MV5",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1505MV5 @ 2.8 GHz",
    ],
    [
        3000000000,
        4000000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1505MV6",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1505MV6 @ 3.0 GHz",
    ],
    [
        2800000000,
        3700000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2016.0,
        "Skylake",
        "E3-1515MV5",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1515MV5 @ 2.8 GHz",
    ],
    [
        2900000000,
        3800000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2015.0,
        "Skylake",
        "E3-1535MV5",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1535MV5 @ 2.9 GHz",
    ],
    [
        3100000000,
        4200000000.0,
        "Intel Xeon Processor E3 v6 Family",
        3,
        2017.0,
        "Kaby Lake",
        "E3-1535MV6",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v6 Family E3-1535MV6 @ 3.1 GHz",
    ],
    [
        2900000000,
        3800000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2016.0,
        "Skylake",
        "E3-1545MV5",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1545MV5 @ 2.9 GHz",
    ],
    [
        1900000000,
        3300000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2016.0,
        "Skylake",
        "E3-1558LV5",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1558LV5 @ 1.9 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2016.0,
        "Skylake",
        "E3-1565LV5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1565LV5 @ 2.5 GHz",
    ],
    [
        3000000000,
        3900000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2016.0,
        "Skylake",
        "E3-1575MV5",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1575MV5 @ 3.0 GHz",
    ],
    [
        2000000000,
        3400000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2016.0,
        "Skylake",
        "E3-1578LV5",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1578LV5 @ 2.0 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2016.0,
        "Skylake",
        "E3-1585LV5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1585LV5 @ 3.0 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E3 v5 Family",
        3,
        2016.0,
        "Skylake",
        "E3-1585V5",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E3 v5 Family E3-1585V5 @ 3.5 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Wolfdale",
        "E3110",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E3110 @ 3.0 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Wolfdale",
        "E3113",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E3113 @ 3.0 GHz",
    ],
    [
        3160000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Wolfdale",
        "E3120",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E3120 @ 3.2 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2009.0,
        "Wolfdale",
        "E3200",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor E3200 @ 2.4 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2009.0,
        "Wolfdale",
        "E3300",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor E3300 @ 2.5 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2010.0,
        "Wolfdale",
        "E3400",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor E3400 @ 2.6 GHz",
    ],
    [
        2700000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2010.0,
        "Wolfdale",
        "E3500",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor E3500 @ 2.7 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Conroe",
        "E4300",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E4300 @ 1.8 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Conroe",
        "E4400",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E4400 @ 2.0 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Conroe",
        "E4500",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E4500 @ 2.2 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Conroe",
        "E4600",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E4600 @ 2.4 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Conroe",
        "E4700",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E4700 @ 2.6 GHz",
    ],
    [
        2800000000,
        3200000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-1410",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-1410 @ 2.8 GHz",
    ],
    [
        2800000000,
        3200000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-1410V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-1410V2 @ 2.8 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-1428L",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-1428L @ 1.8 GHz",
    ],
    [
        2200000000,
        2700000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-1428LV2",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-1428LV2 @ 2.2 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        2,
        2015.0,
        "Haswell",
        "E5-1428LV3",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-1428LV3 @ 2.0 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-1603",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-1603 @ 2.8 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-1603V3",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-1603V3 @ 2.8 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-1603V4",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-1603V4 @ 2.8 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-1607",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-1607 @ 3.0 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-1607V2",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-1607V2 @ 3.0 GHz",
    ],
    [
        3100000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-1607V3",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-1607V3 @ 3.1 GHz",
    ],
    [
        3100000000,
        None,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-1607V4",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-1607V4 @ 3.1 GHz",
    ],
    [
        3600000000,
        3800000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-1620",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-1620 @ 3.6 GHz",
    ],
    [
        3700000000,
        3900000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-1620V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-1620V2 @ 3.7 GHz",
    ],
    [
        3500000000,
        3600000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-1620V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-1620V3 @ 3.5 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-1620V4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-1620V4 @ 3.5 GHz",
    ],
    [
        3700000000,
        3800000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-1630V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-1630V3 @ 3.7 GHz",
    ],
    [
        3700000000,
        4000000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-1630V4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-1630V4 @ 3.7 GHz",
    ],
    [
        3200000000,
        3800000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-1650",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-1650 @ 3.2 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-1650V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-1650V2 @ 3.5 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-1650V3",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-1650V3 @ 3.5 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-1650V4",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-1650V4 @ 3.6 GHz",
    ],
    [
        3300000000,
        3900000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-1660",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-1660 @ 3.3 GHz",
    ],
    [
        3700000000,
        4000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-1660V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-1660V2 @ 3.7 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-1660V3",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-1660V3 @ 3.0 GHz",
    ],
    [
        3200000000,
        3800000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-1660V4",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-1660V4 @ 3.2 GHz",
    ],
    [
        3000000000,
        3900000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-1680V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-1680V2 @ 3.0 GHz",
    ],
    [
        3200000000,
        3800000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-1680V3",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-1680V3 @ 3.2 GHz",
    ],
    [
        3400000000,
        4000000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-1680V4",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-1680V4 @ 3.4 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-2403",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2403 @ 1.8 GHz",
    ],
    [
        1800000000,
        1800000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2403V2",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2403V2 @ 1.8 GHz",
    ],
    [
        2200000000,
        None,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-2407",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2407 @ 2.2 GHz",
    ],
    [
        2400000000,
        2400000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2407V2",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2407V2 @ 2.4 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        2,
        2015.0,
        "Haswell",
        "E5-2408LV3",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2408LV3 @ 1.8 GHz",
    ],
    [
        2000000000,
        2100000000.0,
        "Intel Xeon Processor E5 Family",
        0,
        2012.0,
        "Sandy Bridge EN",
        "E5-2418L",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2418L @ 2.0 GHz",
    ],
    [
        2000000000,
        2000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2418LV2",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2418LV2 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        2,
        2015.0,
        "Haswell",
        "E5-2418LV3",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2418LV3 @ 2.0 GHz",
    ],
    [
        1900000000,
        2400000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-2420",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2420 @ 1.9 GHz",
    ],
    [
        2200000000,
        2700000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2420V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2420V2 @ 2.2 GHz",
    ],
    [
        1800000000,
        2000000000.0,
        "Intel Xeon Processor E5 Family",
        0,
        2012.0,
        "Sandy Bridge EN",
        "E5-2428L",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2428L @ 1.8 GHz",
    ],
    [
        1800000000,
        2300000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2428LV2",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2428LV2 @ 1.8 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        2,
        2015.0,
        "Haswell",
        "E5-2428LV3",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2428LV3 @ 1.8 GHz",
    ],
    [
        2200000000,
        2700000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-2430",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2430 @ 2.2 GHz",
    ],
    [
        2000000000,
        2500000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-2430L",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2430L @ 2.0 GHz",
    ],
    [
        2400000000,
        2800000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2430LV2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2430LV2 @ 2.4 GHz",
    ],
    [
        2500000000,
        3000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2430V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2430V2 @ 2.5 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        2,
        2015.0,
        "Haswell",
        "E5-2438LV3",
        10,
        "20",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2438LV3 @ 1.8 GHz",
    ],
    [
        2400000000,
        2900000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-2440",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2440 @ 2.4 GHz",
    ],
    [
        1900000000,
        2400000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2440V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2440V2 @ 1.9 GHz",
    ],
    [
        1800000000,
        2100000000.0,
        "Intel Xeon Processor E5 Family",
        0,
        2012.0,
        "Sandy Bridge EN",
        "E5-2448L",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2448L @ 1.8 GHz",
    ],
    [
        1800000000,
        2400000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2448LV2",
        10,
        "20",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2448LV2 @ 1.8 GHz",
    ],
    [
        2100000000,
        2900000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-2450",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2450 @ 2.1 GHz",
    ],
    [
        1800000000,
        2300000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-2450L",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2450L @ 1.8 GHz",
    ],
    [
        1700000000,
        2100000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2450LV2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2450LV2 @ 1.7 GHz",
    ],
    [
        2500000000,
        3300000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2450V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2450V2 @ 2.5 GHz",
    ],
    [
        2300000000,
        3100000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EN",
        "E5-2470",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2470 @ 2.3 GHz",
    ],
    [
        2400000000,
        3200000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EN",
        "E5-2470V2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2470V2 @ 2.4 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2603",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2603 @ 1.8 GHz",
    ],
    [
        1800000000,
        None,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2603V2",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2603V2 @ 1.8 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2603V3",
        6,
        "6",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2603V3 @ 1.6 GHz",
    ],
    [
        1700000000,
        None,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2603V4",
        6,
        "6",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2603V4 @ 1.7 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2608LV3",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2608LV3 @ 2.0 GHz",
    ],
    [
        1600000000,
        1700000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2608LV4",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2608LV4 @ 1.6 GHz",
    ],
    [
        2400000000,
        None,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2609",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2609 @ 2.4 GHz",
    ],
    [
        2500000000,
        2500000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2609V2",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2609V2 @ 2.5 GHz",
    ],
    [
        1900000000,
        None,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2609V3",
        6,
        "6",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2609V3 @ 1.9 GHz",
    ],
    [
        1700000000,
        None,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2609V4",
        8,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2609V4 @ 1.7 GHz",
    ],
    [
        2000000000,
        2000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2618LV2",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2618LV2 @ 2.0 GHz",
    ],
    [
        2300000000,
        3400000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2618LV3",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2618LV3 @ 2.3 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2618LV4",
        10,
        "20",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2618LV4 @ 2.2 GHz",
    ],
    [
        2000000000,
        2500000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2620",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2620 @ 2.0 GHz",
    ],
    [
        2100000000,
        2600000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2620V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2620V2 @ 2.1 GHz",
    ],
    [
        2400000000,
        3200000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2620V3",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2620V3 @ 2.4 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2620V4",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2620V4 @ 2.1 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2623V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2623V3 @ 3.0 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2623V4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2623V4 @ 2.6 GHz",
    ],
    [
        1900000000,
        2400000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2628LV2",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2628LV2 @ 1.9 GHz",
    ],
    [
        2000000000,
        2500000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2628LV3",
        10,
        "20",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2628LV3 @ 2.0 GHz",
    ],
    [
        1900000000,
        2400000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2628LV4",
        12,
        "24",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2628LV4 @ 1.9 GHz",
    ],
    [
        2300000000,
        2800000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2630",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2630 @ 2.3 GHz",
    ],
    [
        2000000000,
        2500000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2630L",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2630L @ 2.0 GHz",
    ],
    [
        2400000000,
        2800000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2630LV2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2630LV2 @ 2.4 GHz",
    ],
    [
        1800000000,
        2900000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2630LV3",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2630LV3 @ 1.8 GHz",
    ],
    [
        1800000000,
        2900000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2630LV4",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2630LV4 @ 1.8 GHz",
    ],
    [
        2600000000,
        3100000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2630V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2630V2 @ 2.6 GHz",
    ],
    [
        2400000000,
        3200000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2630V3",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2630V3 @ 2.4 GHz",
    ],
    [
        2200000000,
        3100000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2630V4",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2630V4 @ 2.2 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2637",
        2,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2637 @ 3.0 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2637V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2637V2 @ 3.5 GHz",
    ],
    [
        3500000000,
        3700000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2637V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2637V3 @ 3.5 GHz",
    ],
    [
        3500000000,
        3700000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2637V4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2637V4 @ 3.5 GHz",
    ],
    [
        2500000000,
        3000000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2640",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2640 @ 2.5 GHz",
    ],
    [
        2000000000,
        2500000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2640V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2640V2 @ 2.0 GHz",
    ],
    [
        2600000000,
        3400000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2640V3",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2640V3 @ 2.6 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2640V4",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2640V4 @ 2.4 GHz",
    ],
    [
        3300000000,
        3500000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2643",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2643 @ 3.3 GHz",
    ],
    [
        3500000000,
        3800000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2643V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2643V2 @ 3.5 GHz",
    ],
    [
        3400000000,
        3700000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2643V3",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2643V3 @ 3.4 GHz",
    ],
    [
        3400000000,
        3700000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2643V4",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2643V4 @ 3.4 GHz",
    ],
    [
        1800000000,
        2100000000.0,
        "Intel Xeon Processor E5 Family",
        0,
        2012.0,
        "Sandy Bridge EP",
        "E5-2648L",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2648L @ 1.8 GHz",
    ],
    [
        1900000000,
        2500000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2648LV2",
        10,
        "20",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2648LV2 @ 1.9 GHz",
    ],
    [
        1800000000,
        2500000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2648LV3",
        12,
        "24",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2648LV3 @ 1.8 GHz",
    ],
    [
        1800000000,
        2500000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2648LV4",
        14,
        "28",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2648LV4 @ 1.8 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2650",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2650 @ 2.0 GHz",
    ],
    [
        1800000000,
        2300000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2650L",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2650L @ 1.8 GHz",
    ],
    [
        1700000000,
        2100000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2650LV2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2650LV2 @ 1.7 GHz",
    ],
    [
        1800000000,
        2500000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2650LV3",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2650LV3 @ 1.8 GHz",
    ],
    [
        1700000000,
        2500000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2650LV4",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2650LV4 @ 1.7 GHz",
    ],
    [
        2600000000,
        3400000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2650V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2650V2 @ 2.6 GHz",
    ],
    [
        2300000000,
        3000000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2650V3",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2650V3 @ 2.3 GHz",
    ],
    [
        2200000000,
        2900000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2650V4",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2650V4 @ 2.2 GHz",
    ],
    [
        2100000000,
        2400000000.0,
        "Intel Xeon Processor E5 Family",
        0,
        2012.0,
        "Sandy Bridge EP",
        "E5-2658",
        8,
        "16",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2658 @ 2.1 GHz",
    ],
    [
        2200000000,
        2900000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-2658AV3",
        12,
        "24",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2658AV3 @ 2.2 GHz",
    ],
    [
        2400000000,
        3000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2658V2",
        10,
        "20",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2658V2 @ 2.4 GHz",
    ],
    [
        2200000000,
        2900000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2658V3",
        12,
        "24",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2658V3 @ 2.2 GHz",
    ],
    [
        2300000000,
        2800000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2658V4",
        14,
        "28",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2658V4 @ 2.3 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2660",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2660 @ 2.2 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2660V2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2660V2 @ 2.2 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2660V3",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2660V3 @ 2.6 GHz",
    ],
    [
        2000000000,
        3200000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2660V4",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2660V4 @ 2.0 GHz",
    ],
    [
        2400000000,
        3100000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2665",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2665 @ 2.4 GHz",
    ],
    [
        2900000000,
        3500000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2667",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2667 @ 2.9 GHz",
    ],
    [
        3300000000,
        4000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2667V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2667V2 @ 3.3 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2667V3",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2667V3 @ 3.2 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2667V4",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2667V4 @ 3.2 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2670",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2670 @ 2.6 GHz",
    ],
    [
        2500000000,
        3300000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2670V2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2670V2 @ 2.5 GHz",
    ],
    [
        2300000000,
        3100000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2670V3",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2670V3 @ 2.3 GHz",
    ],
    [
        3300000000,
        4000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2673V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2673V2 @ 3.3 GHz",
    ],
    [
        2400000000,
        3200000000,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2013.0,
        "Ivy Bridge EP",
        "E5-2673V3",
        12,
        24,
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2673V3 @ 2.4 GHz",
    ],
    [
        2500000000,
        3300000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2679V4",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2679V4 @ 2.5 GHz",
    ],
    [
        2700000000,
        3500000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2680",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2680 @ 2.7 GHz",
    ],
    [
        2800000000,
        3600000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2680V2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2680V2 @ 2.8 GHz",
    ],
    [
        2500000000,
        3300000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2680V3",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2680V3 @ 2.5 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2680V4",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2680V4 @ 2.4 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2683V3",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2683V3 @ 2.0 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2683V4",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2683V4 @ 2.1 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2685V3",
        12,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2685V3 @ 2.6 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2687W",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2687W @ 3.1 GHz",
    ],
    [
        3400000000,
        4000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2687WV2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2687WV2 @ 3.4 GHz",
    ],
    [
        3100000000,
        3500000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2687WV3",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2687WV3 @ 3.1 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2687WV4",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2687WV4 @ 3.0 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2689V4",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2689V4 @ 3.1 GHz",
    ],
    [
        2900000000,
        3800000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-2690",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-2690 @ 2.9 GHz",
    ],
    [
        3000000000,
        3600000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2690V2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2690V2 @ 3.0 GHz",
    ],
    [
        2600000000,
        3500000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2690V3",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2690V3 @ 2.6 GHz",
    ],
    [
        2600000000,
        3500000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2690V4",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2690V4 @ 2.6 GHz",
    ],
    [
        2400000000,
        3200000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2695V2",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2695V2 @ 2.4 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2695V3",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2695V3 @ 2.3 GHz",
    ],
    [
        2100000000,
        3300000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2695V4",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2695V4 @ 2.1 GHz",
    ],
    [
        2600000000,
        3600000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2697AV4",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2697AV4 @ 2.6 GHz",
    ],
    [
        2700000000,
        3500000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2013.0,
        "Ivy Bridge EP",
        "E5-2697V2",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-2697V2 @ 2.7 GHz",
    ],
    [
        2600000000,
        3600000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2697V3",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2697V3 @ 2.6 GHz",
    ],
    [
        2300000000,
        3600000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2697V4",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2697V4 @ 2.3 GHz",
    ],
    [
        2300000000,
        3600000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2698V3",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2698V3 @ 2.3 GHz",
    ],
    [
        2200000000,
        3600000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2698V4",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2698V4 @ 2.2 GHz",
    ],
    [
        2400000000,
        3600000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2699AV4",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2699AV4 @ 2.4 GHz",
    ],
    [
        2200000000,
        3600000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2699RV4",
        22,
        "44",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2699RV4 @ 2.2 GHz",
    ],
    [
        2300000000,
        3600000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2014.0,
        "Haswell",
        "E5-2699V3",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-2699V3 @ 2.3 GHz",
    ],
    [
        2200000000,
        3600000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-2699V4",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-2699V4 @ 2.2 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-4603",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-4603 @ 2.0 GHz",
    ],
    [
        2200000000,
        2200000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EP",
        "E5-4603V2",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-4603V2 @ 2.2 GHz",
    ],
    [
        2200000000,
        None,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-4607",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-4607 @ 2.2 GHz",
    ],
    [
        2600000000,
        2600000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EP",
        "E5-4607V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-4607V2 @ 2.6 GHz",
    ],
    [
        2400000000,
        2900000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-4610",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-4610 @ 2.4 GHz",
    ],
    [
        2300000000,
        2700000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EP",
        "E5-4610V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-4610V2 @ 2.3 GHz",
    ],
    [
        1700000000,
        1700000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4610V3",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4610V3 @ 1.7 GHz",
    ],
    [
        1800000000,
        1800000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4610V4",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4610V4 @ 1.8 GHz",
    ],
    [
        2900000000,
        3400000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-4617",
        6,
        "6",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-4617 @ 2.9 GHz",
    ],
    [
        2200000000,
        2600000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-4620",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-4620 @ 2.2 GHz",
    ],
    [
        2600000000,
        3000000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EP",
        "E5-4620V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-4620V2 @ 2.6 GHz",
    ],
    [
        2000000000,
        2600000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4620V3",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4620V3 @ 2.0 GHz",
    ],
    [
        2100000000,
        2600000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4620V4",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4620V4 @ 2.1 GHz",
    ],
    [
        1900000000,
        2500000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EP",
        "E5-4624LV2",
        10,
        "20",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-4624LV2 @ 1.9 GHz",
    ],
    [
        3300000000,
        3600000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EP",
        "E5-4627V2",
        8,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-4627V2 @ 3.3 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4627V3",
        10,
        "10",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4627V3 @ 2.6 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4627V4",
        10,
        "10",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4627V4 @ 2.6 GHz",
    ],
    [
        1800000000,
        2200000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4628LV4",
        14,
        "28",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4628LV4 @ 1.8 GHz",
    ],
    [
        2400000000,
        2800000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-4640",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-4640 @ 2.4 GHz",
    ],
    [
        2200000000,
        2700000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EP",
        "E5-4640V2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-4640V2 @ 2.2 GHz",
    ],
    [
        1900000000,
        2600000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4640V3",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4640V3 @ 1.9 GHz",
    ],
    [
        2100000000,
        2600000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4640V4",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4640V4 @ 2.1 GHz",
    ],
    [
        1700000000,
        2200000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4648V3",
        12,
        "24",
        "Embedded",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4648V3 @ 1.7 GHz",
    ],
    [
        2700000000,
        3300000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-4650",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-4650 @ 2.7 GHz",
    ],
    [
        2600000000,
        3100000000.0,
        "Intel Xeon Processor E5 Family",
        2,
        2012.0,
        "Sandy Bridge EP",
        "E5-4650L",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 Family E5-4650L @ 2.6 GHz",
    ],
    [
        2400000000,
        2900000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EP",
        "E5-4650V2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-4650V2 @ 2.4 GHz",
    ],
    [
        2100000000,
        2800000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4650V3",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4650V3 @ 2.1 GHz",
    ],
    [
        2200000000,
        2800000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4650V4",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4650V4 @ 2.2 GHz",
    ],
    [
        2900000000,
        3200000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4655V3",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4655V3 @ 2.9 GHz",
    ],
    [
        2500000000,
        3200000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4655V4",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4655V4 @ 2.5 GHz",
    ],
    [
        2400000000,
        2900000000.0,
        "Intel Xeon Processor E5 v2 Family",
        2,
        2014.0,
        "Ivy Bridge EP",
        "E5-4657LV2",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v2 Family E5-4657LV2 @ 2.4 GHz",
    ],
    [
        2100000000,
        2900000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4660V3",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4660V3 @ 2.1 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4660V4",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4660V4 @ 2.2 GHz",
    ],
    [
        2000000000,
        2900000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4667V3",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4667V3 @ 2.0 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4667V4",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4667V4 @ 2.2 GHz",
    ],
    [
        2100000000,
        2900000000.0,
        "Intel Xeon Processor E5 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E5-4669V3",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v3 Family E5-4669V3 @ 2.1 GHz",
    ],
    [
        2200000000,
        3000000000.0,
        "Intel Xeon Processor E5 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E5-4669V4",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E5 v4 Family E5-4669V4 @ 2.2 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2008.0,
        "Wolfdale",
        "E5200",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E5200 @ 2.5 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Wolfdale",
        "E5205",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5205 @ 1.9 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Wolfdale",
        "E5220",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5220 @ 2.3 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Wolfdale",
        "E5240",
        2,
        2,
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5240 @ 3.0 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2008.0,
        "Wolfdale",
        "E5300",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E5300 @ 2.6 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2006.0,
        "Clovertown",
        "E5310",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5310 @ 1.6 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2006.0,
        "Clovertown",
        "E5320",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5320 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Clovertown",
        "E5335",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5335 @ 2.0 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Clovertown",
        "E5345",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5345 @ 2.3 GHz",
    ],
    [
        2700000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2009.0,
        "Wolfdale",
        "E5400",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E5400 @ 2.7 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "E5405",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5405 @ 2.0 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "E5410",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5410 @ 2.3 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "E5420",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5420 @ 2.5 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "E5430",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5430 @ 2.7 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Harpertown",
        "E5440",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5440 @ 2.8 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "E5450",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5450 @ 3.0 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "E5462",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5462 @ 2.8 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "E5472",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5472 @ 3.0 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Wolfdale",
        "E5500",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E5500 @ 2.8 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "E5502",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5502 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EP",
        "E5503",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5503 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "E5504",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5504 @ 2.0 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "E5506",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5506 @ 2.1 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EP",
        "E5507",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5507 @ 2.3 GHz",
    ],
    [
        2260000000,
        2530000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "E5520",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5520 @ 2.3 GHz",
    ],
    [
        2400000000,
        2660000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "E5530",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5530 @ 2.4 GHz",
    ],
    [
        2530000000,
        2800000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "E5540",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5540 @ 2.5 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Westmere EP",
        "E5603",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5603 @ 1.6 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Westmere EP",
        "E5606",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5606 @ 2.1 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Westmere EP",
        "E5607",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5607 @ 2.3 GHz",
    ],
    [
        2400000000,
        2660000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "E5620",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5620 @ 2.4 GHz",
    ],
    [
        2530000000,
        2800000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "E5630",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5630 @ 2.5 GHz",
    ],
    [
        2660000000,
        2930000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "E5640",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5640 @ 2.7 GHz",
    ],
    [
        2400000000,
        2670000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "E5645",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5645 @ 2.4 GHz",
    ],
    [
        2530000000,
        2930000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Westmere EP",
        "E5649",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E5649 @ 2.5 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Wolfdale",
        "E5700",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E5700 @ 3.0 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Wolfdale",
        "E5800",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E5800 @ 3.2 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2009.0,
        "Wolfdale",
        "E6300",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E6300 @ 2.8 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Conroe",
        "E6305",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E6305 @ 1.9 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Conroe",
        "E6320",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E6320 @ 1.9 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Conroe",
        "E6400",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E6400 @ 2.1 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Conroe",
        "E6405",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E6405 @ 2.1 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Conroe",
        "E6420",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E6420 @ 2.1 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2008.0,
        "Wolfdale",
        "E6500",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E6500 @ 2.9 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2009.0,
        "Wolfdale",
        "E6500K",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E6500K @ 2.9 GHz",
    ],
    [
        1730000000,
        1730000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "E6510",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E6510 @ 1.7 GHz",
    ],
    [
        2000000000,
        2270000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "E6540",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E6540 @ 2.0 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Conroe",
        "E6550",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E6550 @ 2.3 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Wolfdale",
        "E6600",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E6600 @ 3.1 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Wolfdale",
        "E6700",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E6700 @ 3.2 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Conroe",
        "E6750",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E6750 @ 2.7 GHz",
    ],
    [
        3330000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2010.0,
        "Wolfdale",
        "E6800",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor E6800 @ 3.3 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Conroe",
        "E6850",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E6850 @ 3.0 GHz",
    ],
    [
        1730000000,
        None,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-2803",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-2803 @ 1.7 GHz",
    ],
    [
        2000000000,
        2270000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-2820",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-2820 @ 2.0 GHz",
    ],
    [
        2130000000,
        2400000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-2830",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-2830 @ 2.1 GHz",
    ],
    [
        2000000000,
        2400000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-2850",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-2850 @ 2.0 GHz",
    ],
    [
        2300000000,
        2800000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-2850V2",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-2850V2 @ 2.3 GHz",
    ],
    [
        2260000000,
        2670000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-2860",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-2860 @ 2.3 GHz",
    ],
    [
        2400000000,
        2800000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-2870",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-2870 @ 2.4 GHz",
    ],
    [
        2300000000,
        2900000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-2870V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-2870V2 @ 2.3 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-2880V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-2880V2 @ 2.5 GHz",
    ],
    [
        2800000000,
        3400000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-2890V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-2890V2 @ 2.8 GHz",
    ],
    [
        1860000000,
        None,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-4807",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-4807 @ 1.9 GHz",
    ],
    [
        1900000000,
        None,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-4809V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-4809V2 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-4809V3",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-4809V3 @ 2.0 GHz",
    ],
    [
        2100000000,
        None,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-4809V4",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-4809V4 @ 2.1 GHz",
    ],
    [
        2000000000,
        2270000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-4820",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-4820 @ 2.0 GHz",
    ],
    [
        2000000000,
        2500000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-4820V2",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-4820V2 @ 2.0 GHz",
    ],
    [
        1900000000,
        None,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-4820V3",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-4820V3 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-4820V4",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-4820V4 @ 2.0 GHz",
    ],
    [
        2130000000,
        2400000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-4830",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-4830 @ 2.1 GHz",
    ],
    [
        2200000000,
        2700000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-4830V2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-4830V2 @ 2.2 GHz",
    ],
    [
        2100000000,
        2700000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-4830V3",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-4830V3 @ 2.1 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-4830V4",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-4830V4 @ 2.0 GHz",
    ],
    [
        2000000000,
        2400000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-4850",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-4850 @ 2.0 GHz",
    ],
    [
        2300000000,
        2800000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-4850V2",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-4850V2 @ 2.3 GHz",
    ],
    [
        2200000000,
        2800000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-4850V3",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-4850V3 @ 2.2 GHz",
    ],
    [
        2100000000,
        2800000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-4850V4",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-4850V4 @ 2.1 GHz",
    ],
    [
        2260000000,
        2670000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-4860",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-4860 @ 2.3 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-4860V2",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-4860V2 @ 2.6 GHz",
    ],
    [
        2400000000,
        2800000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-4870",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-4870 @ 2.4 GHz",
    ],
    [
        2300000000,
        2900000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-4870V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-4870V2 @ 2.3 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-4880V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-4880V2 @ 2.5 GHz",
    ],
    [
        2800000000,
        3400000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-4890V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-4890V2 @ 2.8 GHz",
    ],
    [
        2130000000,
        2400000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-8830",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-8830 @ 2.1 GHz",
    ],
    [
        2660000000,
        2800000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-8837",
        8,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-8837 @ 2.7 GHz",
    ],
    [
        2000000000,
        2400000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-8850",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-8850 @ 2.0 GHz",
    ],
    [
        2300000000,
        2800000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-8850V2",
        12,
        "24",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-8850V2 @ 2.3 GHz",
    ],
    [
        2100000000,
        2800000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-8855V4",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-8855V4 @ 2.1 GHz",
    ],
    [
        3000000000,
        3600000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-8857V2",
        12,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-8857V2 @ 3.0 GHz",
    ],
    [
        2260000000,
        2670000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-8860",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-8860 @ 2.3 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-8860V3",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-8860V3 @ 2.2 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-8860V4",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-8860V4 @ 2.2 GHz",
    ],
    [
        2130000000,
        2530000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-8867L",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-8867L @ 2.1 GHz",
    ],
    [
        2500000000,
        3300000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-8867V3",
        16,
        "32",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-8867V3 @ 2.5 GHz",
    ],
    [
        2400000000,
        3300000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-8867V4",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-8867V4 @ 2.4 GHz",
    ],
    [
        2400000000,
        2800000000.0,
        "Intel Xeon Processor E7 Family",
        1,
        2011.0,
        "Westmere EX",
        "E7-8870",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 Family E7-8870 @ 2.4 GHz",
    ],
    [
        2300000000,
        2900000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-8870V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-8870V2 @ 2.3 GHz",
    ],
    [
        2100000000,
        2900000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-8870V3",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-8870V3 @ 2.1 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-8870V4",
        20,
        "40",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-8870V4 @ 2.1 GHz",
    ],
    [
        2200000000,
        2800000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-8880LV2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-8880LV2 @ 2.2 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-8880LV3",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-8880LV3 @ 2.0 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-8880V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-8880V2 @ 2.5 GHz",
    ],
    [
        2300000000,
        3100000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-8880V3",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-8880V3 @ 2.3 GHz",
    ],
    [
        2200000000,
        3300000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-8880V4",
        22,
        "44",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-8880V4 @ 2.2 GHz",
    ],
    [
        2800000000,
        3400000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-8890V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-8890V2 @ 2.8 GHz",
    ],
    [
        2500000000,
        3300000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-8890V3",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-8890V3 @ 2.5 GHz",
    ],
    [
        2200000000,
        3400000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-8890V4",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-8890V4 @ 2.2 GHz",
    ],
    [
        3200000000,
        3700000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-8891V2",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-8891V2 @ 3.2 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-8891V3",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-8891V3 @ 2.8 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-8891V4",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-8891V4 @ 2.8 GHz",
    ],
    [
        3400000000,
        3700000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-8893V2",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-8893V2 @ 3.4 GHz",
    ],
    [
        3200000000,
        3500000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-8893V3",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-8893V3 @ 3.2 GHz",
    ],
    [
        3200000000,
        3500000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2016.0,
        "Broadwell",
        "E7-8893V4",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-8893V4 @ 3.2 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "Intel Xeon Processor E7 v4 Family",
        3,
        2017.0,
        "Broadwell",
        "E7-8894V4",
        24,
        "48",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v4 Family E7-8894V4 @ 2.4 GHz",
    ],
    [
        2800000000,
        3600000000.0,
        "Intel Xeon Processor E7 v2 Family",
        2,
        2014.0,
        "Ivy Bridge",
        "E7-8895V2",
        15,
        "30",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v2 Family E7-8895V2 @ 2.8 GHz",
    ],
    [
        2600000000,
        3500000000.0,
        "Intel Xeon Processor E7 v3 Family",
        3,
        2015.0,
        "Haswell",
        "E7-8895V3",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon Processor E7 v3 Family E7-8895V3 @ 2.6 GHz",
    ],
    [
        2530000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Wolfdale",
        "E7200",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E7200 @ 2.5 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Tigerton",
        "E7210",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7210 @ 2.4 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Tigerton",
        "E7220",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7220 @ 2.9 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Wolfdale",
        "E7300",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E7300 @ 2.7 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Tigerton",
        "E7310",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7310 @ 1.6 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Tigerton",
        "E7320",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7320 @ 2.1 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Tigerton",
        "E7330",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7330 @ 2.4 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Tigerton",
        "E7340",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7340 @ 2.4 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Wolfdale",
        "E7400",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E7400 @ 2.8 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Dunnington",
        "E7420",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7420 @ 2.1 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Dunnington",
        "E7430",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7430 @ 2.1 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Dunnington",
        "E7440",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7440 @ 2.4 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Dunnington",
        "E7450",
        6,
        6,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7450 @ 2.4 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Wolfdale",
        "E7500",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E7500 @ 2.9 GHz",
    ],
    [
        1870000000,
        1870000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "E7520",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7520 @ 1.9 GHz",
    ],
    [
        1870000000,
        2130000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "E7530",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7530 @ 1.9 GHz",
    ],
    [
        2000000000,
        2270000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "E7540",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors E7540 @ 2.0 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Wolfdale",
        "E7600",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E7600 @ 3.1 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Wolfdale",
        "E8190",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E8190 @ 2.7 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Wolfdale",
        "E8200",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E8200 @ 2.7 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Wolfdale",
        "E8300",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E8300 @ 2.8 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Wolfdale",
        "E8400",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E8400 @ 3.0 GHz",
    ],
    [
        3160000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Wolfdale",
        "E8500",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E8500 @ 3.2 GHz",
    ],
    [
        3330000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Wolfdale",
        "E8600",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors E8600 @ 3.3 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Jasper Forest",
        "EC3539",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors EC3539 @ 2.1 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Jasper Forest",
        "EC5509",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors EC5509 @ 2.0 GHz",
    ],
    [
        2270000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Jasper Forest",
        "EC5539",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors EC5539 @ 2.3 GHz",
    ],
    [
        2530000000,
        2930000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Jasper Forest",
        "EC5549",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors EC5549 @ 2.5 GHz",
    ],
    [
        2260000000,
        None,
        "Intel Celeron Processor G Series",
        0,
        2010.0,
        "Clarkdale",
        "G1101",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1101 @ 2.3 GHz",
    ],
    [
        2600000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G1610",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1610 @ 2.6 GHz",
    ],
    [
        2300000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G1610T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1610T @ 2.3 GHz",
    ],
    [
        2700000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G1620",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1620 @ 2.7 GHz",
    ],
    [
        2400000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G1620T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1620T @ 2.4 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G1630",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1630 @ 2.8 GHz",
    ],
    [
        2700000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G1820",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1820 @ 2.7 GHz",
    ],
    [
        2400000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G1820T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1820T @ 2.4 GHz",
    ],
    [
        2200000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G1820TE",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1820TE @ 2.2 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G1830",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1830 @ 2.8 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G1840",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1840 @ 2.8 GHz",
    ],
    [
        2500000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G1840T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1840T @ 2.5 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G1850",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G1850 @ 2.9 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G2010",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2010 @ 2.8 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G2020",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2020 @ 2.9 GHz",
    ],
    [
        2500000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G2020T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2020T @ 2.5 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G2030",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2030 @ 3.0 GHz",
    ],
    [
        2600000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G2030T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2030T @ 2.6 GHz",
    ],
    [
        2600000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2012.0,
        "Ivy Bridge",
        "G2100T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2100T @ 2.6 GHz",
    ],
    [
        3100000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2012.0,
        "Ivy Bridge",
        "G2120",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2120 @ 3.1 GHz",
    ],
    [
        2700000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G2120T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2120T @ 2.7 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G2130",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2130 @ 3.2 GHz",
    ],
    [
        3300000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Ivy Bridge",
        "G2140",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G2140 @ 3.3 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Haswell",
        "G3220",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3220 @ 3.0 GHz",
    ],
    [
        2600000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Haswell",
        "G3220T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3220T @ 2.6 GHz",
    ],
    [
        3100000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3240",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3240 @ 3.1 GHz",
    ],
    [
        2700000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3240T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3240T @ 2.7 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3250",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3250 @ 3.2 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3250T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3250T @ 2.8 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3258",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3258 @ 3.2 GHz",
    ],
    [
        3300000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Haswell",
        "G3260",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3260 @ 3.3 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Haswell",
        "G3260T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3260T @ 2.9 GHz",
    ],
    [
        2300000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Haswell",
        "G3320TE",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3320TE @ 2.3 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Haswell",
        "G3420",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3420 @ 3.2 GHz",
    ],
    [
        2700000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Haswell",
        "G3420T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3420T @ 2.7 GHz",
    ],
    [
        3300000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2013.0,
        "Haswell",
        "G3430",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3430 @ 3.3 GHz",
    ],
    [
        3300000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3440",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3440 @ 3.3 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3440T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3440T @ 2.8 GHz",
    ],
    [
        3400000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3450",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3450 @ 3.4 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3450T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3450T @ 2.9 GHz",
    ],
    [
        3500000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2014.0,
        "Haswell",
        "G3460",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3460 @ 3.5 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Haswell",
        "G3460T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3460T @ 3.0 GHz",
    ],
    [
        3600000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Haswell",
        "G3470",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G3470 @ 3.6 GHz",
    ],
    [
        2800000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G3900",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3900 @ 2.8 GHz",
    ],
    [
        2400000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2016.0,
        "Skylake",
        "G3900E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3900E @ 2.4 GHz",
    ],
    [
        2600000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G3900T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3900T @ 2.6 GHz",
    ],
    [
        2300000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G3900TE",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3900TE @ 2.3 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2016.0,
        "Skylake",
        "G3902E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3902E @ 1.6 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G3920",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3920 @ 2.9 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G3930",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3930 @ 2.9 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G3930E",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3930E @ 2.9 GHz",
    ],
    [
        2700000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G3930T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3930T @ 2.7 GHz",
    ],
    [
        2700000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G3930TE",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3930TE @ 2.7 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G3950",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G3950 @ 3.0 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G440",
        1,
        "1",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G440 @ 1.6 GHz",
    ],
    [
        3300000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G4400",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4400 @ 3.3 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G4400T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4400T @ 2.9 GHz",
    ],
    [
        2400000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G4400TE",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4400TE @ 2.4 GHz",
    ],
    [
        3500000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G4500",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4500 @ 3.5 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G4500T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4500T @ 3.0 GHz",
    ],
    [
        3600000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2015.0,
        "Skylake",
        "G4520",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4520 @ 3.6 GHz",
    ],
    [
        3500000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G4560",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4560 @ 3.5 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G4560T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4560T @ 2.9 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G460",
        1,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G460 @ 1.8 GHz",
    ],
    [
        3600000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G4600",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4600 @ 3.6 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G4600T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4600T @ 3.0 GHz",
    ],
    [
        3700000000,
        None,
        "Intel Pentium Processor G Series",
        1,
        2017.0,
        "Kaby Lake",
        "G4620",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor G Series G4620 @ 3.7 GHz",
    ],
    [
        1900000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G465",
        1,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G465 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2013.0,
        "Sandy Bridge",
        "G470",
        1,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G470 @ 2.0 GHz",
    ],
    [
        3100000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2018.0,
        "Coffee Lake",
        "G4900",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G4900 @ 3.1 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2018.0,
        "Coffee Lake",
        "G4900T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G4900T @ 2.9 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2018.0,
        "Coffee Lake",
        "G4920",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G4920 @ 3.2 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2019.0,
        "Coffee Lake",
        "G4930",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G4930 @ 3.2 GHz",
    ],
    [
        2400000000,
        2400000000.0,
        "Intel Celeron Processor G Series",
        1,
        2019.0,
        "Coffee Lake",
        "G4930E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G4930E @ 2.4 GHz",
    ],
    [
        3000000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2019.0,
        "Coffee Lake",
        "G4930T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G4930T @ 3.0 GHz",
    ],
    [
        1900000000,
        1900000000.0,
        "Intel Celeron Processor G Series",
        1,
        2019.0,
        "Coffee Lake",
        "G4932E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G4932E @ 1.9 GHz",
    ],
    [
        3300000000,
        None,
        "Intel Celeron Processor G Series",
        1,
        2019.0,
        "Coffee Lake",
        "G4950",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor G Series G4950 @ 3.3 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G530",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G530 @ 2.4 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G530T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G530T @ 2.0 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G540",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G540 @ 2.5 GHz",
    ],
    [
        3700000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2018.0,
        "Coffee Lake",
        "G5400",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5400 @ 3.7 GHz",
    ],
    [
        3100000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2018.0,
        "Coffee Lake",
        "G5400T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5400T @ 3.1 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G540T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G540T @ 2.1 GHz",
    ],
    [
        3800000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2019.0,
        "Coffee Lake",
        "G5420",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5420 @ 3.8 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2019.0,
        "Coffee Lake",
        "G5420T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5420T @ 3.2 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G550",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G550 @ 2.6 GHz",
    ],
    [
        3800000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2018.0,
        "Coffee Lake",
        "G5500",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5500 @ 3.8 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2018.0,
        "Coffee Lake",
        "G5500T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5500T @ 3.2 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G550T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G550T @ 2.2 GHz",
    ],
    [
        2700000000,
        None,
        "Legacy Intel Celeron Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G555",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor G555 @ 2.7 GHz",
    ],
    [
        3900000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2018.0,
        "Coffee Lake",
        "G5600",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5600 @ 3.9 GHz",
    ],
    [
        2600000000,
        3100000000.0,
        "Intel Pentium Gold Processor Series",
        1,
        2019.0,
        "Coffee Lake",
        "G5600E",
        2,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5600E @ 2.6 GHz",
    ],
    [
        3300000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2019.0,
        "Coffee Lake",
        "G5600T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5600T @ 3.3 GHz",
    ],
    [
        4000000000,
        None,
        "Intel Pentium Gold Processor Series",
        1,
        2019.0,
        "Coffee Lake",
        "G5620",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Gold Processor Series G5620 @ 4.0 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G620",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G620 @ 2.6 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G620T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G620T @ 2.2 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G622",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G622 @ 2.6 GHz",
    ],
    [
        2700000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G630",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G630 @ 2.7 GHz",
    ],
    [
        2300000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G630T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G630T @ 2.3 GHz",
    ],
    [
        2700000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G632",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G632 @ 2.7 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G640",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G640 @ 2.8 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G640T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G640T @ 2.4 GHz",
    ],
    [
        2900000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G645",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G645 @ 2.9 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G645T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G645T @ 2.5 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Clarkdale",
        "G6950",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G6950 @ 2.8 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2011.0,
        "Clarkdale",
        "G6951",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G6951 @ 2.8 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2011.0,
        "Clarkdale",
        "G6960",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G6960 @ 2.9 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G840",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G840 @ 2.8 GHz",
    ],
    [
        2900000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G850",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G850 @ 2.9 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2011.0,
        "Sandy Bridge",
        "G860",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G860 @ 3.0 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G860T",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G860T @ 2.6 GHz",
    ],
    [
        3100000000,
        None,
        "Legacy Intel Pentium Processor",
        1,
        2012.0,
        "Sandy Bridge",
        "G870",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor G870 @ 3.1 GHz",
    ],
    [
        2410000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2013.0,
        "Bay Trail",
        "J1750",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J1750 @ 2.4 GHz",
    ],
    [
        2410000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2013.0,
        "Bay Trail",
        "J1800",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J1800 @ 2.4 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2013.0,
        "Bay Trail",
        "J1850",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J1850 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2013.0,
        "Bay Trail",
        "J1900",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J1900 @ 2.0 GHz",
    ],
    [
        2410000000,
        None,
        "Intel Pentium Processor J Series",
        0,
        2013.0,
        "Bay Trail",
        "J2850",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor J Series J2850 @ 2.4 GHz",
    ],
    [
        2410000000,
        None,
        "Intel Pentium Processor J Series",
        0,
        2013.0,
        "Bay Trail",
        "J2900",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor J Series J2900 @ 2.4 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2016.0,
        "Braswell",
        "J3060",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J3060 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2016.0,
        "Braswell",
        "J3160",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J3160 @ 1.6 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2016.0,
        "Apollo Lake",
        "J3355",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J3355 @ 2.0 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2016.0,
        "Apollo Lake",
        "J3455",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J3455 @ 1.5 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Pentium Processor J Series",
        0,
        2016.0,
        "Braswell",
        "J3710",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor J Series J3710 @ 1.6 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2017.0,
        "Gemini Lake",
        "J4005",
        2,
        "2",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J4005 @ 2.0 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Celeron Processor J Series",
        0,
        2017.0,
        "Gemini Lake",
        "J4105",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Celeron Processor J Series J4105 @ 1.5 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Pentium Processor J Series",
        0,
        2016.0,
        "Apollo Lake",
        "J4205",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Processor J Series J4205 @ 1.5 GHz",
    ],
    [
        1500000000,
        None,
        "Intel Pentium Silver Processor Series",
        0,
        2017.0,
        "Gemini Lake",
        "J5005",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Pentium Silver Processor Series J5005 @ 1.5 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "L2300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors L2300 @ 1.5 GHz",
    ],
    [
        1660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Yonah",
        "L2400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors L2400 @ 1.7 GHz",
    ],
    [
        1830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "L2500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors L2500 @ 1.8 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Wolfdale",
        "L3014",
        1,
        1,
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L3014 @ 2.4 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Wolfdale",
        "L3110",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L3110 @ 3.0 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Yorkfield",
        "L3360",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L3360 @ 2.8 GHz",
    ],
    [
        2260000000,
        2530000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Clarkdale",
        "L3406",
        2,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L3406 @ 2.3 GHz",
    ],
    [
        1860000000,
        3200000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Lynnfield",
        "L3426",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L3426 @ 1.9 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Wolfdale",
        "L5215",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5215 @ 1.9 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Wolfdale",
        "L5238",
        2,
        2,
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5238 @ 2.7 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Wolfdale",
        "L5240",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5240 @ 3.0 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Wolfdale",
        "L5248",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5248 @ 3.0 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Clovertown",
        "L5310",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5310 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Clovertown",
        "L5318",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5318 @ 1.6 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Clovertown",
        "L5320",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5320 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Clovertown",
        "L5335",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5335 @ 2.0 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Harpertown",
        "L5408",
        4,
        4,
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5408 @ 2.1 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Harpertown",
        "L5410",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5410 @ 2.3 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Harpertown",
        "L5420",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5420 @ 2.5 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Harpertown",
        "L5430",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5430 @ 2.7 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "L5506",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5506 @ 2.1 GHz",
    ],
    [
        2000000000,
        2400000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "L5508",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5508 @ 2.0 GHz",
    ],
    [
        2130000000,
        2400000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "L5518",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5518 @ 2.1 GHz",
    ],
    [
        2260000000,
        2480000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "L5520",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5520 @ 2.3 GHz",
    ],
    [
        2400000000,
        2660000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "L5530",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5530 @ 2.4 GHz",
    ],
    [
        1860000000,
        1860000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "L5609",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5609 @ 1.9 GHz",
    ],
    [
        1870000000,
        2260000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Westmere EP",
        "L5618",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5618 @ 1.9 GHz",
    ],
    [
        2130000000,
        2400000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "L5630",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5630 @ 2.1 GHz",
    ],
    [
        2000000000,
        2400000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Westmere EP",
        "L5638",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5638 @ 2.0 GHz",
    ],
    [
        2260000000,
        2800000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "L5640",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L5640 @ 2.3 GHz",
    ],
    [
        1330000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "L7200",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors L7200 @ 1.3 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "L7300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors L7300 @ 1.4 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Tigerton",
        "L7345",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L7345 @ 1.9 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Merom",
        "L7400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors L7400 @ 1.5 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Dunnington",
        "L7445",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L7445 @ 2.1 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Dunnington",
        "L7455",
        6,
        6,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L7455 @ 2.1 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Merom",
        "L7500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors L7500 @ 1.6 GHz",
    ],
    [
        1870000000,
        2530000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "L7545",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L7545 @ 1.9 GHz",
    ],
    [
        1870000000,
        2530000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "L7555",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors L7555 @ 1.9 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "L7700",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors L7700 @ 1.8 GHz",
    ],
    [
        1730000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Jasper Forest",
        "LC3518",
        1,
        "1",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors LC3518 @ 1.7 GHz",
    ],
    [
        1730000000,
        1870000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Jasper Forest",
        "LC3528",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors LC3528 @ 1.7 GHz",
    ],
    [
        1730000000,
        2130000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Jasper Forest",
        "LC5518",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors LC5518 @ 1.7 GHz",
    ],
    [
        2130000000,
        2530000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Jasper Forest",
        "LC5528",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors LC5528 @ 2.1 GHz",
    ],
    [
        900000000,
        2200000000.0,
        "6th Generation Intel Core m Processors",
        3,
        2015.0,
        "Skylake",
        "M3-6Y30",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core m Processors M3-6Y30 @ 0.9 GHz",
    ],
    [
        1000000000,
        2600000000.0,
        "7th Generation Intel Core m Processors",
        3,
        2016.0,
        "Kaby Lake",
        "M3-7Y30",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core m Processors M3-7Y30 @ 1.0 GHz",
    ],
    [
        1100000000,
        3000000000.0,
        "7th Generation Intel Core m Processors",
        3,
        2017.0,
        "Kaby Lake",
        "M3-7Y32",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core m Processors M3-7Y32 @ 1.1 GHz",
    ],
    [
        1100000000,
        2700000000.0,
        "6th Generation Intel Core m Processors",
        3,
        2015.0,
        "Skylake",
        "M5-6Y54",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core m Processors M5-6Y54 @ 1.1 GHz",
    ],
    [
        1100000000,
        2800000000.0,
        "6th Generation Intel Core m Processors",
        3,
        2015.0,
        "Skylake",
        "M5-6Y57",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core m Processors M5-6Y57 @ 1.1 GHz",
    ],
    [
        1200000000,
        3100000000.0,
        "6th Generation Intel Core m Processors",
        3,
        2015.0,
        "Skylake",
        "M7-6Y75",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core m Processors M7-6Y75 @ 1.2 GHz",
    ],
    [
        1460000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2013.0,
        "Bay Trail",
        "N2805",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2805 @ 1.5 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2013.0,
        "Bay Trail",
        "N2806",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2806 @ 1.6 GHz",
    ],
    [
        1580000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2014.0,
        "Bay Trail",
        "N2807",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2807 @ 1.6 GHz",
    ],
    [
        1580000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2014.0,
        "Bay Trail",
        "N2808",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2808 @ 1.6 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2013.0,
        "Bay Trail",
        "N2810",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2810 @ 2.0 GHz",
    ],
    [
        1860000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2013.0,
        "Bay Trail",
        "N2815",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2815 @ 1.9 GHz",
    ],
    [
        2130000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2013.0,
        "Bay Trail",
        "N2820",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2820 @ 2.1 GHz",
    ],
    [
        2160000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2014.0,
        "Bay Trail",
        "N2830",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2830 @ 2.2 GHz",
    ],
    [
        2160000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2014.0,
        "Bay Trail",
        "N2840",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2840 @ 2.2 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2013.0,
        "Bay Trail",
        "N2910",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2910 @ 1.6 GHz",
    ],
    [
        1860000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2013.0,
        "Bay Trail",
        "N2920",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2920 @ 1.9 GHz",
    ],
    [
        1830000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2014.0,
        "Bay Trail",
        "N2930",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2930 @ 1.8 GHz",
    ],
    [
        1830000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2014.0,
        "Bay Trail",
        "N2940",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N2940 @ 1.8 GHz",
    ],
    [
        1040000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2015.0,
        "Braswell",
        "N3000",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N3000 @ 1.0 GHz",
    ],
    [
        1040000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2016.0,
        "Braswell",
        "N3010",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N3010 @ 1.0 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2015.0,
        "Braswell",
        "N3050",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N3050 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2016.0,
        "Braswell",
        "N3060",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N3060 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2015.0,
        "Braswell",
        "N3150",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N3150 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2016.0,
        "Braswell",
        "N3160",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N3160 @ 1.6 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2016.0,
        "Apollo Lake",
        "N3350",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N3350 @ 1.1 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2016.0,
        "Apollo Lake",
        "N3450",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N3450 @ 1.1 GHz",
    ],
    [
        2000000000,
        None,
        "Intel Pentium Processor N Series",
        0,
        2013.0,
        "Bay Trail",
        "N3510",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor N Series N3510 @ 2.0 GHz",
    ],
    [
        2170000000,
        None,
        "Intel Pentium Processor N Series",
        0,
        2013.0,
        "Bay Trail",
        "N3520",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor N Series N3520 @ 2.2 GHz",
    ],
    [
        2160000000,
        None,
        "Intel Pentium Processor N Series",
        0,
        2014.0,
        "Bay Trail",
        "N3530",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor N Series N3530 @ 2.2 GHz",
    ],
    [
        2160000000,
        None,
        "Intel Pentium Processor N Series",
        0,
        2014.0,
        "Bay Trail",
        "N3540",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor N Series N3540 @ 2.2 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Pentium Processor N Series",
        0,
        2015.0,
        "Braswell",
        "N3700",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor N Series N3700 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Intel Pentium Processor N Series",
        0,
        2016.0,
        "Braswell",
        "N3710",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor N Series N3710 @ 1.6 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2017.0,
        "Gemini Lake",
        "N4000",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N4000 @ 1.1 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Celeron Processor N Series",
        0,
        2017.0,
        "Gemini Lake",
        "N4100",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Celeron Processor N Series N4100 @ 1.1 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Pentium Processor N Series",
        0,
        2016.0,
        "Apollo Lake",
        "N4200",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Processor N Series N4200 @ 1.1 GHz",
    ],
    [
        1100000000,
        None,
        "Intel Pentium Silver Processor Series",
        0,
        2017.0,
        "Gemini Lake",
        "N5000",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Intel Pentium Silver Processor Series N5000 @ 1.1 GHz",
    ],
    [
        1330000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2010.0,
        "Jasper Forest",
        "P1053",
        1,
        "2",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor P1053 @ 1.3 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2010.0,
        "Arrandale",
        "P4500",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor P4500 @ 1.9 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2010.0,
        "Arrandale",
        "P4505",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor P4505 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2010.0,
        "Arrandale",
        "P4600",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor P4600 @ 2.0 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Arrandale",
        "P6000",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor P6000 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Arrandale",
        "P6100",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor P6100 @ 2.0 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Arrandale",
        "P6200",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor P6200 @ 2.1 GHz",
    ],
    [
        2270000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2011.0,
        "Arrandale",
        "P6300",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor P6300 @ 2.3 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "P7350",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P7350 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "P7370",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P7370 @ 2.0 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "P7450",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P7450 @ 2.1 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "P7550",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P7550 @ 2.3 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "P7570",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P7570 @ 2.3 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "P8400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P8400 @ 2.3 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "P8600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P8600 @ 2.4 GHz",
    ],
    [
        2530000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "P8700",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P8700 @ 2.5 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "P8800",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P8800 @ 2.7 GHz",
    ],
    [
        2530000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "P9500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P9500 @ 2.5 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "P9600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P9600 @ 2.7 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "P9700",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors P9700 @ 2.8 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Kentsfield",
        "Q6600",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q6600 @ 2.4 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Kentsfield",
        "Q6700",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q6700 @ 2.7 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Yorkfield",
        "Q8200",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q8200 @ 2.3 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Yorkfield",
        "Q8200S",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q8200S @ 2.3 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Yorkfield",
        "Q8300",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q8300 @ 2.5 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Yorkfield",
        "Q8400",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q8400 @ 2.7 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Yorkfield",
        "Q8400S",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q8400S @ 2.7 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "Q9000",
        4,
        4,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9000 @ 2.0 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "Q9100",
        4,
        4,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9100 @ 2.3 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Yorkfield",
        "Q9300",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9300 @ 2.5 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Yorkfield",
        "Q9400",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9400 @ 2.7 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Yorkfield",
        "Q9400S",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9400S @ 2.7 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Yorkfield",
        "Q9450",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9450 @ 2.7 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yorkfield",
        "Q9500",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9500 @ 2.8 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Yorkfield",
        "Q9505",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9505 @ 2.8 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Yorkfield",
        "Q9505S",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9505S @ 2.8 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Yorkfield",
        "Q9550",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9550 @ 2.8 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Yorkfield",
        "Q9550S",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9550S @ 2.8 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Yorkfield",
        "Q9650",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors Q9650 @ 3.0 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Kentsfield",
        "QX6700",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors QX6700 @ 2.7 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Kentsfield",
        "QX6800",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors QX6800 @ 2.9 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Kentsfield",
        "QX6850",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors QX6850 @ 3.0 GHz",
    ],
    [
        2530000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "QX9300",
        4,
        4,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors QX9300 @ 2.5 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Yorkfield",
        "QX9650",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors QX9650 @ 3.0 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Yorkfield",
        "QX9770",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors QX9770 @ 3.2 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Yorkfield",
        "QX9775",
        4,
        4,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors QX9775 @ 3.2 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "SL3400",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SL3400 @ 1.9 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "SL9300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SL9300 @ 1.6 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "SL9380",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SL9380 @ 1.8 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "SL9400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SL9400 @ 1.9 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "SL9600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SL9600 @ 2.1 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "SP9300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SP9300 @ 2.3 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "SP9400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SP9400 @ 2.4 GHz",
    ],
    [
        2530000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "SP9600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SP9600 @ 2.5 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2009.0,
        "Penryn",
        "SU2300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor SU2300 @ 1.2 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2009.0,
        "Penryn",
        "SU2700",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor SU2700 @ 1.3 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "SU3300",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SU3300 @ 1.2 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "SU3500",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SU3500 @ 1.3 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2009.0,
        "Penryn",
        "SU4100",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor SU4100 @ 1.3 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Core Processors",
        1,
        None,
        "Penryn",
        "SU7300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SU7300 @ 1.3 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "SU9300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SU9300 @ 1.2 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "SU9400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SU9400 @ 1.4 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "SU9600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors SU9600 @ 1.6 GHz",
    ],
    [
        1730000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T1250",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T1250 @ 1.7 GHz",
    ],
    [
        1660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T1300",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T1300 @ 1.7 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T1350",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T1350 @ 1.9 GHz",
    ],
    [
        1830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T1400",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T1400 @ 1.8 GHz",
    ],
    [
        1660000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2008.0,
        "Merom",
        "T1600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor T1600 @ 1.7 GHz",
    ],
    [
        1830000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2008.0,
        "Merom",
        "T1700",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor T1700 @ 1.8 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T2050",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2050 @ 1.6 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2007.0,
        "Yonah",
        "T2060",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T2060 @ 1.6 GHz",
    ],
    [
        1730000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2007.0,
        "Yonah",
        "T2080",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T2080 @ 1.7 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2007.0,
        "Yonah",
        "T2130",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T2130 @ 1.9 GHz",
    ],
    [
        1730000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T2250",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2250 @ 1.7 GHz",
    ],
    [
        1660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T2300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2300 @ 1.7 GHz",
    ],
    [
        1660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T2300E",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2300E @ 1.7 GHz",
    ],
    [
        1460000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2007.0,
        "Merom",
        "T2310",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T2310 @ 1.5 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2007.0,
        "Merom",
        "T2330",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T2330 @ 1.6 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T2350",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2350 @ 1.9 GHz",
    ],
    [
        1730000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2008.0,
        "Merom",
        "T2370",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T2370 @ 1.7 GHz",
    ],
    [
        1860000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Merom",
        "T2390",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T2390 @ 1.9 GHz",
    ],
    [
        1830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T2400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2400 @ 1.8 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T2450",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2450 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Yonah",
        "T2500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2500 @ 2.0 GHz",
    ],
    [
        2160000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T2600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2600 @ 2.2 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "T2700",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T2700 @ 2.3 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2009.0,
        "Penryn",
        "T3000",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor T3000 @ 1.8 GHz",
    ],
    [
        1900000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2008.0,
        "Penryn",
        "T3100",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor T3100 @ 1.9 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Merom",
        "T3200",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T3200 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2010.0,
        "Penryn",
        "T3300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor T3300 @ 2.0 GHz",
    ],
    [
        2160000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2008.0,
        "Merom",
        "T3400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T3400 @ 2.2 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2010.0,
        "Penryn",
        "T3500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor T3500 @ 2.1 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2009.0,
        "Penryn",
        "T4200",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T4200 @ 2.0 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        None,
        "Penryn",
        "T4300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T4300 @ 2.1 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2009.0,
        "Penryn",
        "T4400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T4400 @ 2.2 GHz",
    ],
    [
        2300000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Penryn",
        "T4500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor T4500 @ 2.3 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5200",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5200 @ 1.6 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5250",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5250 @ 1.5 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5270",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5270 @ 1.4 GHz",
    ],
    [
        1730000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5300 @ 1.7 GHz",
    ],
    [
        1660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5450",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5450 @ 1.7 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5470",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5470 @ 1.6 GHz",
    ],
    [
        1660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5500 @ 1.7 GHz",
    ],
    [
        1830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5550",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5550 @ 1.8 GHz",
    ],
    [
        1830000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5600 @ 1.8 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Merom",
        "T5670",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5670 @ 1.8 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Merom",
        "T5750",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5750 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Merom",
        "T5800",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5800 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T5870",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T5870 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "T6400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T6400 @ 2.0 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "T6500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T6500 @ 2.1 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "T6570",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T6570 @ 2.1 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "T6600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T6600 @ 2.2 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "T6670",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T6670 @ 2.2 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "T7100",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T7100 @ 1.8 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "T7200",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T7200 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "T7250",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T7250 @ 2.0 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "T7300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T7300 @ 2.0 GHz",
    ],
    [
        2160000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Merom",
        "T7400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T7400 @ 2.2 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Merom",
        "T7500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T7500 @ 2.2 GHz",
    ],
    [
        2330000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "T7600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T7600 @ 2.3 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "T7700",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T7700 @ 2.4 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "T7800",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T7800 @ 2.6 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "T8100",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T8100 @ 2.1 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "T8300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T8300 @ 2.4 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "T9300",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T9300 @ 2.5 GHz",
    ],
    [
        2530000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "T9400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T9400 @ 2.5 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "T9500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T9500 @ 2.6 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "T9550",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T9550 @ 2.7 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "T9600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T9600 @ 2.8 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "T9800",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T9800 @ 2.9 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2009.0,
        "Penryn",
        "T9900",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors T9900 @ 3.1 GHz",
    ],
    [
        1060000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "U1300",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U1300 @ 1.1 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "U1400",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U1400 @ 1.2 GHz",
    ],
    [
        1330000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "U1500",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U1500 @ 1.3 GHz",
    ],
    [
        1060000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "U2100",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U2100 @ 1.1 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "U2200",
        1,
        1,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U2200 @ 1.2 GHz",
    ],
    [
        1060000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Yonah",
        "U2400",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U2400 @ 1.1 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Yonah",
        "U2500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U2500 @ 1.2 GHz",
    ],
    [
        1060000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2010.0,
        "Arrandale",
        "U3400",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor U3400 @ 1.1 GHz",
    ],
    [
        1060000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2010.0,
        "Arrandale",
        "U3405",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor U3405 @ 1.1 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Celeron Processor",
        0,
        2011.0,
        "Arrandale",
        "U3600",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Celeron Processor U3600 @ 1.2 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2010.0,
        "Arrandale",
        "U5400",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor U5400 @ 1.2 GHz",
    ],
    [
        1330000000,
        None,
        "Legacy Intel Pentium Processor",
        0,
        2011.0,
        "Arrandale",
        "U5600",
        2,
        "2",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Pentium Processor U5600 @ 1.3 GHz",
    ],
    [
        1060000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Merom",
        "U7500",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U7500 @ 1.1 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Core Processors",
        0,
        None,
        "Merom",
        "U7600",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U7600 @ 1.2 GHz",
    ],
    [
        1330000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Merom",
        "U7700",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors U7700 @ 1.3 GHz",
    ],
    [
        2900000000,
        None,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2102",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2102 @ 2.9 GHz",
    ],
    [
        3200000000,
        None,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2104",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2104 @ 3.2 GHz",
    ],
    [
        3600000000,
        3900000000.0,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2123",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2123 @ 3.6 GHz",
    ],
    [
        4000000000,
        4500000000.0,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2125",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2125 @ 4.0 GHz",
    ],
    [
        3600000000,
        3900000000.0,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2133",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2133 @ 3.6 GHz",
    ],
    [
        3700000000,
        4500000000.0,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2135",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2135 @ 3.7 GHz",
    ],
    [
        3700000000,
        4500000000.0,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2145",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2145 @ 3.7 GHz",
    ],
    [
        3300000000,
        4500000000.0,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2155",
        10,
        "20",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2155 @ 3.3 GHz",
    ],
    [
        2500000000,
        4300000000.0,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2175",
        14,
        "28",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2175 @ 2.5 GHz",
    ],
    [
        2300000000,
        4300000000.0,
        "Intel Xeon W Processor",
        4,
        2017.0,
        "Skylake",
        "W-2195",
        18,
        "36",
        "Server",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-2195 @ 2.3 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "Intel Xeon W Processor",
        4,
        2018.0,
        "Skylake",
        "W-3175X",
        28,
        "56",
        "Desktop",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3175X @ 3.1 GHz",
    ],
    [
        3500000000,
        4000000000.0,
        "Intel Xeon W Processor",
        4,
        2019.0,
        "Cascade Lake",
        "W-3223",
        8,
        "16",
        "Workstation",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3223 @ 3.5 GHz",
    ],
    [
        3700000000,
        4300000000.0,
        "Intel Xeon W Processor",
        4,
        2019.0,
        "Cascade Lake",
        "W-3225",
        8,
        "16",
        "Workstation",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3225 @ 3.7 GHz",
    ],
    [
        3300000000,
        4400000000.0,
        "Intel Xeon W Processor",
        4,
        2019.0,
        "Cascade Lake",
        "W-3235",
        12,
        "24",
        "Workstation",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3235 @ 3.3 GHz",
    ],
    [
        3200000000,
        4400000000.0,
        "Intel Xeon W Processor",
        4,
        2019.0,
        "Cascade Lake",
        "W-3245",
        16,
        "32",
        "Workstation",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3245 @ 3.2 GHz",
    ],
    [
        3200000000,
        4400000000.0,
        "Intel Xeon W Processor",
        4,
        2019.0,
        "Cascade Lake",
        "W-3245M",
        16,
        "32",
        "Workstation",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3245M @ 3.2 GHz",
    ],
    [
        2700000000,
        4400000000.0,
        "Intel Xeon W Processor",
        4,
        2019.0,
        "Cascade Lake",
        "W-3265",
        24,
        "48",
        "Workstation",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3265 @ 2.7 GHz",
    ],
    [
        2700000000,
        4400000000.0,
        "Intel Xeon W Processor",
        4,
        2019.0,
        "Cascade Lake",
        "W-3265M",
        24,
        "48",
        "Workstation",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3265M @ 2.7 GHz",
    ],
    [
        2500000000,
        4400000000.0,
        "Intel Xeon W Processor",
        4,
        2019.0,
        "Cascade Lake",
        "W-3275",
        28,
        "56",
        "Workstation",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3275 @ 2.5 GHz",
    ],
    [
        2500000000,
        4400000000.0,
        "Intel Xeon W Processor",
        4,
        2019.0,
        "Cascade Lake",
        "W-3275M",
        28,
        "56",
        "Workstation",
        "cpu",
        "intel",
        " Intel Xeon W Processor W-3275M @ 2.5 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Bloomfield",
        "W3503",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3503 @ 2.4 GHz",
    ],
    [
        2530000000,
        None,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Bloomfield",
        "W3505",
        2,
        "2",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3505 @ 2.5 GHz",
    ],
    [
        2660000000,
        2930000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Bloomfield",
        "W3520",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3520 @ 2.7 GHz",
    ],
    [
        2800000000,
        3060000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Bloomfield",
        "W3530",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3530 @ 2.8 GHz",
    ],
    [
        2930000000,
        3200000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Bloomfield",
        "W3540",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3540 @ 2.9 GHz",
    ],
    [
        3060000000,
        3330000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Bloomfield",
        "W3550",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3550 @ 3.1 GHz",
    ],
    [
        3200000000,
        3460000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Bloomfield",
        "W3565",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3565 @ 3.2 GHz",
    ],
    [
        3200000000,
        3460000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Bloomfield",
        "W3570",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3570 @ 3.2 GHz",
    ],
    [
        3330000000,
        3600000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Bloomfield",
        "W3580",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3580 @ 3.3 GHz",
    ],
    [
        3200000000,
        3460000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "W3670",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3670 @ 3.2 GHz",
    ],
    [
        3330000000,
        3600000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "W3680",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3680 @ 3.3 GHz",
    ],
    [
        3460000000,
        3730000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Gulftown",
        "W3690",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W3690 @ 3.5 GHz",
    ],
    [
        3200000000,
        3460000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "W5580",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W5580 @ 3.2 GHz",
    ],
    [
        3330000000,
        3600000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "W5590",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors W5590 @ 3.3 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Kentsfield",
        "X3210",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3210 @ 2.1 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Kentsfield",
        "X3220",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3220 @ 2.4 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Kentsfield",
        "X3230",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3230 @ 2.7 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Yorkfield",
        "X3320",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3320 @ 2.5 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Yorkfield",
        "X3323",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3323 @ 2.5 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Yorkfield",
        "X3330",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3330 @ 2.7 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Yorkfield",
        "X3350",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3350 @ 2.7 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Yorkfield",
        "X3353",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3353 @ 2.7 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Yorkfield",
        "X3360",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3360 @ 2.8 GHz",
    ],
    [
        2830000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        None,
        "Yorkfield",
        "X3363",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3363 @ 2.8 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Yorkfield",
        "X3370",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3370 @ 3.0 GHz",
    ],
    [
        3160000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Yorkfield",
        "X3380",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3380 @ 3.2 GHz",
    ],
    [
        2400000000,
        2800000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Lynnfield",
        "X3430",
        4,
        "4",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3430 @ 2.4 GHz",
    ],
    [
        2530000000,
        2930000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Lynnfield",
        "X3440",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3440 @ 2.5 GHz",
    ],
    [
        2660000000,
        3200000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Lynnfield",
        "X3450",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3450 @ 2.7 GHz",
    ],
    [
        2800000000,
        3460000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Lynnfield",
        "X3460",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3460 @ 2.8 GHz",
    ],
    [
        2930000000,
        3600000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2009.0,
        "Lynnfield",
        "X3470",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3470 @ 2.9 GHz",
    ],
    [
        3060000000,
        3730000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Lynnfield",
        "X3480",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X3480 @ 3.1 GHz",
    ],
    [
        3330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Wolfdale",
        "X5260",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5260 @ 3.3 GHz",
    ],
    [
        3500000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Wolfdale",
        "X5270",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5270 @ 3.5 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Wolfdale",
        "X5272",
        2,
        2,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5272 @ 3.4 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2006.0,
        "Clovertown",
        "X5355",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5355 @ 2.7 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Clovertown",
        "X5365",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5365 @ 3.0 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "X5450",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5450 @ 3.0 GHz",
    ],
    [
        3160000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "X5460",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5460 @ 3.2 GHz",
    ],
    [
        3330000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Harpertown",
        "X5470",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5470 @ 3.3 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "X5472",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5472 @ 3.0 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Harpertown",
        "X5482",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5482 @ 3.2 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Harpertown",
        "X5492",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5492 @ 3.4 GHz",
    ],
    [
        2660000000,
        3060000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "X5550",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5550 @ 2.7 GHz",
    ],
    [
        2800000000,
        3200000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "X5560",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5560 @ 2.8 GHz",
    ],
    [
        2930000000,
        3330000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2009.0,
        "Nehalem EP",
        "X5570",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5570 @ 2.9 GHz",
    ],
    [
        2930000000,
        3200000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Westmere EP",
        "X5647",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5647 @ 2.9 GHz",
    ],
    [
        2660000000,
        3060000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "X5650",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5650 @ 2.7 GHz",
    ],
    [
        2800000000,
        3200000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "X5660",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5660 @ 2.8 GHz",
    ],
    [
        3060000000,
        3460000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "X5667",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5667 @ 3.1 GHz",
    ],
    [
        2930000000,
        3330000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "X5670",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5670 @ 2.9 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Westmere EP",
        "X5672",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5672 @ 3.2 GHz",
    ],
    [
        3060000000,
        3460000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Westmere EP",
        "X5675",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5675 @ 3.1 GHz",
    ],
    [
        3460000000,
        3730000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "X5677",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5677 @ 3.5 GHz",
    ],
    [
        3330000000,
        3600000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2010.0,
        "Westmere EP",
        "X5680",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5680 @ 3.3 GHz",
    ],
    [
        3600000000,
        3860000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Westmere EP",
        "X5687",
        4,
        "8",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5687 @ 3.6 GHz",
    ],
    [
        3460000000,
        3730000000.0,
        "Legacy Intel Xeon Processors",
        1,
        2011.0,
        "Westmere EP",
        "X5690",
        6,
        "12",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X5690 @ 3.5 GHz",
    ],
    [
        2000000000,
        2400000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "X6550",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X6550 @ 2.0 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2006.0,
        "Conroe",
        "X6800",
        2,
        2,
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors X6800 @ 2.9 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2007.0,
        "Tigerton",
        "X7350",
        4,
        4,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X7350 @ 2.9 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Xeon Processors",
        0,
        2008.0,
        "Dunnington",
        "X7460",
        6,
        6,
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X7460 @ 2.7 GHz",
    ],
    [
        2670000000,
        2800000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "X7542",
        6,
        "6",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X7542 @ 2.7 GHz",
    ],
    [
        2000000000,
        2400000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "X7550",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X7550 @ 2.0 GHz",
    ],
    [
        2270000000,
        2670000000.0,
        "Legacy Intel Xeon Processors",
        0,
        2010.0,
        "Nehalem EX",
        "X7560",
        8,
        "16",
        "Server",
        "cpu",
        "intel",
        " Legacy Intel Xeon Processors X7560 @ 2.3 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "X7800",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors X7800 @ 2.6 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2007.0,
        "Merom",
        "X7900",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors X7900 @ 2.8 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "X9000",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors X9000 @ 2.8 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Core Processors",
        0,
        2008.0,
        "Penryn",
        "X9100",
        2,
        2,
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors X9100 @ 3.1 GHz",
    ],
    [
        3100000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2100",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2100 @ 3.1 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2100T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2100T @ 2.5 GHz",
    ],
    [
        3100000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2102",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2102 @ 3.1 GHz",
    ],
    [
        3100000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2105",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2105 @ 3.1 GHz",
    ],
    [
        2000000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Gladden",
        "i3-2115C",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2115C @ 2.0 GHz",
    ],
    [
        3300000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2120",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2120 @ 3.3 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2120T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2120T @ 2.6 GHz",
    ],
    [
        3300000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2125",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2125 @ 3.3 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2130",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2130 @ 3.4 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2310E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2310E @ 2.1 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2310M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2310M @ 2.1 GHz",
    ],
    [
        2100000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2312M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2312M @ 2.1 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Sandy Bridge",
        "i3-2328M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2328M @ 2.2 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2330E",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2330E @ 2.2 GHz",
    ],
    [
        2200000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2330M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2330M @ 2.2 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2340UE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2340UE @ 1.3 GHz",
    ],
    [
        2300000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Sandy Bridge",
        "i3-2348M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2348M @ 2.3 GHz",
    ],
    [
        2300000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2350M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2350M @ 2.3 GHz",
    ],
    [
        1300000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2357M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2357M @ 1.3 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Sandy Bridge",
        "i3-2365M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2365M @ 1.4 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i3-2367M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2367M @ 1.4 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Sandy Bridge",
        "i3-2370M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2370M @ 2.4 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Sandy Bridge",
        "i3-2375M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2375M @ 1.5 GHz",
    ],
    [
        1500000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Sandy Bridge",
        "i3-2377M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-2377M @ 1.5 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3110M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3110M @ 2.4 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Gladden",
        "i3-3115C",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3115C @ 2.5 GHz",
    ],
    [
        2500000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3120M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3120M @ 2.5 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3120ME",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3120ME @ 2.4 GHz",
    ],
    [
        2600000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i3-3130M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3130M @ 2.6 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i3-3210",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3210 @ 3.2 GHz",
    ],
    [
        1800000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3217U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3217U @ 1.8 GHz",
    ],
    [
        1600000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3217UE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3217UE @ 1.6 GHz",
    ],
    [
        3300000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3220",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3220 @ 3.3 GHz",
    ],
    [
        2800000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3220T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3220T @ 2.8 GHz",
    ],
    [
        3300000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3225",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3225 @ 3.3 GHz",
    ],
    [
        1900000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i3-3227U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3227U @ 1.9 GHz",
    ],
    [
        1400000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i3-3229Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3229Y @ 1.4 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3240",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3240 @ 3.4 GHz",
    ],
    [
        2900000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i3-3240T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3240T @ 2.9 GHz",
    ],
    [
        3400000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i3-3245",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3245 @ 3.4 GHz",
    ],
    [
        3500000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i3-3250",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3250 @ 3.5 GHz",
    ],
    [
        3000000000,
        None,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i3-3250T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-3250T @ 3.0 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i3-330E",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-330E @ 2.1 GHz",
    ],
    [
        2130000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i3-330M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-330M @ 2.1 GHz",
    ],
    [
        1200000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i3-330UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-330UM @ 1.2 GHz",
    ],
    [
        2260000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i3-350M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-350M @ 2.3 GHz",
    ],
    [
        2400000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i3-370M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-370M @ 2.4 GHz",
    ],
    [
        2530000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i3-380M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-380M @ 2.5 GHz",
    ],
    [
        1330000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i3-380UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-380UM @ 1.3 GHz",
    ],
    [
        2660000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2011.0,
        "Arrandale",
        "i3-390M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-390M @ 2.7 GHz",
    ],
    [
        2400000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4000M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4000M @ 2.4 GHz",
    ],
    [
        1700000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4005U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4005U @ 1.7 GHz",
    ],
    [
        1700000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4010U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4010U @ 1.7 GHz",
    ],
    [
        1300000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4010Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4010Y @ 1.3 GHz",
    ],
    [
        1500000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4012Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4012Y @ 1.5 GHz",
    ],
    [
        1500000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4020Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4020Y @ 1.5 GHz",
    ],
    [
        1900000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4025U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4025U @ 1.9 GHz",
    ],
    [
        1900000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4030U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4030U @ 1.9 GHz",
    ],
    [
        1600000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4030Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4030Y @ 1.6 GHz",
    ],
    [
        2400000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4100E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4100E @ 2.4 GHz",
    ],
    [
        2500000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4100M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4100M @ 2.5 GHz",
    ],
    [
        1800000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4100U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4100U @ 1.8 GHz",
    ],
    [
        1600000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4102E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4102E @ 1.6 GHz",
    ],
    [
        2600000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4110E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4110E @ 2.6 GHz",
    ],
    [
        2600000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4110M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4110M @ 2.6 GHz",
    ],
    [
        1800000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4112E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4112E @ 1.8 GHz",
    ],
    [
        2000000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4120U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4120U @ 2.0 GHz",
    ],
    [
        3400000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4130",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4130 @ 3.4 GHz",
    ],
    [
        2900000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4130T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4130T @ 2.9 GHz",
    ],
    [
        3500000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4150",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4150 @ 3.5 GHz",
    ],
    [
        3000000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4150T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4150T @ 3.0 GHz",
    ],
    [
        2000000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4158U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4158U @ 2.0 GHz",
    ],
    [
        3600000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4160",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4160 @ 3.6 GHz",
    ],
    [
        3100000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4160T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4160T @ 3.1 GHz",
    ],
    [
        3700000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Haswell",
        "i3-4170",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4170 @ 3.7 GHz",
    ],
    [
        3200000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Haswell",
        "i3-4170T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4170T @ 3.2 GHz",
    ],
    [
        3500000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4330",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4330 @ 3.5 GHz",
    ],
    [
        3000000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4330T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4330T @ 3.0 GHz",
    ],
    [
        2400000000,
        None,
        "4th Generation Intel Core i3 Processors",
        2,
        2013.0,
        "Haswell",
        "i3-4330TE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4330TE @ 2.4 GHz",
    ],
    [
        3600000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2013.0,
        "Haswell",
        "i3-4340",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4340 @ 3.6 GHz",
    ],
    [
        2600000000,
        None,
        "4th Generation Intel Core i3 Processors",
        2,
        2014.0,
        "Haswell",
        "i3-4340TE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4340TE @ 2.6 GHz",
    ],
    [
        3600000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4350",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4350 @ 3.6 GHz",
    ],
    [
        3100000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4350T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4350T @ 3.1 GHz",
    ],
    [
        3700000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4360",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4360 @ 3.7 GHz",
    ],
    [
        3200000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4360T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4360T @ 3.2 GHz",
    ],
    [
        3800000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2014.0,
        "Haswell",
        "i3-4370",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4370 @ 3.8 GHz",
    ],
    [
        3300000000,
        None,
        "4th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Haswell",
        "i3-4370T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i3 Processors i3-4370T @ 3.3 GHz",
    ],
    [
        2000000000,
        None,
        "5th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Broadwell",
        "i3-5005U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i3 Processors i3-5005U @ 2.0 GHz",
    ],
    [
        2100000000,
        None,
        "5th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Broadwell",
        "i3-5010U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i3 Processors i3-5010U @ 2.1 GHz",
    ],
    [
        2100000000,
        None,
        "5th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Broadwell",
        "i3-5015U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i3 Processors i3-5015U @ 2.1 GHz",
    ],
    [
        2200000000,
        None,
        "5th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Broadwell",
        "i3-5020U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i3 Processors i3-5020U @ 2.2 GHz",
    ],
    [
        2500000000,
        None,
        "5th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Broadwell",
        "i3-5157U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i3 Processors i3-5157U @ 2.5 GHz",
    ],
    [
        2930000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i3-530",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-530 @ 2.9 GHz",
    ],
    [
        3060000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i3-540",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-540 @ 3.1 GHz",
    ],
    [
        3200000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i3-550",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-550 @ 3.2 GHz",
    ],
    [
        3330000000,
        None,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i3-560",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i3-560 @ 3.3 GHz",
    ],
    [
        2000000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2016.0,
        "Skylake",
        "i3-6006U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6006U @ 2.0 GHz",
    ],
    [
        3600000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6098P",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6098P @ 3.6 GHz",
    ],
    [
        3700000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6100",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6100 @ 3.7 GHz",
    ],
    [
        2700000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6100E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6100E @ 2.7 GHz",
    ],
    [
        2700000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6100H",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6100H @ 2.7 GHz",
    ],
    [
        3200000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6100T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6100T @ 3.2 GHz",
    ],
    [
        2700000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6100TE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6100TE @ 2.7 GHz",
    ],
    [
        2300000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6100U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6100U @ 2.3 GHz",
    ],
    [
        1900000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6102E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6102E @ 1.9 GHz",
    ],
    [
        2400000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2016.0,
        "Skylake",
        "i3-6157U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6157U @ 2.4 GHz",
    ],
    [
        2700000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6167U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6167U @ 2.7 GHz",
    ],
    [
        3800000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6300",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6300 @ 3.8 GHz",
    ],
    [
        3300000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6300T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6300T @ 3.3 GHz",
    ],
    [
        3900000000,
        None,
        "6th Generation Intel Core i3 Processors",
        3,
        2015.0,
        "Skylake",
        "i3-6320",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i3 Processors i3-6320 @ 3.9 GHz",
    ],
    [
        2300000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2018.0,
        "Kaby Lake",
        "i3-7020U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7020U @ 2.3 GHz",
    ],
    [
        3900000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7100",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7100 @ 3.9 GHz",
    ],
    [
        2900000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7100E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7100E @ 2.9 GHz",
    ],
    [
        3000000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7100H",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7100H @ 3.0 GHz",
    ],
    [
        3400000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7100T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7100T @ 3.4 GHz",
    ],
    [
        2400000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2016.0,
        "Kaby Lake",
        "i3-7100U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7100U @ 2.4 GHz",
    ],
    [
        3900000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7101E",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7101E @ 3.9 GHz",
    ],
    [
        3400000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7101TE",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7101TE @ 3.4 GHz",
    ],
    [
        2100000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7102E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7102E @ 2.1 GHz",
    ],
    [
        2700000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7130U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7130U @ 2.7 GHz",
    ],
    [
        2800000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7167U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7167U @ 2.8 GHz",
    ],
    [
        4000000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7300",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7300 @ 4.0 GHz",
    ],
    [
        3500000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7300T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7300T @ 3.5 GHz",
    ],
    [
        4099999999,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7320",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7320 @ 4.1 GHz",
    ],
    [
        4200000000,
        None,
        "7th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i3-7350K",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i3 Processors i3-7350K @ 4.2 GHz",
    ],
    [
        3600000000,
        None,
        "8th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Coffee Lake",
        "i3-8100",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8100 @ 3.6 GHz",
    ],
    [
        3600000000,
        None,
        "8th Generation Intel Core i3 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i3-8100B",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8100B @ 3.6 GHz",
    ],
    [
        3000000000,
        None,
        "8th Generation Intel Core i3 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i3-8100H",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8100H @ 3.0 GHz",
    ],
    [
        3100000000,
        None,
        "8th Generation Intel Core i3 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i3-8100T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8100T @ 3.1 GHz",
    ],
    [
        3000000000,
        3600000000.0,
        "8th Generation Intel Core i3 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i3-8109U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8109U @ 3.0 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "8th Generation Intel Core i3 Processors",
        4,
        2018.0,
        "Cannon Lake",
        "i3-8121U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8121U @ 2.2 GHz",
    ],
    [
        2200000000,
        3400000000.0,
        "8th Generation Intel Core i3 Processors",
        3,
        2018.0,
        "Kaby Lake",
        "i3-8130U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8130U @ 2.2 GHz",
    ],
    [
        2100000000,
        3900000000.0,
        "8th Generation Intel Core i3 Processors",
        3,
        2018.0,
        "Whiskey Lake",
        "i3-8145U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8145U @ 2.1 GHz",
    ],
    [
        2200000000,
        3900000000.0,
        "8th Generation Intel Core i3 Processors",
        0,
        2019.0,
        "Whiskey Lake",
        "i3-8145UE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8145UE @ 2.2 GHz",
    ],
    [
        3700000000,
        None,
        "8th Generation Intel Core i3 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i3-8300",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8300 @ 3.7 GHz",
    ],
    [
        3200000000,
        None,
        "8th Generation Intel Core i3 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i3-8300T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8300T @ 3.2 GHz",
    ],
    [
        4000000000,
        None,
        "8th Generation Intel Core i3 Processors",
        3,
        2017.0,
        "Coffee Lake",
        "i3-8350K",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i3 Processors i3-8350K @ 4.0 GHz",
    ],
    [
        3600000000,
        4200000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9100",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9100 @ 3.6 GHz",
    ],
    [
        3100000000,
        3700000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9100E",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9100E @ 3.1 GHz",
    ],
    [
        3600000000,
        4200000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9100F",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9100F @ 3.6 GHz",
    ],
    [
        1600000000,
        2900000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9100HL",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9100HL @ 1.6 GHz",
    ],
    [
        3100000000,
        3700000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9100T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9100T @ 3.1 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9100TE",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9100TE @ 2.2 GHz",
    ],
    [
        3700000000,
        4300000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9300",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9300 @ 3.7 GHz",
    ],
    [
        3200000000,
        3800000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9300T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9300T @ 3.2 GHz",
    ],
    [
        3700000000,
        4400000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9320",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9320 @ 3.7 GHz",
    ],
    [
        4000000000,
        4600000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9350K",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9350K @ 4.0 GHz",
    ],
    [
        4000000000,
        4600000000.0,
        "9th Generation Intel Core i3 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i3-9350KF",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i3 Processors i3-9350KF @ 4.0 GHz",
    ],
    [
        2800000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2300",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2300 @ 2.8 GHz",
    ],
    [
        2900000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2310",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2310 @ 2.9 GHz",
    ],
    [
        3000000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2320",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2320 @ 3.0 GHz",
    ],
    [
        3100000000,
        3400000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Sandy Bridge",
        "i5-2380P",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2380P @ 3.1 GHz",
    ],
    [
        2700000000,
        3500000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2390T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2390T @ 2.7 GHz",
    ],
    [
        3100000000,
        3400000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2400",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2400 @ 3.1 GHz",
    ],
    [
        2500000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2400S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2400S @ 2.5 GHz",
    ],
    [
        2500000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2405S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2405S @ 2.5 GHz",
    ],
    [
        2300000000,
        2900000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2410M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2410M @ 2.3 GHz",
    ],
    [
        2300000000,
        2900000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2415M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2415M @ 2.3 GHz",
    ],
    [
        2400000000,
        3000000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2430M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2430M @ 2.4 GHz",
    ],
    [
        2400000000,
        3000000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2435M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2435M @ 2.4 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Sandy Bridge",
        "i5-2450M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2450M @ 2.5 GHz",
    ],
    [
        3200000000,
        3500000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Sandy Bridge",
        "i5-2450P",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2450P @ 3.2 GHz",
    ],
    [
        1600000000,
        2300000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2467M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2467M @ 1.6 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2500",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2500 @ 3.3 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2500K",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2500K @ 3.3 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2500S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2500S @ 2.7 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2500T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2500T @ 2.3 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2510E",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2510E @ 2.5 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2515E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2515E @ 2.5 GHz",
    ],
    [
        2500000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2520M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2520M @ 2.5 GHz",
    ],
    [
        1400000000,
        2300000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2537M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2537M @ 1.4 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2540M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2540M @ 2.6 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Sandy Bridge",
        "i5-2550K",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2550K @ 3.4 GHz",
    ],
    [
        1700000000,
        2700000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i5-2557M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-2557M @ 1.7 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3210M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3210M @ 2.5 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i5-3230M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3230M @ 2.6 GHz",
    ],
    [
        1700000000,
        2600000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3317U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3317U @ 1.7 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3320M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3320M @ 2.6 GHz",
    ],
    [
        3000000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3330",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3330 @ 3.0 GHz",
    ],
    [
        2700000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3330S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3330S @ 2.7 GHz",
    ],
    [
        1800000000,
        2700000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i5-3337U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3337U @ 1.8 GHz",
    ],
    [
        1500000000,
        2000000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i5-3339Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3339Y @ 1.5 GHz",
    ],
    [
        3100000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i5-3340",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3340 @ 3.1 GHz",
    ],
    [
        2700000000,
        3400000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i5-3340M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3340M @ 2.7 GHz",
    ],
    [
        2800000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i5-3340S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3340S @ 2.8 GHz",
    ],
    [
        3100000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3350P",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3350P @ 3.1 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3360M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3360M @ 2.8 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i5-3380M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3380M @ 2.9 GHz",
    ],
    [
        1800000000,
        2800000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3427U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3427U @ 1.8 GHz",
    ],
    [
        1900000000,
        2900000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i5-3437U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3437U @ 1.9 GHz",
    ],
    [
        1500000000,
        2300000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i5-3439Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3439Y @ 1.5 GHz",
    ],
    [
        3100000000,
        3500000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3450",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3450 @ 3.1 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3450S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3450S @ 2.8 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3470",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3470 @ 3.2 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3470S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3470S @ 2.9 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3470T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3470T @ 2.9 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3475S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3475S @ 2.9 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3550",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3550 @ 3.3 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3550S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3550S @ 3.0 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3570",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3570 @ 3.4 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3570K",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3570K @ 3.4 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3570S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3570S @ 3.1 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3570T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3570T @ 2.3 GHz",
    ],
    [
        2700000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i5-3610ME",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-3610ME @ 2.7 GHz",
    ],
    [
        2800000000,
        3400000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4200H",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4200H @ 2.8 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4200M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4200M @ 2.5 GHz",
    ],
    [
        1600000000,
        2600000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4200U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4200U @ 1.6 GHz",
    ],
    [
        1400000000,
        1900000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4200Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4200Y @ 1.4 GHz",
    ],
    [
        1600000000,
        2000000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4202Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4202Y @ 1.6 GHz",
    ],
    [
        2900000000,
        3500000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4210H",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4210H @ 2.9 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4210M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4210M @ 2.6 GHz",
    ],
    [
        1700000000,
        2700000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4210U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4210U @ 1.7 GHz",
    ],
    [
        1500000000,
        1900000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4210Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4210Y @ 1.5 GHz",
    ],
    [
        1600000000,
        2000000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4220Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4220Y @ 1.6 GHz",
    ],
    [
        1300000000,
        2600000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4250U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4250U @ 1.3 GHz",
    ],
    [
        2400000000,
        2900000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4258U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4258U @ 2.4 GHz",
    ],
    [
        1400000000,
        2700000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4260U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4260U @ 1.4 GHz",
    ],
    [
        2600000000,
        3100000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4278U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4278U @ 2.6 GHz",
    ],
    [
        2600000000,
        3100000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4288U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4288U @ 2.6 GHz",
    ],
    [
        2600000000,
        3300000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4300M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4300M @ 2.6 GHz",
    ],
    [
        1900000000,
        2900000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4300U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4300U @ 1.9 GHz",
    ],
    [
        1600000000,
        2300000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4300Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4300Y @ 1.6 GHz",
    ],
    [
        1600000000,
        2300000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4302Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4302Y @ 1.6 GHz",
    ],
    [
        2800000000,
        3300000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4308U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4308U @ 2.8 GHz",
    ],
    [
        2260000000,
        2530000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-430M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-430M @ 2.3 GHz",
    ],
    [
        1200000000,
        1730000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-430UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-430UM @ 1.2 GHz",
    ],
    [
        2700000000,
        3400000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4310M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4310M @ 2.7 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4310U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4310U @ 2.0 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4330M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4330M @ 2.8 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4340M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4340M @ 2.9 GHz",
    ],
    [
        1400000000,
        2900000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4350U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4350U @ 1.4 GHz",
    ],
    [
        1500000000,
        3000000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4360U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4360U @ 1.5 GHz",
    ],
    [
        2700000000,
        3300000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4400E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4400E @ 2.7 GHz",
    ],
    [
        1600000000,
        2700000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4402E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4402E @ 1.6 GHz",
    ],
    [
        2500000000,
        None,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4402EC",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4402EC @ 2.5 GHz",
    ],
    [
        2900000000,
        None,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4410E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4410E @ 2.9 GHz",
    ],
    [
        1800000000,
        2900000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4422E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4422E @ 1.8 GHz",
    ],
    [
        3000000000,
        3200000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4430",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4430 @ 3.0 GHz",
    ],
    [
        2700000000,
        3200000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4430S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4430S @ 2.7 GHz",
    ],
    [
        3100000000,
        3300000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4440",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4440 @ 3.1 GHz",
    ],
    [
        2800000000,
        3300000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4440S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4440S @ 2.8 GHz",
    ],
    [
        3200000000,
        3400000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4460",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4460 @ 3.2 GHz",
    ],
    [
        2900000000,
        3400000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4460S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4460S @ 2.9 GHz",
    ],
    [
        1900000000,
        2700000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4460T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4460T @ 1.9 GHz",
    ],
    [
        2400000000,
        2660000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-450M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-450M @ 2.4 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4570",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4570 @ 3.2 GHz",
    ],
    [
        2700000000,
        3200000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Crystal Well",
        "i5-4570R",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4570R @ 2.7 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4570S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4570S @ 2.9 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4570T",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4570T @ 2.9 GHz",
    ],
    [
        2700000000,
        3300000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4570TE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4570TE @ 2.7 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4590",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4590 @ 3.3 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4590S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4590S @ 3.0 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4590T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4590T @ 2.0 GHz",
    ],
    [
        2530000000,
        2800000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-460M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-460M @ 2.5 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4670",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4670 @ 3.4 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4670K",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4670K @ 3.4 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Crystal Well",
        "i5-4670R",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4670R @ 3.0 GHz",
    ],
    [
        3100000000,
        3800000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4670S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4670S @ 3.1 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2013.0,
        "Haswell",
        "i5-4670T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4670T @ 2.3 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4690",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4690 @ 3.5 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Devil's Canyon",
        "i5-4690K",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4690K @ 3.5 GHz",
    ],
    [
        3200000000,
        3900000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4690S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4690S @ 3.2 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "4th Generation Intel Core i5 Processors",
        3,
        2014.0,
        "Haswell",
        "i5-4690T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i5 Processors i5-4690T @ 2.5 GHz",
    ],
    [
        1330000000,
        1860000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-470UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-470UM @ 1.3 GHz",
    ],
    [
        2660000000,
        2930000000.0,
        "Legacy Intel Core Processors",
        1,
        2011.0,
        "Arrandale",
        "i5-480M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-480M @ 2.7 GHz",
    ],
    [
        2200000000,
        2700000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5200U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5200U @ 2.2 GHz",
    ],
    [
        2400000000,
        2930000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-520E",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-520E @ 2.4 GHz",
    ],
    [
        2400000000,
        2930000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-520M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-520M @ 2.4 GHz",
    ],
    [
        1070000000,
        1870000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-520UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-520UM @ 1.1 GHz",
    ],
    [
        1600000000,
        2700000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5250U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5250U @ 1.6 GHz",
    ],
    [
        2700000000,
        3100000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5257U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5257U @ 2.7 GHz",
    ],
    [
        2900000000,
        3300000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5287U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5287U @ 2.9 GHz",
    ],
    [
        2300000000,
        2900000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5300U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5300U @ 2.3 GHz",
    ],
    [
        3100000000,
        3500000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5350H",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5350H @ 3.1 GHz",
    ],
    [
        1800000000,
        2900000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5350U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5350U @ 1.8 GHz",
    ],
    [
        2530000000,
        3070000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-540M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-540M @ 2.5 GHz",
    ],
    [
        1200000000,
        2000000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-540UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-540UM @ 1.2 GHz",
    ],
    [
        2800000000,
        3300000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5575R",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5575R @ 2.8 GHz",
    ],
    [
        2660000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-560M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-560M @ 2.7 GHz",
    ],
    [
        1330000000,
        2130000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-560UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-560UM @ 1.3 GHz",
    ],
    [
        3100000000,
        3600000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5675C",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5675C @ 3.1 GHz",
    ],
    [
        3100000000,
        3600000000.0,
        "5th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Broadwell",
        "i5-5675R",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 5th Generation Intel Core i5 Processors i5-5675R @ 3.1 GHz",
    ],
    [
        2660000000,
        3330000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i5-580M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-580M @ 2.7 GHz",
    ],
    [
        2300000000,
        2800000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6198DU",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6198DU @ 2.3 GHz",
    ],
    [
        2300000000,
        2800000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6200U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6200U @ 2.3 GHz",
    ],
    [
        1800000000,
        2900000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6260U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6260U @ 1.8 GHz",
    ],
    [
        2900000000,
        3300000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6267U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6267U @ 2.9 GHz",
    ],
    [
        3100000000,
        3500000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6287U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6287U @ 3.1 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6300HQ",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6300HQ @ 2.3 GHz",
    ],
    [
        2400000000,
        3000000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6300U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6300U @ 2.4 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2016.0,
        "Skylake",
        "i5-6350HQ",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6350HQ @ 2.3 GHz",
    ],
    [
        2000000000,
        3100000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6360U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6360U @ 2.0 GHz",
    ],
    [
        2700000000,
        3300000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6400",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6400 @ 2.7 GHz",
    ],
    [
        2200000000,
        2800000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6400T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6400T @ 2.2 GHz",
    ],
    [
        2800000000,
        3400000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6402P",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6402P @ 2.8 GHz",
    ],
    [
        2700000000,
        3400000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6440EQ",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6440EQ @ 2.7 GHz",
    ],
    [
        2600000000,
        3500000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6440HQ",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6440HQ @ 2.6 GHz",
    ],
    [
        1900000000,
        2700000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6442EQ",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6442EQ @ 1.9 GHz",
    ],
    [
        3200000000,
        3460000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i5-650",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-650 @ 3.2 GHz",
    ],
    [
        3200000000,
        3600000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6500",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6500 @ 3.2 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6500T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6500T @ 2.5 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6500TE",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6500TE @ 2.3 GHz",
    ],
    [
        3200000000,
        3460000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i5-655K",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-655K @ 3.2 GHz",
    ],
    [
        2800000000,
        3600000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2016.0,
        "Skylake",
        "i5-6585R",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6585R @ 2.8 GHz",
    ],
    [
        3330000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i5-660",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-660 @ 3.3 GHz",
    ],
    [
        3300000000,
        3900000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6600",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6600 @ 3.3 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6600K",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6600K @ 3.5 GHz",
    ],
    [
        2700000000,
        3500000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2015.0,
        "Skylake",
        "i5-6600T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6600T @ 2.7 GHz",
    ],
    [
        3330000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i5-661",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-661 @ 3.3 GHz",
    ],
    [
        3200000000,
        3800000000.0,
        "6th Generation Intel Core i5 Processors",
        3,
        2016.0,
        "Skylake",
        "i5-6685R",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i5 Processors i5-6685R @ 3.2 GHz",
    ],
    [
        3460000000,
        3730000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i5-670",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-670 @ 3.5 GHz",
    ],
    [
        3600000000,
        3860000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarkdale",
        "i5-680",
        2,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-680 @ 3.6 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2016.0,
        "Kaby Lake",
        "i5-7200U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7200U @ 2.5 GHz",
    ],
    [
        2200000000,
        3400000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7260U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7260U @ 2.2 GHz",
    ],
    [
        3100000000,
        3500000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7267U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7267U @ 3.1 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7287U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7287U @ 3.3 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7300HQ",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7300HQ @ 2.5 GHz",
    ],
    [
        2600000000,
        3500000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7300U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7300U @ 2.6 GHz",
    ],
    [
        2300000000,
        3600000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7360U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7360U @ 2.3 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7400",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7400 @ 3.0 GHz",
    ],
    [
        2400000000,
        3000000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7400T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7400T @ 2.4 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7440EQ",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7440EQ @ 2.9 GHz",
    ],
    [
        2800000000,
        3800000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7440HQ",
        4,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7440HQ @ 2.8 GHz",
    ],
    [
        2100000000,
        2900000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7442EQ",
        4,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7442EQ @ 2.1 GHz",
    ],
    [
        2660000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        1,
        2009.0,
        "Lynnfield",
        "i5-750",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-750 @ 2.7 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7500",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7500 @ 3.4 GHz",
    ],
    [
        2700000000,
        3300000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7500T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7500T @ 2.7 GHz",
    ],
    [
        2400000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Lynnfield",
        "i5-750S",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-750S @ 2.4 GHz",
    ],
    [
        2800000000,
        3330000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Lynnfield",
        "i5-760",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i5-760 @ 2.8 GHz",
    ],
    [
        3500000000,
        4099999999.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7600",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7600 @ 3.5 GHz",
    ],
    [
        3800000000,
        4200000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7600K",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7600K @ 3.8 GHz",
    ],
    [
        2800000000,
        3700000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7600T",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7600T @ 2.8 GHz",
    ],
    [
        4000000000,
        4200000000.0,
        "Intel Core X-series Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7640X",
        4,
        "4",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i5-7640X @ 4.0 GHz",
    ],
    [
        1200000000,
        3200000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2016.0,
        "Kaby Lake",
        "i5-7Y54",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7Y54 @ 1.2 GHz",
    ],
    [
        1200000000,
        3300000000.0,
        "7th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i5-7Y57",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i5 Processors i5-7Y57 @ 1.2 GHz",
    ],
    [
        1300000000,
        3900000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Amber Lake Y",
        "i5-8200Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8200Y @ 1.3 GHz",
    ],
    [
        1600000000,
        3900000000.0,
        "8th Generation Intel Core i5 Processors",
        1,
        2019.0,
        "Amber Lake Y",
        "i5-8210Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8210Y @ 1.6 GHz",
    ],
    [
        1600000000,
        3400000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake R",
        "i5-8250U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8250U @ 1.6 GHz",
    ],
    [
        1400000000,
        3900000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-8257U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8257U @ 1.4 GHz",
    ],
    [
        2300000000,
        3800000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8259U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8259U @ 2.3 GHz",
    ],
    [
        1600000000,
        3900000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Whiskey Lake",
        "i5-8265U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8265U @ 1.6 GHz",
    ],
    [
        2600000000,
        4200000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8269U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8269U @ 2.6 GHz",
    ],
    [
        2400000000,
        4099999999.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-8279U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8279U @ 2.4 GHz",
    ],
    [
        2300000000,
        4000000000.0,
        "8th Generation Intel Core i5 Processors",
        0,
        2018.0,
        "Coffee Lake",
        "i5-8300H",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8300H @ 2.3 GHz",
    ],
    [
        2800000000,
        3800000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Kaby Lake G",
        "i5-8305G",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8305G @ 2.8 GHz",
    ],
    [
        1600000000,
        3900000000.0,
        "8th Generation Intel Core i5 Processors",
        1,
        2019.0,
        "Amber Lake Y",
        "i5-8310Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8310Y @ 1.6 GHz",
    ],
    [
        1700000000,
        3600000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Kaby Lake R",
        "i5-8350U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8350U @ 1.7 GHz",
    ],
    [
        1600000000,
        4099999999.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Whiskey Lake",
        "i5-8365U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8365U @ 1.6 GHz",
    ],
    [
        1600000000,
        4099999999.0,
        "8th Generation Intel Core i5 Processors",
        0,
        2019.0,
        "Whiskey Lake",
        "i5-8365UE",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8365UE @ 1.6 GHz",
    ],
    [
        2800000000,
        4000000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8400",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8400 @ 2.8 GHz",
    ],
    [
        2800000000,
        4000000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8400B",
        6,
        "6",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8400B @ 2.8 GHz",
    ],
    [
        2500000000,
        4200000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8400H",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8400H @ 2.5 GHz",
    ],
    [
        1700000000,
        3300000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8400T",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8400T @ 1.7 GHz",
    ],
    [
        3000000000,
        4099999999.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8500",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8500 @ 3.0 GHz",
    ],
    [
        3000000000,
        4099999999.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8500B",
        6,
        "6",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8500B @ 3.0 GHz",
    ],
    [
        2100000000,
        3500000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8500T",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8500T @ 2.1 GHz",
    ],
    [
        3100000000,
        4300000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8600",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8600 @ 3.1 GHz",
    ],
    [
        3600000000,
        4300000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2017.0,
        "Coffee Lake",
        "i5-8600K",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8600K @ 3.6 GHz",
    ],
    [
        2300000000,
        3700000000.0,
        "8th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-8600T",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i5 Processors i5-8600T @ 2.3 GHz",
    ],
    [
        2400000000,
        4099999999.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9300H",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9300H @ 2.4 GHz",
    ],
    [
        2900000000,
        4099999999.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9400",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9400 @ 2.9 GHz",
    ],
    [
        2900000000,
        4099999999.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9400F",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9400F @ 2.9 GHz",
    ],
    [
        2500000000,
        4300000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9400H",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9400H @ 2.5 GHz",
    ],
    [
        1800000000,
        3400000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9400T",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9400T @ 1.8 GHz",
    ],
    [
        3000000000,
        4400000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9500",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9500 @ 3.0 GHz",
    ],
    [
        3000000000,
        4200000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9500E",
        6,
        "6",
        "Embedded",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9500E @ 3.0 GHz",
    ],
    [
        3000000000,
        4400000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9500F",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9500F @ 3.0 GHz",
    ],
    [
        2200000000,
        3700000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9500T",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9500T @ 2.2 GHz",
    ],
    [
        2200000000,
        3600000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9500TE",
        6,
        "6",
        "Embedded",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9500TE @ 2.2 GHz",
    ],
    [
        3100000000,
        4600000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9600",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9600 @ 3.1 GHz",
    ],
    [
        3700000000,
        4600000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i5-9600K",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9600K @ 3.7 GHz",
    ],
    [
        3700000000,
        4600000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9600KF",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9600KF @ 3.7 GHz",
    ],
    [
        2300000000,
        3900000000.0,
        "9th Generation Intel Core i5 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i5-9600T",
        6,
        "6",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i5 Processors i5-9600T @ 2.3 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2600",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2600 @ 3.4 GHz",
    ],
    [
        3400000000,
        3800000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2600K",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2600K @ 3.4 GHz",
    ],
    [
        2800000000,
        3800000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2600S",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2600S @ 2.8 GHz",
    ],
    [
        1500000000,
        2400000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2610UE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2610UE @ 1.5 GHz",
    ],
    [
        1500000000,
        2600000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2617M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2617M @ 1.5 GHz",
    ],
    [
        2700000000,
        3400000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2620M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2620M @ 2.7 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2629M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2629M @ 2.1 GHz",
    ],
    [
        2000000000,
        2900000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2630QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2630QM @ 2.0 GHz",
    ],
    [
        2000000000,
        2900000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2635QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2635QM @ 2.0 GHz",
    ],
    [
        1700000000,
        2800000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2637M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2637M @ 1.7 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2640M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2640M @ 2.8 GHz",
    ],
    [
        2300000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2649M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2649M @ 2.3 GHz",
    ],
    [
        2200000000,
        2900000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2655LE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2655LE @ 2.2 GHz",
    ],
    [
        1600000000,
        2700000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2657M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2657M @ 1.6 GHz",
    ],
    [
        2200000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2670QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2670QM @ 2.2 GHz",
    ],
    [
        2200000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2675QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2675QM @ 2.2 GHz",
    ],
    [
        1800000000,
        2900000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2677M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2677M @ 1.8 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2700K",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2700K @ 3.5 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2710QE",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2710QE @ 2.1 GHz",
    ],
    [
        2100000000,
        3000000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2715QE",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2715QE @ 2.1 GHz",
    ],
    [
        2200000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2720QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2720QM @ 2.2 GHz",
    ],
    [
        2400000000,
        3500000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2760QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2760QM @ 2.4 GHz",
    ],
    [
        2300000000,
        3400000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2820QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2820QM @ 2.3 GHz",
    ],
    [
        2500000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2860QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2860QM @ 2.5 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2920XM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2920XM @ 2.5 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2011.0,
        "Sandy Bridge",
        "i7-2960XM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-2960XM @ 2.7 GHz",
    ],
    [
        1900000000,
        3000000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3517U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3517U @ 1.9 GHz",
    ],
    [
        1700000000,
        2800000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3517UE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3517UE @ 1.7 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3520M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3520M @ 2.9 GHz",
    ],
    [
        2000000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i7-3537U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3537U @ 2.0 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i7-3540M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3540M @ 3.0 GHz",
    ],
    [
        2500000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3555LE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3555LE @ 2.5 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3610QE",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3610QE @ 2.3 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3610QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3610QM @ 2.3 GHz",
    ],
    [
        2100000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3612QE",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3612QE @ 2.1 GHz",
    ],
    [
        2100000000,
        3100000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3612QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3612QM @ 2.1 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3615QE",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3615QE @ 2.3 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3615QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3615QM @ 2.3 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3630QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3630QM @ 2.4 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3632QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3632QM @ 2.2 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3635QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3635QM @ 2.4 GHz",
    ],
    [
        2000000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3667U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3667U @ 2.0 GHz",
    ],
    [
        2100000000,
        3300000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i7-3687U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3687U @ 2.1 GHz",
    ],
    [
        1500000000,
        2600000000.0,
        "Legacy Intel Core Processors",
        2,
        2013.0,
        "Ivy Bridge",
        "i7-3689Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3689Y @ 1.5 GHz",
    ],
    [
        2600000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3720QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3720QM @ 2.6 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3740QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3740QM @ 2.7 GHz",
    ],
    [
        3400000000,
        3900000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3770",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3770 @ 3.4 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3770K",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3770K @ 3.5 GHz",
    ],
    [
        3100000000,
        3900000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3770S",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3770S @ 3.1 GHz",
    ],
    [
        2500000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3770T",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3770T @ 2.5 GHz",
    ],
    [
        3600000000,
        3800000000.0,
        "Intel Core X-series Processors",
        2,
        2012.0,
        "Sandy Bridge E",
        "i7-3820",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-3820 @ 3.6 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3820QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3820QM @ 2.7 GHz",
    ],
    [
        2800000000,
        3800000000.0,
        "Legacy Intel Core Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3840QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-3840QM @ 2.8 GHz",
    ],
    [
        2900000000,
        3800000000.0,
        "Intel Core X-series Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3920XM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-3920XM @ 2.9 GHz",
    ],
    [
        3200000000,
        3800000000.0,
        "Intel Core X-series Processors",
        2,
        2011.0,
        "Sandy Bridge E",
        "i7-3930K",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-3930K @ 3.2 GHz",
    ],
    [
        3000000000,
        3900000000.0,
        "Intel Core X-series Processors",
        2,
        2012.0,
        "Ivy Bridge",
        "i7-3940XM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-3940XM @ 3.0 GHz",
    ],
    [
        3300000000,
        3900000000.0,
        "Intel Core X-series Processors",
        2,
        2011.0,
        "Sandy Bridge E",
        "i7-3960X",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-3960X @ 3.3 GHz",
    ],
    [
        3500000000,
        4000000000.0,
        "Intel Core X-series Processors",
        2,
        2012.0,
        "Sandy Bridge E",
        "i7-3970X",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-3970X @ 3.5 GHz",
    ],
    [
        1800000000,
        3000000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4500U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4500U @ 1.8 GHz",
    ],
    [
        2000000000,
        3100000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4510U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4510U @ 2.0 GHz",
    ],
    [
        1500000000,
        3000000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4550U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4550U @ 1.5 GHz",
    ],
    [
        2800000000,
        3300000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4558U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4558U @ 2.8 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4578U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4578U @ 3.0 GHz",
    ],
    [
        2900000000,
        3600000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4600M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4600M @ 2.9 GHz",
    ],
    [
        2100000000,
        3300000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4600U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4600U @ 2.1 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4610M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4610M @ 3.0 GHz",
    ],
    [
        1700000000,
        2900000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4610Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4610Y @ 1.7 GHz",
    ],
    [
        1700000000,
        3300000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4650U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4650U @ 1.7 GHz",
    ],
    [
        2700000000,
        None,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4700EC",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4700EC @ 2.7 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4700EQ",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4700EQ @ 2.4 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4700HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4700HQ @ 2.4 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4700MQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4700MQ @ 2.4 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4701EQ",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4701EQ @ 2.4 GHz",
    ],
    [
        2000000000,
        None,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4702EC",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4702EC @ 2.0 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4702HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4702HQ @ 2.2 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4702MQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4702MQ @ 2.2 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4710HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4710HQ @ 2.5 GHz",
    ],
    [
        2500000000,
        3500000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4710MQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4710MQ @ 2.5 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4712HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4712HQ @ 2.3 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4712MQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4712MQ @ 2.3 GHz",
    ],
    [
        2600000000,
        3600000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Haswell",
        "i7-4720HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4720HQ @ 2.6 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Haswell",
        "i7-4722HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4722HQ @ 2.4 GHz",
    ],
    [
        2000000000,
        3200000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Crystal Well",
        "i7-4750HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4750HQ @ 2.0 GHz",
    ],
    [
        2100000000,
        3300000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Crystal Well",
        "i7-4760HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4760HQ @ 2.1 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4765T",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4765T @ 2.0 GHz",
    ],
    [
        3400000000,
        3900000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4770",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4770 @ 3.4 GHz",
    ],
    [
        2200000000,
        3400000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Crystal Well",
        "i7-4770HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4770HQ @ 2.2 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4770K",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4770K @ 3.5 GHz",
    ],
    [
        3200000000,
        3900000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Crystal Well",
        "i7-4770R",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4770R @ 3.2 GHz",
    ],
    [
        3100000000,
        3900000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4770S",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4770S @ 3.1 GHz",
    ],
    [
        2500000000,
        3700000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4770T",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4770T @ 2.5 GHz",
    ],
    [
        2300000000,
        3300000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4770TE",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4770TE @ 2.3 GHz",
    ],
    [
        3500000000,
        3900000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4771",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4771 @ 3.5 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4785T",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4785T @ 2.2 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4790",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4790 @ 3.6 GHz",
    ],
    [
        4000000000,
        4400000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Devil's Canyon",
        "i7-4790K",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4790K @ 4.0 GHz",
    ],
    [
        3200000000,
        4000000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4790S",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4790S @ 3.2 GHz",
    ],
    [
        2700000000,
        3900000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4790T",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4790T @ 2.7 GHz",
    ],
    [
        2700000000,
        3700000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4800MQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4800MQ @ 2.7 GHz",
    ],
    [
        2800000000,
        3800000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4810MQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4810MQ @ 2.8 GHz",
    ],
    [
        3700000000,
        3900000000.0,
        "Intel Core X-series Processors",
        2,
        2013.0,
        "Ivy Bridge E",
        "i7-4820K",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-4820K @ 3.7 GHz",
    ],
    [
        1600000000,
        3200000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Crystal Well",
        "i7-4850EQ",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4850EQ @ 1.6 GHz",
    ],
    [
        2300000000,
        3500000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Crystal Well",
        "i7-4850HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4850HQ @ 2.3 GHz",
    ],
    [
        1800000000,
        3200000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Crystal Well",
        "i7-4860EQ",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4860EQ @ 1.8 GHz",
    ],
    [
        2400000000,
        3600000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Crystal Well",
        "i7-4860HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4860HQ @ 2.4 GHz",
    ],
    [
        2500000000,
        3700000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Crystal Well",
        "i7-4870HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4870HQ @ 2.5 GHz",
    ],
    [
        2800000000,
        3800000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4900MQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4900MQ @ 2.8 GHz",
    ],
    [
        2900000000,
        3900000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4910MQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4910MQ @ 2.9 GHz",
    ],
    [
        3400000000,
        3900000000.0,
        "Intel Core X-series Processors",
        2,
        2013.0,
        "Ivy Bridge E",
        "i7-4930K",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-4930K @ 3.4 GHz",
    ],
    [
        3000000000,
        3900000000.0,
        "Intel Core X-series Processors",
        3,
        2013.0,
        "Haswell",
        "i7-4930MX",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-4930MX @ 3.0 GHz",
    ],
    [
        3100000000,
        4000000000.0,
        "Intel Core X-series Processors",
        3,
        2014.0,
        "Haswell",
        "i7-4940MX",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-4940MX @ 3.1 GHz",
    ],
    [
        2400000000,
        3600000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Crystal Well",
        "i7-4950HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4950HQ @ 2.4 GHz",
    ],
    [
        2600000000,
        3800000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2013.0,
        "Crystal Well",
        "i7-4960HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4960HQ @ 2.6 GHz",
    ],
    [
        3600000000,
        4000000000.0,
        "Intel Core X-series Processors",
        2,
        2013.0,
        "Ivy Bridge E",
        "i7-4960X",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-4960X @ 3.6 GHz",
    ],
    [
        2800000000,
        4000000000.0,
        "4th Generation Intel Core i7 Processors",
        3,
        2014.0,
        "Crystal Well",
        "i7-4980HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 4th Generation Intel Core i7 Processors i7-4980HQ @ 2.8 GHz",
    ],
    [
        2400000000,
        3000000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5500U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5500U @ 2.4 GHz",
    ],
    [
        2000000000,
        3000000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5550U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5550U @ 2.0 GHz",
    ],
    [
        3100000000,
        3400000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5557U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5557U @ 3.1 GHz",
    ],
    [
        2600000000,
        3200000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5600U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5600U @ 2.6 GHz",
    ],
    [
        2200000000,
        3100000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5650U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5650U @ 2.2 GHz",
    ],
    [
        2600000000,
        3400000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5700EQ",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5700EQ @ 2.6 GHz",
    ],
    [
        2700000000,
        3500000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5700HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5700HQ @ 2.7 GHz",
    ],
    [
        2500000000,
        3400000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5750HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5750HQ @ 2.5 GHz",
    ],
    [
        3300000000,
        3700000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5775C",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5775C @ 3.3 GHz",
    ],
    [
        3300000000,
        3800000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5775R",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5775R @ 3.3 GHz",
    ],
    [
        3300000000,
        3600000000.0,
        "Intel Core X-series Processors",
        3,
        2014.0,
        "Haswell E",
        "i7-5820K",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-5820K @ 3.3 GHz",
    ],
    [
        2700000000,
        3400000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5850EQ",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5850EQ @ 2.7 GHz",
    ],
    [
        2700000000,
        3600000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5850HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5850HQ @ 2.7 GHz",
    ],
    [
        3500000000,
        3700000000.0,
        "Intel Core X-series Processors",
        3,
        2014.0,
        "Haswell E",
        "i7-5930K",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-5930K @ 3.5 GHz",
    ],
    [
        2900000000,
        3800000000.0,
        "5th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Broadwell",
        "i7-5950HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 5th Generation Intel Core i7 Processors i7-5950HQ @ 2.9 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "Intel Core X-series Processors",
        3,
        2014.0,
        "Haswell E",
        "i7-5960X",
        8,
        "16",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-5960X @ 3.0 GHz",
    ],
    [
        2530000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-610E",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-610E @ 2.5 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-620LE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-620LE @ 2.0 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-620LM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-620LM @ 2.0 GHz",
    ],
    [
        2660000000,
        3330000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-620M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-620M @ 2.7 GHz",
    ],
    [
        1060000000,
        2130000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-620UE",
        2,
        "4",
        "Embedded",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-620UE @ 1.1 GHz",
    ],
    [
        1060000000,
        2130000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-620UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-620UM @ 1.1 GHz",
    ],
    [
        2130000000,
        2930000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-640LM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-640LM @ 2.1 GHz",
    ],
    [
        2800000000,
        3460000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-640M",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-640M @ 2.8 GHz",
    ],
    [
        1200000000,
        2270000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-640UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-640UM @ 1.2 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6498DU",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6498DU @ 2.5 GHz",
    ],
    [
        2500000000,
        3100000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6500U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6500U @ 2.5 GHz",
    ],
    [
        2200000000,
        3200000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6560U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6560U @ 2.2 GHz",
    ],
    [
        3300000000,
        3600000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6567U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6567U @ 3.3 GHz",
    ],
    [
        2600000000,
        3400000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6600U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6600U @ 2.6 GHz",
    ],
    [
        2260000000,
        3060000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-660LM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-660LM @ 2.3 GHz",
    ],
    [
        1330000000,
        2400000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-660UE",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-660UE @ 1.3 GHz",
    ],
    [
        1330000000,
        2400000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-660UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-660UM @ 1.3 GHz",
    ],
    [
        2200000000,
        3400000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6650U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6650U @ 2.2 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2016.0,
        "Skylake",
        "i7-6660U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6660U @ 2.4 GHz",
    ],
    [
        3400000000,
        4000000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6700",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6700 @ 3.4 GHz",
    ],
    [
        2600000000,
        3500000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6700HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6700HQ @ 2.6 GHz",
    ],
    [
        4000000000,
        4200000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6700K",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6700K @ 4.0 GHz",
    ],
    [
        2800000000,
        3600000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6700T",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6700T @ 2.8 GHz",
    ],
    [
        2400000000,
        3400000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6700TE",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6700TE @ 2.4 GHz",
    ],
    [
        2600000000,
        3500000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2016.0,
        "Skylake",
        "i7-6770HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6770HQ @ 2.6 GHz",
    ],
    [
        3300000000,
        3900000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2016.0,
        "Skylake",
        "i7-6785R",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6785R @ 3.3 GHz",
    ],
    [
        3400000000,
        3600000000.0,
        "Intel Core X-series Processors",
        0,
        2016.0,
        "Broadwell E",
        "i7-6800K",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-6800K @ 3.4 GHz",
    ],
    [
        1460000000,
        2530000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Arrandale",
        "i7-680UM",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-680UM @ 1.5 GHz",
    ],
    [
        2800000000,
        3500000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6820EQ",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6820EQ @ 2.8 GHz",
    ],
    [
        2700000000,
        3600000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6820HK",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6820HK @ 2.7 GHz",
    ],
    [
        2700000000,
        3600000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6820HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6820HQ @ 2.7 GHz",
    ],
    [
        2000000000,
        2800000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6822EQ",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6822EQ @ 2.0 GHz",
    ],
    [
        3600000000,
        3800000000.0,
        "Intel Core X-series Processors",
        0,
        2016.0,
        "Broadwell E",
        "i7-6850K",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-6850K @ 3.6 GHz",
    ],
    [
        2700000000,
        3600000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2016.0,
        "Skylake",
        "i7-6870HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6870HQ @ 2.7 GHz",
    ],
    [
        3200000000,
        3700000000.0,
        "Intel Core X-series Processors",
        0,
        2016.0,
        "Broadwell E",
        "i7-6900K",
        8,
        "16",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-6900K @ 3.2 GHz",
    ],
    [
        2900000000,
        3800000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2015.0,
        "Skylake",
        "i7-6920HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6920HQ @ 2.9 GHz",
    ],
    [
        3000000000,
        3500000000.0,
        "Intel Core X-series Processors",
        0,
        2016.0,
        "Broadwell E",
        "i7-6950X",
        10,
        "20",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-6950X @ 3.0 GHz",
    ],
    [
        2800000000,
        3700000000.0,
        "6th Generation Intel Core i7 Processors",
        3,
        2016.0,
        "Skylake",
        "i7-6970HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 6th Generation Intel Core i7 Processors i7-6970HQ @ 2.8 GHz",
    ],
    [
        1600000000,
        2800000000.0,
        "Legacy Intel Core Processors",
        1,
        2009.0,
        "Clarksfield",
        "i7-720QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-720QM @ 1.6 GHz",
    ],
    [
        1730000000,
        2930000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarksfield",
        "i7-740QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-740QM @ 1.7 GHz",
    ],
    [
        2700000000,
        3500000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2016.0,
        "Kaby Lake",
        "i7-7500U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7500U @ 2.7 GHz",
    ],
    [
        2400000000,
        3800000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7560U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7560U @ 2.4 GHz",
    ],
    [
        3500000000,
        4000000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7567U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7567U @ 3.5 GHz",
    ],
    [
        2800000000,
        3900000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7600U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7600U @ 2.8 GHz",
    ],
    [
        2500000000,
        4000000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7660U",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7660U @ 2.5 GHz",
    ],
    [
        3600000000,
        4200000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7700",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7700 @ 3.6 GHz",
    ],
    [
        2800000000,
        3800000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7700HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7700HQ @ 2.8 GHz",
    ],
    [
        4200000000,
        4500000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7700K",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7700K @ 4.2 GHz",
    ],
    [
        2900000000,
        3800000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7700T",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7700T @ 2.9 GHz",
    ],
    [
        4300000000,
        4500000000.0,
        "Intel Core X-series Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7740X",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-7740X @ 4.3 GHz",
    ],
    [
        3500000000,
        4000000000.0,
        "Intel Core X-series Processors",
        4,
        2017.0,
        "Skylake",
        "i7-7800X",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-7800X @ 3.5 GHz",
    ],
    [
        3000000000,
        3700000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7820EQ",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7820EQ @ 3.0 GHz",
    ],
    [
        2900000000,
        3900000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7820HK",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7820HK @ 2.9 GHz",
    ],
    [
        2900000000,
        3900000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7820HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7820HQ @ 2.9 GHz",
    ],
    [
        3600000000,
        4300000000.0,
        "Intel Core X-series Processors",
        4,
        2017.0,
        "Skylake",
        "i7-7820X",
        8,
        "16",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-7820X @ 3.6 GHz",
    ],
    [
        3100000000,
        4099999999.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake",
        "i7-7920HQ",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7920HQ @ 3.1 GHz",
    ],
    [
        1300000000,
        3600000000.0,
        "7th Generation Intel Core i7 Processors",
        3,
        2016.0,
        "Kaby Lake",
        "i7-7Y75",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 7th Generation Intel Core i7 Processors i7-7Y75 @ 1.3 GHz",
    ],
    [
        4000000000,
        5000000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i7-8086K",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8086K @ 4.0 GHz",
    ],
    [
        1730000000,
        3060000000.0,
        "Legacy Intel Core Processors",
        1,
        2009.0,
        "Clarksfield",
        "i7-820QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-820QM @ 1.7 GHz",
    ],
    [
        1860000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarksfield",
        "i7-840QM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-840QM @ 1.9 GHz",
    ],
    [
        1500000000,
        3900000000.0,
        "8th Generation Intel Core i7 Processors",
        1,
        2019.0,
        "Amber Lake Y",
        "i7-8500Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8500Y @ 1.5 GHz",
    ],
    [
        1800000000,
        3900000000.0,
        "8th Generation Intel Core i7 Processors",
        1,
        2019.0,
        "Amber Lake Y",
        "i7-8510Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8510Y @ 1.8 GHz",
    ],
    [
        1800000000,
        4000000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake R",
        "i7-8550U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8550U @ 1.8 GHz",
    ],
    [
        1700000000,
        4500000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-8557U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8557U @ 1.7 GHz",
    ],
    [
        2700000000,
        4500000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i7-8559U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8559U @ 2.7 GHz",
    ],
    [
        1800000000,
        4600000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Whiskey Lake",
        "i7-8565U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8565U @ 1.8 GHz",
    ],
    [
        2800000000,
        4700000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-8569U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8569U @ 2.8 GHz",
    ],
    [
        2800000000,
        3460000000.0,
        "Legacy Intel Core Processors",
        1,
        2009.0,
        "Lynnfield",
        "i7-860",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-860 @ 2.8 GHz",
    ],
    [
        2530000000,
        3460000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Lynnfield",
        "i7-860S",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-860S @ 2.5 GHz",
    ],
    [
        1900000000,
        4200000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Kaby Lake R",
        "i7-8650U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8650U @ 1.9 GHz",
    ],
    [
        1900000000,
        4800000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Whiskey Lake",
        "i7-8665U",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8665U @ 1.9 GHz",
    ],
    [
        1700000000,
        4400000000.0,
        "8th Generation Intel Core i7 Processors",
        1,
        2019.0,
        "Whiskey Lake",
        "i7-8665UE",
        4,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8665UE @ 1.7 GHz",
    ],
    [
        2930000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        1,
        2009.0,
        "Lynnfield",
        "i7-870",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-870 @ 2.9 GHz",
    ],
    [
        3200000000,
        4600000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i7-8700",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8700 @ 3.2 GHz",
    ],
    [
        3200000000,
        4600000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i7-8700B",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8700B @ 3.2 GHz",
    ],
    [
        3700000000,
        4700000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2017.0,
        "Coffee Lake",
        "i7-8700K",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8700K @ 3.7 GHz",
    ],
    [
        2400000000,
        4000000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i7-8700T",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8700T @ 2.4 GHz",
    ],
    [
        3100000000,
        4099999999.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Kaby Lake G",
        "i7-8705G",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8705G @ 3.1 GHz",
    ],
    [
        3100000000,
        4099999999.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Kaby Lake G",
        "i7-8706G",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8706G @ 3.1 GHz",
    ],
    [
        3100000000,
        4099999999.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Kaby Lake G",
        "i7-8709G",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8709G @ 3.1 GHz",
    ],
    [
        2660000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Lynnfield",
        "i7-870S",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-870S @ 2.7 GHz",
    ],
    [
        2200000000,
        4099999999.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i7-8750H",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8750H @ 2.2 GHz",
    ],
    [
        2930000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Lynnfield",
        "i7-875K",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-875K @ 2.9 GHz",
    ],
    [
        3060000000,
        3730000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Lynnfield",
        "i7-880",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-880 @ 3.1 GHz",
    ],
    [
        3100000000,
        4200000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Kaby Lake G",
        "i7-8809G",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8809G @ 3.1 GHz",
    ],
    [
        2600000000,
        4300000000.0,
        "8th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i7-8850H",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i7 Processors i7-8850H @ 2.6 GHz",
    ],
    [
        2660000000,
        2930000000.0,
        "Legacy Intel Core Processors",
        1,
        2008.0,
        "Bloomfield",
        "i7-920",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-920 @ 2.7 GHz",
    ],
    [
        2000000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        1,
        2009.0,
        "Clarksfield",
        "i7-920XM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-920XM @ 2.0 GHz",
    ],
    [
        2800000000,
        3060000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Bloomfield",
        "i7-930",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-930 @ 2.8 GHz",
    ],
    [
        2930000000,
        3200000000.0,
        "Legacy Intel Core Processors",
        1,
        2008.0,
        "Bloomfield",
        "i7-940",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-940 @ 2.9 GHz",
    ],
    [
        2130000000,
        3330000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Clarksfield",
        "i7-940XM",
        4,
        "8",
        "Mobile",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-940XM @ 2.1 GHz",
    ],
    [
        3060000000,
        3330000000.0,
        "Legacy Intel Core Processors",
        1,
        2009.0,
        "Bloomfield",
        "i7-950",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-950 @ 3.1 GHz",
    ],
    [
        3200000000,
        3460000000.0,
        "Legacy Intel Core Processors",
        1,
        2009.0,
        "Bloomfield",
        "i7-960",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-960 @ 3.2 GHz",
    ],
    [
        3200000000,
        3460000000.0,
        "Legacy Intel Core Processors",
        1,
        2008.0,
        "Bloomfield",
        "i7-965",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-965 @ 3.2 GHz",
    ],
    [
        3200000000,
        3460000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Gulftown",
        "i7-970",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-970 @ 3.2 GHz",
    ],
    [
        3000000000,
        4700000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9700",
        8,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9700 @ 3.0 GHz",
    ],
    [
        2600000000,
        4400000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9700E",
        8,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9700E @ 2.6 GHz",
    ],
    [
        3000000000,
        4700000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9700F",
        8,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9700F @ 3.0 GHz",
    ],
    [
        3600000000,
        4900000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i7-9700K",
        8,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9700K @ 3.6 GHz",
    ],
    [
        3600000000,
        4900000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9700KF",
        8,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9700KF @ 3.6 GHz",
    ],
    [
        2000000000,
        4300000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9700T",
        8,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9700T @ 2.0 GHz",
    ],
    [
        1800000000,
        3800000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9700TE",
        8,
        "8",
        "Embedded",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9700TE @ 1.8 GHz",
    ],
    [
        3330000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        1,
        2009.0,
        "Bloomfield",
        "i7-975",
        4,
        "8",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-975 @ 3.3 GHz",
    ],
    [
        2600000000,
        4500000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9750H",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9750H @ 2.6 GHz",
    ],
    [
        2600000000,
        4500000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9750HF",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9750HF @ 2.6 GHz",
    ],
    [
        3330000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        1,
        2011.0,
        "Gulftown",
        "i7-980",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-980 @ 3.3 GHz",
    ],
    [
        3800000000,
        4400000000.0,
        "Intel Core X-series Processors",
        4,
        2018.0,
        "Skylake",
        "i7-9800X",
        8,
        "16",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i7-9800X @ 3.8 GHz",
    ],
    [
        3330000000,
        3600000000.0,
        "Legacy Intel Core Processors",
        1,
        2010.0,
        "Gulftown",
        "i7-980X",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-980X @ 3.3 GHz",
    ],
    [
        2600000000,
        4600000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9850H",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9850H @ 2.6 GHz",
    ],
    [
        2700000000,
        4400000000.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9850HE",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9850HE @ 2.7 GHz",
    ],
    [
        1900000000,
        4099999999.0,
        "9th Generation Intel Core i7 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i7-9850HL",
        6,
        "12",
        "Embedded",
        "cpu",
        "intel",
        " 9th Generation Intel Core i7 Processors i7-9850HL @ 1.9 GHz",
    ],
    [
        3460000000,
        3730000000.0,
        "Legacy Intel Core Processors",
        1,
        2011.0,
        "Gulftown",
        "i7-990X",
        6,
        "12",
        "Desktop",
        "cpu",
        "intel",
        " Legacy Intel Core Processors i7-990X @ 3.5 GHz",
    ],
    [
        3300000000,
        4300000000.0,
        "Intel Core X-series Processors",
        4,
        2017.0,
        "Skylake",
        "i9-7900X",
        10,
        "20",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-7900X @ 3.3 GHz",
    ],
    [
        2900000000,
        4300000000.0,
        "Intel Core X-series Processors",
        4,
        2017.0,
        "Skylake",
        "i9-7920X",
        12,
        "24",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-7920X @ 2.9 GHz",
    ],
    [
        3100000000,
        4300000000.0,
        "Intel Core X-series Processors",
        4,
        2017.0,
        "Skylake",
        "i9-7940X",
        14,
        "28",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-7940X @ 3.1 GHz",
    ],
    [
        2800000000,
        4200000000.0,
        "Intel Core X-series Processors",
        4,
        2017.0,
        "Skylake",
        "i9-7960X",
        16,
        "32",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-7960X @ 2.8 GHz",
    ],
    [
        2600000000,
        4200000000.0,
        "Intel Core X-series Processors",
        4,
        2017.0,
        "Skylake",
        "i9-7980XE",
        18,
        "36",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-7980XE @ 2.6 GHz",
    ],
    [
        2900000000,
        4800000000.0,
        "8th Generation Intel Core i9 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i9-8950HK",
        6,
        "12",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core i9 Processors i9-8950HK @ 2.9 GHz",
    ],
    [
        3300000000,
        4099999999.0,
        "Intel Core X-series Processors",
        4,
        2018.0,
        "Skylake",
        "i9-9820X",
        10,
        "20",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-9820X @ 3.3 GHz",
    ],
    [
        2300000000,
        4800000000.0,
        "9th Generation Intel Core i9 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i9-9880H",
        8,
        "16",
        "Mobile",
        "cpu",
        "intel",
        " 9th Generation Intel Core i9 Processors i9-9880H @ 2.3 GHz",
    ],
    [
        3100000000,
        5000000000.0,
        "9th Generation Intel Core i9 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i9-9900",
        8,
        "16",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i9 Processors i9-9900 @ 3.1 GHz",
    ],
    [
        3600000000,
        5000000000.0,
        "9th Generation Intel Core i9 Processors",
        3,
        2018.0,
        "Coffee Lake",
        "i9-9900K",
        8,
        "16",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i9 Processors i9-9900K @ 3.6 GHz",
    ],
    [
        3600000000,
        5000000000.0,
        "9th Generation Intel Core i9 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i9-9900KF",
        8,
        "16",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i9 Processors i9-9900KF @ 3.6 GHz",
    ],
    [
        2100000000,
        4400000000.0,
        "9th Generation Intel Core i9 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i9-9900T",
        8,
        "16",
        "Desktop",
        "cpu",
        "intel",
        " 9th Generation Intel Core i9 Processors i9-9900T @ 2.1 GHz",
    ],
    [
        3500000000,
        4400000000.0,
        "Intel Core X-series Processors",
        4,
        2018.0,
        "Skylake",
        "i9-9900X",
        10,
        "20",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-9900X @ 3.5 GHz",
    ],
    [
        3500000000,
        4400000000.0,
        "Intel Core X-series Processors",
        4,
        2018.0,
        "Skylake",
        "i9-9920X",
        12,
        "24",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-9920X @ 3.5 GHz",
    ],
    [
        3300000000,
        4400000000.0,
        "Intel Core X-series Processors",
        4,
        2018.0,
        "Skylake",
        "i9-9940X",
        14,
        "28",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-9940X @ 3.3 GHz",
    ],
    [
        3100000000,
        4400000000.0,
        "Intel Core X-series Processors",
        4,
        2018.0,
        "Skylake",
        "i9-9960X",
        16,
        "32",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-9960X @ 3.1 GHz",
    ],
    [
        2400000000,
        5000000000.0,
        "9th Generation Intel Core i9 Processors",
        3,
        2019.0,
        "Coffee Lake",
        "i9-9980HK",
        8,
        "16",
        "Mobile",
        "cpu",
        "intel",
        " 9th Generation Intel Core i9 Processors i9-9980HK @ 2.4 GHz",
    ],
    [
        3000000000,
        4400000000.0,
        "Intel Core X-series Processors",
        4,
        2018.0,
        "Skylake",
        "i9-9980XE",
        18,
        "36",
        "Desktop",
        "cpu",
        "intel",
        " Intel Core X-series Processors i9-9980XE @ 3.0 GHz",
    ],
    [
        1100000000,
        3400000000.0,
        "8th Generation Intel Core m Processors",
        3,
        2018.0,
        "Amber Lake Y",
        "m3-8100Y",
        2,
        "4",
        "Mobile",
        "cpu",
        "intel",
        " 8th Generation Intel Core m Processors m3-8100Y @ 1.1 GHz",
    ],
]
data_columns = [
    "base_clock",
    "boost_clock",
    "family",
    "instructions",
    "launch_date",
    "microarchitecture",
    "model",
    "ncores",
    "nthreads",
    "target_use",
    "type",
    "vendor",
    "name",
]
data_blob = [{k: v for k, v in zip(data_columns, row)} for row in data_rows]
